/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 * @content: implementation of the garbage collector.
 *
 * @author JcHartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-15: JcHartmut creation from older sources
 *
 ****************************************************************************/
//#include "CRuntimeJavalike.h"
//#include "ThreadJc.h"
#include "BlockHeapJc_internal.h"
#include "Jc/StringJc.h"
#include "Fwc/fw_LogMessage.h"
#include "Fwc/fw_Exception.h"

//#include <stdio.h>
//#include <stdlib.h>
#include <string.h>







void activateGarbageCollectorAccess_BlockHeapJc(ObjectJc* yObj)
{
  BlockHeapJc* heap = null;
  BlockHeapBlockJc* block;

  block = deduceBlockHeapBlockFromObject(yObj, &heap);
  if(heap != null)
  { block->typeOrMaxRef |= mConsideredInGarbage_Type_Object;
  }
  else
  { //do nothing, it may be alloced in startup time.
  }

}



static void flushFullReportBuffer(GarbageCollectorJc* ythis, int sizeUsed, ThCxt* _thCxt)
{ if(ythis->idxReportBuffer >= (int)(sizeof(ythis->reportBuffer) - sizeUsed))
  { sendMsg_s0_LogMessageFW(ythis->log, ythis->kIdentMsgGcWork, ythis->reportBuffer, _thCxt);
    ythis->idxReportBuffer = sprintf(ythis->reportBuffer, "...GC; ");
  }
}





static bool testBlock_GarbageCollectorJc(GarbageCollectorJc* ythis, BlockHeapBlockJc* block)
{ int maxBackref = block->typeOrMaxRef & mTypeOrMaxRef_BlockHeapJc;
  int idxBackref = 0;
  bool bInUse = false;
  STACKTRC_ENTRY("testBlock_GarbageCollectorJc");

  do
  { //test any back reference. If it is a reference from another block, mark it.
    //if it is a reference outside any blockheaps, the block is used.
    //ObjectJcREF* ref = block->head.backRefs[idxBackref];  //the backRef contains a reference to the reference to this block.
    ObjectJcREF* ref = block->backRefs->data[idxBackref];  //the backRef contains a reference to the reference to this block.
    if(ref != null)
    { //look where the reference to this block is located:
      BlockHeapJc* retHeap;
      BlockHeapBlockJc* blockWithReference = deduceBlockHeapBlockFromObject(ref, &retHeap);
      if(  retHeap != null    //the reference is in any heap.
        && (blockWithReference->typeOrMaxRef & mConsideredInGarbage_Type_Object) //the block is to be tested in GC
        )
      { /** The reference to this block is located in another or this block.
         *  It is possible that this other block is referenced from any other or not.
         *  The blockWithReference builds a cluster with all the other blocks with references between it.
         *  If any block of the cluster have a reference outside, the blocks are used.
         *  But if no reference from outside is found, all of the blocks of the cluster are freed.
         */
        if( blockWithReference->nextBlock == null) //and the block isnot already observed by GC
        { blockWithReference->nextBlock = block->nextBlock;
          block->nextBlock = blockWithReference;
          { //report:
            flushFullReportBuffer(ythis, 32, _thCxt);
            ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, " +%p ", blockWithReference);
          }
        }
        else
        { /**The block is already the queue and part of the cluster.
           * It will be tested later.
           */
        }
        flushFullReportBuffer(ythis, 32, _thCxt);
        ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, " in %p(%p), ", ref, retHeap);
      }
      else
      { /**A reference outside a GC-controlled block indicates, that the block is referenced, in use.
         * Therefore all blocks of the cluster are used.
         */
        bInUse = true;
        flushFullReportBuffer(ythis, 32, _thCxt);
        ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, " out=%p, ", ref);
      }
    }
  }while(!bInUse && ++idxBackref < maxBackref);
  STACKTRC_LEAVE; return bInUse;
}






/**This method starts with one block and collects and tests all blocks, there have back references to the first block
 * or to a always tested block of the cluster. If no reference from outside is found to any block of the cluster,
 * the method returns false. All tested blocks are marked started with the first block with a chain of blocks,
 * using the nextBlock-association of the BlockHeapBlockJc-struct.
 * Therefore started from the first block the cluster is known from outside.
 *
 * @param block The first block to test.
 * @return true if a outside reference is found.
 */
static bool testBlockCluster_GarbageCollectorJc(GarbageCollectorJc* ythis, BlockHeapBlockJc* block)
{
  BlockHeapBlockJc* firstBlock = block;
  bool bInUse = false;
  STACKTRC_ENTRY("testBlockCluster_GarbageCollectorJc");
  block->nextBlock = kLastBlockInGarbageCtest_BlockHeapJc;  //mark it with !=null
  { //report:
    if(ythis->idxReportBuffer >0)
    { sendMsg_s0_LogMessageFW(ythis->log, 1001, ythis->reportBuffer, _thCxt);
    }
    ythis->idxReportBuffer = sprintf(ythis->reportBuffer, "GC-test; block=%p; ref:", block);
  }
  do
  { /* test all blocks of a cluster which are referencing one to another,
     * pick up all this blocks and test wether a from outside is found to a block of this cluster
     */
    ythis->countStep +=1;
    bInUse = testBlock_GarbageCollectorJc(ythis, block);
    if(ythis->testMethod != null)
    { ythis->testMethod(ythis->countStep);
    }
    //either all references are tested, and no external reference is found, or a external reference is found.
    if(!bInUse)
    { /**no exterenal reference is found, but the cluster of blocks may increased, test the next:
       */
      block = block->nextBlock;
    }
  }while(  !bInUse                                        //break if any block is used, than the cluster is used.
        && block != kLastBlockInGarbageCtest_BlockHeapJc  //break if all blocks of the cluster are tested.
        && !theGarbageCollectorJc.bAbortTest              //break if any block in cluster is reused.
        );
  //
  if(theGarbageCollectorJc.bAbortTest)
  { flushFullReportBuffer(ythis, 32, _thCxt);
    ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, " -abort because reuse of any block");
    bInUse = true;
  }
  //
  if(bInUse)
  { /* At least one block of this cluster of blocks is used from outside,
     * Than all blocks of the cluster are used, because the others are used from blocks.
     * Or the test is aborted because a block of the cluster is resused in any way. The test have to repeat than.
     * Thad do not delete it.
     * But the queue of blocks should be disintegrated.
     */
    do
    { //remove the chain of tested blocks from the blocks to prevent disturbance of a later check.
      block = firstBlock;
      firstBlock = block->nextBlock;
      block->nextBlock = null;
    } while(firstBlock != kLastBlockInGarbageCtest_BlockHeapJc);
    theGarbageCollectorJc.bAbortTest = false;
  }
  else
  { //all blocks were tested, they are not referenced from outside.
    do
    { BlockHeapJc* blockHeap;
      block = firstBlock;      //process all blocks of the cluster.
      deduceBlockHeapBlockFromObject(block, &blockHeap);  //to get the heap control structure.
      firstBlock = block->nextBlock;
      if(block->obj != null)
      { Mtbl_ObjectJc const* mtbl;
        CALLINE; mtbl = (Mtbl_ObjectJc const*)getMtbl_ObjectJc(block->obj, null);
        //mtbl may be null if no reflection of obj is given.
        if(mtbl != null)
        { mtbl->finalize(block->obj, _thCxt);
        }
      }
      //call finalize
      memset(block, 0, blockHeap->bytesNormalBlock);
      block->typeOrMaxRef = kFree_Type_BlockHeapBlock;
      { //report
        flushFullReportBuffer(ythis, 32, _thCxt);
        ythis->idxReportBuffer += sprintf(ythis->reportBuffer + ythis->idxReportBuffer, " => freed=%p", block);
      }
      blockHeap->lastFreeBlock->nextBlock = block;
      blockHeap->lastFreeBlock = block;
    } while(firstBlock != kLastBlockInGarbageCtest_BlockHeapJc);
  }/*while: test all references from outside to all blocks of this cluster*/
  //
  STACKTRC_LEAVE; return bInUse;
}








boolean garbageCollection__GarbageCollectorJc(GarbageCollectorJc* ythis)
{

  BlockHeapBlockJc* block;
  bool bInUse = false;

  STACKTRC_ENTRY("garbageCollection_BlockHeapJc");

  if(ythis->testedHeap == null)
  { ythis->testedHeap = theBlockHeapList;
    ythis->log = ythis->testedHeap->log;
    ythis->kIdentMsgFree = ythis->testedHeap->kIdentMsgFree;
    ythis->kIdentMsgGcWork = ythis->testedHeap->kIdentMsgGcWork;
    ythis->idxBlock = -1;
  }  //load on first run.

  do
  { //this loop searches the next used block.
    if(++ythis->idxBlock >= ythis->testedHeap->zBlock)
    { //evaluate the next heap
      ythis->idxBlock = 0;
      ythis->testedHeap = ythis->testedHeap->nextHeap;  //null at end
      if(ythis->testedHeap != null)
      { ythis->log = ythis->testedHeap->log;
        ythis->kIdentMsgFree = ythis->testedHeap->kIdentMsgFree;
        ythis->kIdentMsgGcWork = ythis->testedHeap->kIdentMsgGcWork;
      }
    }
    block = ythis->testedHeap == null ? null : (BlockHeapBlockJc*)( addOffset_MemAreaC(ythis->testedHeap->heapBegin, ythis->idxBlock * ythis->testedHeap->bytesNormalBlock));

  } while(  block != null          //break if all heaps are pass through.
         && ( (block->typeOrMaxRef & mTypeOrMaxRef_BlockHeapJc) == kFree_Type_BlockHeapBlock  //break if a used block is found.
            ||(block->typeOrMaxRef & mConsideredInGarbage_Type_Object) == 0  //loop if block is not considered by GC
            )
         );

  if(block != null)
  { //a used block is found.
    bInUse = testBlockCluster_GarbageCollectorJc(ythis, block);
    if(!bInUse)
    { //all blocks were tested, they are not referenced from outside.
      sendMsg_s0_LogMessageFW(ythis->log, ythis->kIdentMsgFree, ythis->reportBuffer, _thCxt);
    }/*while: test all references from outside to all blocks of this cluster*/
    else
    {
      sendMsg_s0_LogMessageFW(ythis->log, ythis->kIdentMsgGcWork, ythis->reportBuffer, _thCxt);
    }
  }/*if a used block is found. */
  STACKTRC_LEAVE;
  return !bInUse;  //true if a block is freed.
}



/**A static instance of the singleton garbage collector class. */
GarbageCollectorJc theGarbageCollectorJc = {0};


boolean garbageCollection_BlockHeapJc()
{
  /**A buffer to write reports running GC. */
  bool bFreed;
  theGarbageCollectorJc.idxReportBuffer = 0;
  bFreed = garbageCollection__GarbageCollectorJc(&theGarbageCollectorJc);
  return bFreed;
}


void setTestMethod(MT_int_Method_int testMethod)
{ theGarbageCollectorJc.testMethod = testMethod;
}


StringJc report_BheapJc(BlockHeapJc* ythis, int* idxBlockP, StringBufferJc* buffer)
{ StringJc ret;
  int idxBlock = *idxBlockP;
  bool bSearch = true;
  STACKTRC_ENTRY("report_BheapJc");
  if(idxBlock < 0) THROW_s0(IndexOutOfBoundsException,"",idxBlock);
  while(bSearch)
  { if(idxBlock >= ythis->zBlock)
    { ret = null_StringJc; idxBlock = -1; bSearch = false;
    }
    else
    { BlockHeapBlockJc* block = (BlockHeapBlockJc*)(&ythis->heapBegin[idxBlock]);
      if( (block->typeOrMaxRef & mTypeOrMaxRef_BlockHeapJc) != kFree_Type_BlockHeapBlock)
      { //found a used block, report it:
        setLength_StringBufferJc(buffer, 0, _thCxt);

      }
      else
      { idxBlock +=1;
      }
    }
  }

  STACKTRC_LEAVE; return ret;

}





