/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 */
#ifndef __fw_Formatter_h__
#define __fw_Formatter_h__

#include <stdarg.h>
#include "OSAL/inc/os_time.h"

/*@CLASS_C ParseResultPrintfStyle_fwFormatter @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**Element of a array of parse results for printf style.
 * It is a union. The common member is specifier. 
 * The element references either a text (specifier == 't') or a argument access.
 */
typedef union ParseResultPrintfStyle_fwFormatter_t
{
  /**If it is a String. */
  struct
  { /**Type of entry. 
     * t if text is present, do not use an argument value.
    */
    int8 specifier;

    /** number of chars for precesion etc, the second number. */
    int8 dummy;
  
    int16 nrofChars;
    char const* text;
    //}value
  } text;
  struct
  { /**Type of entry. 
     * * s const char*
     * * d integer
     */
    int8 specifier;

    /**Type of the argument. It determines the number of bytes of value in stack.
     * For argument values: BSIJFD for int8, int16, int32, int64, float, double. ZCz for bool, char, char*. 
     * * C a char
     * * B a byte
     * * S int16
     * * I int32 
     * * J int64
     * * P pointer of 32 bit
     * * % the percent char
     * * F float, 32 bit
     * * D double, 64 bit
     * * ? unknown
     * * N 
     */
    int8 argument;
    
    /**Nr of chars of representation. */
    int8 width;
    /**second number of chars for representation. */
    int8 precision;
  } value;
  
  
}ParseResultPrintfStyle_fwFormatter;


/**A string literal which designates a variable argument list with information, it contains a type string.
 * This is the first value in variable arguments than.
 */
extern char const typedVaArg_VaArgBuffer[];


METHOD_C int parsePrinfStyleString_fwFormatter(ParseResultPrintfStyle_fwFormatter* dst, int maxDst, char const* src, int zSrc);

METHOD_C int toString_Integer_FW(char* buffer, int zBuffer, int32 value, int radix, int minNrofChars, struct ThreadContextFW_t* _thCxt);

METHOD_C int format_va_arg_Formatter_FW(struct ThreadContextFW_t* _thCxt, const char* sText, int zText, char* buffer, int zBuffer, va_list vargList);


/**This method convers the timestamp to a readable format. It is enhanced respectively java.lang.Date, to simplify using.
 * The format characters are the same as java.text.SimpleDateFormat, but not all from there:
 * * yyyy Year with 4 digits like 1970 or 2007
 * * yy   Year with 2 digits like 70 or 07
 * * MMMM   Month in english full word like January, July, December
 * * MMM    Month in english short word like Jan, Jul, Dec
 * * M, MM  Month with 2 digits like 01, 07, 12
 * * D..DDD Day in the year from 1 o 366 using at least the number of D letters as digits, leading 0.
 * * d..dd  Day in the month from 1 o 31 using at least the number of D letters as digits, leading 0.
 * * F      Day of week from 1 o 7.
 * * EEEE   Day in Week
 * * EEE    Day in Week Mon, Tue
 * * a      Am/pm marker
 * * HH     Hour in Day 0..23
 * * kk     Hour in Day 1..24
 * * hh      Hour in am/pm 0..11
 * * kk      Hour in am/pm 1..12
 * * mm     Minute in hour
 * * ss     Second in minute
 * * SS     10 Millisecond in second.
 * * SSS    Millisecond in second.
 *
 * @param buffer The buffer in which the data string is written in. The buffer is not initialized. 
 *               Only the returned nr of chars are written, other content isn't changed.
 * @param sizeBuffer max. Number of chars which will be written at buffer. 
 * If the date string is longer as this param, the rest is not written. (No Exception).
 * @param time The time 
 * @param format String to determine the positions of the parts of readable date.
 * @param timeZoneAdjustHours Anzahl der Stunden fr die Zeitzonenverschiebung.
                              Es wird davon ausgegangen, dass intern nach Greenwich Mean Time gespeichert ist.
              Die Angabe von 1 bedeutet eine Verschiebung in Richtung MEZ (Winter).
 * @return nr of chars written.
 */
int toStringFormat_Fw(char* buffer, int sizeBuffer, OS_TimeStamp const* time, char const* sFormat, int timeZoneAdjustHours);


#endif //__fw_Formatter_h__
