/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content This source contains routines to implement the LogMessage interface at C-level.
 * The LogMessageFW_i interface is an abstract C-like class. The not-abstract C-routines are contained here.
 * The iLogMessageFW interface is the C++ incarnation of the same interface, its non-abstract routines are implemented here.
 * Additional some helper routines for the MsgDispatcher_MSG are found here.       
 *
 * @author Hartmut Schorrig
 * @version sf-0.83
 * list of changes:
 * 2008-04-22: Hartmut creation as poor implementation
 * 2008-12-01: Hartmut implementation of the LogMessage-Concept
 *
 ************************************************************************************************/

#include "Fwc/fw_LogMessage.h"
#include "Jc/StringJc.h"
//#include "VaArgBuffer.h"
#include "Fwc/fw_Formatter.h"
#include "OSAL/inc/os_time.h"
#include "Fwc/fw_Exception.h"
#include <stdarg.h>

extern ClassJc const reflection_LogMessageFW_i;

ClassJc const reflection_LogMessageFWxxx =
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &reflection_LogMessageFWxxx, null)
, "reflection_LogMessageFW"
, 0     //posObjectJc
, sizeof(reflection_LogMessageFWxxx)
, null  //attributes and associations
, null  //method
, null  //superclass
, null  //interfaces
, 0x0000000  //modifiers
, null  //No Mtbl, because it isn't instantiable
};


const char identText_LogMessageFW[] = "LogMessage_FW";

void const* const sign_Mtbl_LogMessageFW = &identText_LogMessageFW;

void sendMsgVaList_LogMessageFW(struct LogMessageFW_t* ythis, int32 identNumber
, OS_TimeStamp timestamp
, StringJc text
, va_list args
//, struct VaArgBuffer_t* args
, ThCxt* _thCxt)
{
  //this method checks the correctness of the instance data
  //and throws an exception if there is a memory overwritten error or initialization error:
  //The casting is deterministic because the type MyStruct is tested.
  Mtbl_LogMessageFW* mtbl;
  STACKTRC_TENTRY("sendMsgVaList_LogMessageFW");
  mtbl = (Mtbl_LogMessageFW*)getMtbl_ObjectJc(&ythis->object, &reflection_LogMessageFW_i);
  if(mtbl == null) THROW_s0(IllegalArgumentException, "Method-table not found",0);
  if(mtbl->sign != &sign_Mtbl_LogMessageFW) THROW_s0(IllegalArgumentException, "The found Method-table is not correct", (int)mtbl);
  mtbl->sendMsgVaList(ythis,identNumber, timestamp, text, args, _thCxt);
  STACKTRC_LEAVE;

}



void sendMsg_iSv_LogMessageFW(struct LogMessageFW_t* ythis
, int identNumber
, StringJc const text
, ...
)
{ //represent the variable arguments as a reference to the VaArgBuffer-type, the instance is located in stack.
  //VaArgBuffer* args = (VaArgBuffer*)(addOffset_MemAreaC(&text, sizeof(text)));
  va_list args;
  STACKTRC_ENTRY("sendMsg_iSv_LogMessageFW");
  va_start(args, text);
  sendMsgVaList_LogMessageFW(ythis, identNumber, os_getDateTime(), text, args, _thCxt);
  STACKTRC_LEAVE;
}



void sendMsg_tsv_LogMessageFW(struct LogMessageFW_t* ythis
, int identNumber
, OS_TimeStamp time
, char const* text
, ...
)
{ //represent the variable arguments as a reference to the VaArgBuffer-type, the instance is located in stack.
  //VaArgBuffer* args = (VaArgBuffer*)(addOffset_MemAreaC(&text, sizeof(text)));
  va_list args;
  STACKTRC_ENTRY("sendMsg_iSv_LogMessageFW");
  va_start(args, text);
  sendMsgVaList_LogMessageFW(ythis, identNumber, time, s0_StringJc(text), args, _thCxt);
  STACKTRC_LEAVE;
}




void sendMsg_s0_LogMessageFW(struct LogMessageFW_t* ythis, int identNumber, const char* text, ThCxt* _thCxt)
{
  sendMsg_iSv_LogMessageFW(ythis, identNumber, s0_StringJc(text));
}

void write_S_LogMessageFW(struct LogMessageFW_t* ythis, int identNumber, StringJc text, ThCxt* _thCxt)
{
  sendMsg_iSv_LogMessageFW(ythis, identNumber, text);
}








/*@CLASS_C MsgDispatcher_MSG ************************************************************************
 *Additional methods, there are not able to translate java2c
 */

int _prepareOutputText_MsgDispatcher_MSG(char* buffer, int zBuffer, int ident, StringJc text, OS_TimeStruct time
, va_list args
//, struct VaArgBuffer_t const* const args
//, MsgDispatcher__Entry__Values* args
)
{
  int nrofChars;
  int zText;
  char const* sText = getCharsAndLength_StringJc(&text, &zText);
  //const char* text1 = text.ref; 
  //char const* arg1 = *(char const**)(&args->v[0]);
  STACKTRC_ENTRY("_prepareOutputText_MsgDispatcher_MSG");
  //if( true )//arg1 == typedVaArg_ObjectJc)
  { nrofChars = format_va_arg_Formatter_FW(_thCxt, sText, zText, buffer, zBuffer, args);
  }
  //else
  { //This routine is not available in all systems, especially not in Rmos.
    //nrofChars = _snprintf(buffer, zBuffer, sText, *args);  //11*4 bytes per value.
    //if(nrofChars > zBuffer)
    { 
    }
  }
  STACKTRC_LEAVE; return nrofChars;
}

