/****************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2006-05-00: www.vishia.de creation
 *
 * @content Definition of a interface for Logging and Messaging.
 *
 ****************************************************************************/
#ifndef __fw_LogMessage_h__
#define __fw_LogMessage_h__

//#include "fw_simpleC.h"
#include "Fwc/objectBaseC.h"   //ObjectJc used.
#include "Fwc/fw_Exception.h"  //Thcxt used.
#include "OSAL/inc/os_time.h"  //struct OS_TimeStamp as argument
#include <stdarg.h>


/* @ CLASS_C LogMessageFW @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**Interface for C-language.
 * This struct should used ones as marker struct for the interface. 
 */
typedef struct  LogMessageFW_t
{ 
  /**The implementation classes based on ObjectJc. Necessary for check.*/
  ObjectJc object;
}LogMessageFW_i;  //_i means an struct useable as interface


//REFTYPEDEFJc(LogMessageFW)


/**Level of outputs.
 */
typedef enum ELevel_Message_ifcFW_t
{ kError_LogMessageFW = 1
, kWarning_LogMessageFW = 2
, kInfo_LogMessageFW = 3
, kFineInfo_LogMessageFW = 4
, kDebug_LogMessageFW = 4
, kFineDebug_LogMessageFW = 6
}eLevel_Message_ifcFW;



/**Sends a message to a log system.
 * @param identNumber The identnumber idents the source and kind of message.
                      It is possible to select the destination of message output by ident numbers.
                      It is also possible to select output texts in foreign languages on output system by ident number.
                      It is possible to mark "comming" and "going" from a message, using the positiv and negativ value of the number.
                      The positiv and negativ value should present the same identNumber.
 * @param text The default, immediate output text
 */
void sendMsg_s0_LogMessageFW(struct LogMessageFW_t* ythis, int identNumber, const char* text, ThCxt* _thCxt);

//void sendMsg_s_LogMessageFW(struct LogMessageFW_t* ythis, int identNumber, StringJc text, ThCxt* _thCxt);


void write_S_LogMessageFW(struct LogMessageFW_t* ythis, int identNumber, StringJc text, ThCxt* _thCxt);

#define sendMsg_s_LogMessageFW(YTHIS, NR, TEXT,THC) write_S_LogMessageFW (YTHIS, NR, TEXT,THC)

void sendMsg_LogMessage_FW(struct LogMessageFW_t* ythis, int identNumber, const char* text, ...);


METHOD_C void sendMsg_iSv_LogMessageFW(struct LogMessageFW_t* ythis
, int identNumber
, StringJc const text
, ...
);


/**Sends a message with a variable argument list. The text should match to the type of arguments.
 */
void sendMsg_tsv_LogMessageFW(struct LogMessageFW_t* ythis
, int identNumber
, OS_TimeStamp time
, char const* text
, ...
);


/**Sends a message with given arguments in a variable argument list using C-standard <stdarg.h>.
 * The text should match to the type of arguments.
 * This is the commonly kind of calling routine to send a message. Only this type of routine 
 * should be implemented by any implementation of Log output. See [[MT_sendMsgVaList_LogMessageFW]]
 */
METHOD_C void sendMsgVaList_LogMessageFW(struct LogMessageFW_t* ythis
, int32 identNumber
, OS_TimeStamp timestamp
, StringJc text
, va_list args
, ThCxt* _thCxt
);



/**Sends a message with given timestamp and parameter.
 * This is the core routine of LogMessage interface.
 */
typedef void MT_sendMsgVaList_LogMessageFW
( struct LogMessageFW_t* ithis
, int32 identNumber
, OS_TimeStamp timestamp
, StringJc text
, va_list args
//, struct VaArgBuffer_t const* const args
, ThCxt* _thCxt
);

/**const struct of method addresses. A constant instance of this struct should be a part of the Reflection-MethodTable
 * of implementation by any Log output.
 */
typedef struct Mtbl_LogMessageFW_t
{ 
  /**The value of sign must be identically with the address of the sign_Mtbl_Message_ifcFW-variable. Use it for safe access. */
  void const* const* sign;
  
  /**Only this method should implemented. */
  MT_sendMsgVaList_LogMessageFW* sendMsgVaList;
  
}Mtbl_LogMessageFW;

/**This is a marker text. The content of text isn't important, but all implementations should use the same address
 * (value of field sign in Mtbl_LogMessageFW).
 */
extern void const* const sign_Mtbl_LogMessageFW;

/* *******************************************************************************************************************/
/**C++-usage: */

#if defined(__CPLUSGEN) && defined(__cplusplus)
class iLogMessageFW
{

  //public: int sendMsg(int32 ident, char const* text);
  
  //public: virtual int hexDump(int ident, void* pBuffer, int length, int bytesPerWord)=0;
    
  public: int sendMsg(int ident, const char* text, ...)
  { va_list args;
    va_start(args, text);
    LogMessageFW_i* ithis = provide_LogMessageFW_i();
    sendMsgVaList_LogMessageFW(ithis, ident, os_getDateTime(), s0_StringJc(text), args, null);  
    return 0;
  }

  /**The same as sendMsg, only for compatibility. */
  public: int writeProt(int ident, const char* text, ...)
  { va_list args;
    va_start(args, text);
    LogMessageFW_i* ithis = provide_LogMessageFW_i();
    sendMsgVaList_LogMessageFW(ithis, ident, os_getDateTime(), s0_StringJc(text), args, null); 
  }
  
  
  //public: virtual int writeState(int ident, bool state, const char* string, ...)=0; 

  public: void sendMsgVaList(int32 ident, OS_TimeStamp timestamp, const char* text, va_list args)
  { LogMessageFW_i* ithis = provide_LogMessageFW_i();
    sendMsgVaList_LogMessageFW(ithis, ident, timestamp, s0_StringJc(text), args, null); 
  } 

  public: virtual LogMessageFW_i* provide_LogMessageFW_i()=0; 
};



#endif  //__CPLUSGEN

/*@DEFINE standard implementations @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/**Initializes a file output.
 * @param othis reference to the instance, the size should be known in the ObjectJc-base. ObjectJc should be initialized already.
 *        If this parameter is null, an instance will be created in heap.
 */
METHOD_C LogMessageFW_i* ctorO_LogMessageConsole(ObjectJc* othis);

/**Initializes a file output.
 * @param othis reference to the instance, the size should be known in the ObjectJc-base. ObjectJc should be initialized already.
 *        If this parameter is null, an instance will be created in heap.
 * @param sFileName: If it contains a part between $...$, it is the pattern for timestamp, see java.text.SimpleDateFormat.
 * @param nrofSecondsToFlush: A flush of content isn't written immediate, so the file is not visible outside. 
 *        This parameter defines, after how many seconds of a first not flushed write the file will be flushed.
 *        Note: A flush won't be done automatically, only if a new message is written. Reason: No extra thread is used.
 *        The value 0 means: flush any message immediately.
 * @param nrofHoursPerFile: After this time a new file will be created 
 */
METHOD_C LogMessageFW_i* ctorO_LogMessageFile(ObjectJc* othis
, StringJc sFileName
, int nrofSecondsToFlush
, int nrofHoursPerFile
);


LogMessageFW_i* create_LogMessageConsole(ThCxt* _thCxt);




/*@CLASS_C MsgDispatcher_MSG ************************************************************************
 *Additional methods, there are not able to translate java2c
 */

/**The MsgDispatcher is defined and implemented in another files. But here an additional access routine is defined,
 */
struct MsgDispatcher_MSG_t;
LogMessageFW_i* convertFromMsgDispatcher_LogMessageFW_i(struct MsgDispatcher_MSG_t* src, ThCxt* _thCxt);

int _prepareOutputText_MsgDispatcher_MSG
(char* buffer, int zBuffer, int ident, StringJc text, OS_TimeStruct time
, va_list  args);
//, struct VaArgBuffer_t const* const  args);



#endif /*__LogMessageFW_h__*/
