/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content definition of A StringJc-struct, which contains the character pointer and its length,
 *   and basicly access methods.
 *   The struct defined well to return per value only in registers.
 *   Therefore an adaption to several compilers and platforms is able to done if necesarry.
 * @author Hartmut Schorrig
 * @created 2008-12-07, from other sources (Objectjc.h), the concept is from 2004
 * @version 0.83
 *************************************************************************************************/
#ifndef __fw_String_h__
#define __fw_String_h__
#include "os_types_def.h"

/*@DEFINE_C some_forward_types @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

struct MemAreaC_t;
/*@CLASS_C StringJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**Definition of the struct StringJc, which contains a string reference, length and control bits.
 * The struct ,,StringJc,, contains the reference to the string and the number of chars.
 * The string itself is a sequence of chars (8-bit-ASCII), present as ,,char[],,,
 * either as constant literal.
 * or data inside a struct StringBufferJc.
 * or a String somewhere else.
 * It may be also a part of the string in a [[StringBufferJc]].
 *
 * This struct is defined not with high level language aspects, but in a cpu-system-opportune manner:
 * The struct should match in registers especially as return by value. 
 * Therefore the content may build with simple int. 
 * Some macros in following text define the access.
 * This structure has to be adapt if necessary with a other or special cpu architecture.
 */
typedef struct  xxxStringJc_t
{ /**The followed attribute is at same position like offsetBackPointer in a Enhanced Reference. See [[ObjectRefJc]].
   * That is because a StringJc is used in the same manner as a enhanced reference.
   * But this value stores also the length of the string.
   * The bits mask with ,,mIdxMtbl_ObjectJc,, in a reference are used for the length information and a ''buffer''-Bit.
   * In the ,,CRuntimeJavalike_SysConventions.h,, 2 definitions should be present for this bits,
   * * ,,mLength__StringJc,,: The length-bits, typical 0x7FFF for up to 32000 chars.
   * * ,,mBuffered__StringJc,, The ''buffered'' Bit. If this Bit is set,
   *   the StringJc referenced the whole string of a StringBufferJc to concat strings.
   * If all bits of the mask ,,mLength__StringJc,, are setted, ref pointeres a zero-terminated String.
   * The length of the string will be calculated on first process with the strlen()-C-funtion.
   *
   * The bits mask with ,,mBackRef_ObjectJc,, have the same meaning like on other enhanced references:
   * If a Block Heap with Gargabe Collection is used, it is the index of the backward reference of a referenced StringBufferJc.
   *
   * @sizeof=4 This attribute is defined in the user modificable CRuntimeJavalike_Sysconventions.h
   *           and should be have the same size like a pointer or int
   */
  int32 refbase;  //NOTE: here adpated to 32 bit architecture, it is possible to use a int16 
  
  /**This value is commonly a reference, if it is in the same manner as an enhanced reference.
   * But it should match to a cpu register. Therefore also a representation as int may be correct. test both.
   * A representation as a char* supports viewing in debugging.
   */
  //int ref;
  char const* ref;      //NOTE: should match to 1 cpu register.
}xxxStringJc;

#define StringJc OS_ValuePtr

#define TYPE_OS_ValuePtr char*

#define reflection_StringJc reflection_OS_ValuePtr


/**Returns the length bytes of the StringJc. */
//#define _LENGTH_R2_StringJc(REF) ((REF).refbase & mLength__StringJc)
#define _LENGTH_R2_StringJc(REF) (value_OS_ValuePtr(REF) & mLength__StringJc)



/**Sets the length bytes of the StringJc. */
//#define _SETLENGTH_R2_StringJc(REF, LEN) ((REF).refbase = ((REF).refbase & ~mLength__StringJc) | (LEN & mLength__StringJc))
#define _SETLENGTH_R2_StringJc(REF, LEN) setValue_OS_ValuePtr(REF, (value_OS_ValuePtr(REF) & ~mLength__StringJc) | ((LEN) & mLength__StringJc))

/**Sets the buffered bit. */
//#define _SETBUFFERED_R2_StringJc(REF) ((REF).refbase |= mBuffered__StringJc)
#define _SETBUFFERED_R2_StringJc(REF) setValue_OS_ValuePtr(REF, value_OS_ValuePtr(REF) | mBuffered__StringJc)

/**Returns the pointer to the char* */
//#define _STRING_R2_StringJc(REF) ((char*)(REF).ref)
#define _STRING_R2_StringJc(REF) PTR_OS_ValuePtr(REF, char const)

/**Sets the pointer to the char* */
//#define _SETSTRING_R2_StringJc(REF, STR) ((REF).ref = /*(int)*/(STR))
#define _SETSTRING_R2_StringJc(REF, STR) ((REF).ptr__ = (TYPE_OS_ValuePtr)(STR))


#define NULL_StringJc { 0}

/**Constant StringJc references initialize data to a string with known length.
 */
#define CONST_s0_i_StringJc(TEXT, LEN) { LEN , (TYPE_OS_ValuePtr)TEXT}
#define CONST_s0_i_StringJc(TEXT, LEN) { LEN , (TYPE_OS_ValuePtr)TEXT}

/**Constant StringJc references initialize data to a string with known length.
 */
#define CONST_i_StringJc(TEXT, LEN) { LEN , (TYPE_OS_ValuePtr)TEXT}
#define CONST_i_StringJc(TEXT, LEN) { LEN , (TYPE_OS_ValuePtr)TEXT}

/** constant StringJc references a zero-terminated string. The length of the string
    is not stored inside StringJc, the length bits are setted to there maximum
    to detect this constellation.
  */
#define CONST_s0_StringJc(TEXT) { mLength__StringJc, (TYPE_OS_ValuePtr)TEXT}
#define CONST_s0_StringJc(TEXT) { mLength__StringJc, (TYPE_OS_ValuePtr)TEXT}


/**Constant StringJc references initialize data to a zero-terminated string.
 * The length of the string
 * is not stored inside StringJc, the length bits are setted to there maximum
 * to detect this constellation.
 */
#define CONST_StringJc(TEXT) { mLength__StringJc, (TYPE_OS_ValuePtr)TEXT}
#define CONST_StringJc(TEXT) { mLength__StringJc, (TYPE_OS_ValuePtr)TEXT}

/** Initialize a StringJc instance with the reference to the given constant zeroterminated string.
  * The up till now value inside YTHIS is not respected. This method is usefull to initialize new instances
  * espacially inside the stack.
  */
#define init_s0i_StringJc(YTHIS, src, nrofChars) set_OS_ValuePtr(YTHIS, (nrofChars), (src))
#define ctor_s0i_StringJc(YTHIS, src, nrofChars) init_s0i_StringJc(YTHIS, src, nrofChars)
//#define ctor_s0i_StringJc(YTHIS, src, nrofChars) { (YTHIS)->ref = /*(int)*/src; (YTHIS)->refbase = nrofChars; }
//#define init_s0i_StringJc(YTHIS, src, nrofChars) { (YTHIS)->ref = /*(int)*/src; (YTHIS)->refbase = nrofChars; }


/** The definition of the StringJc-structure is done in ObjectJc.h,
  * because it is necessary as output of toString_ObjectJc.
  */
/** Initialize a StringJc instance with the reference to the given constant zeroterminated string.
  * The up till now value inside YTHIS is not respected. This method is usefull to initialize new instances
  * espacially inside the stack.
  */
#define init_s0_StringJc(YTHIS, src) init_s0i_StringJc(YTHIS, src, strlen(src))



/** Initialize a StringJc instance with an other given StringJc.
  * The up till now value inside YTHIS is not respected. This method is usefull to initialize new instances
  * espacially inside the stack.
  */
#define init_s_StringJc(YTHIS, src) { YTHIS = src;  }



/**A pointer to StringJc structure containing null-values.
 * NOTE: a pointer is necessary because a const problem occurs otherwise.
 */
extern StringJc const null_StringJc;

/* Creates a String reference to the given zero terminated char array.
 * This is a simple way if a method requires a StringJc object as value argument but the user want to write only a constant char literal.
 * It is a fast C-routine, because the StringJc is stored in CPU-registers, no memory is need.
 * @return the String reference per value, it is hold in 2 register by most of C-compilers and therefore effective.
 */
METHOD_C StringJc s0_StringJc(const char* src);

/* Creates a String reference to the given char array with given length.
 * This is a common way to get a StringJc-Instance if a char-buffer is given.
 * It is a fast C-routine, because the StringJc is stored in CPU-registers, no memory is need.
 * The buffer is referenced in the StringJc. If the reference is stored persistent, 
 * a memory management for the buffer should be exist. 
 * The stored String may be an enhanced reference, using the BlockHeapJc-concept of Garbage Collection.
 * But the yet created StringJc-instance isn't persistent and isn't an enhanced reference therefore.
 * @param src a buffer containing the String
 * @param len the number of chars valid from buffer.
 * @return the String reference per value, it is hold in 2 register by most of C-compilers and therefore effective.
 */
METHOD_C StringJc s0i_StringJc(char const* src, int len);

/** Set this as reference to a String.
  * The string src may be a constant string, a substring or a StringBuffer.
  * If Runtimeheap with garbage collection is used, and src is a StringBuffer,
  * the back referenced to ythis will be registered in buffer.
  * If ythis was a reference before, the reference will be
  * released. If the reference is already a reference to the same buffer,
  * nothing will be done.
  * @param src The source StringJc. Note: It is given per value, not per reference,
           so direct output from StringJc-returning methods are useable as argument.
           The value consist of 2 processor registers.
  */
METHOD_C void set_StringJc(StringJc* ythis, StringJc src);

/** Returns the length
 */
METHOD_C int length_StringJc(const StringJc ythis);
//METHOD_C int length_StringJc(StringJc const* ythis);


/** Copy the content to a buffer of the user.
 * @param buffer The destination buffer
 * @param ythis The source
 * @param maxSizeBuffer The number of bytes in Buffer. If the string-length is less than length, 0-Bytes were setted to the end.
 *               If the string-length is equal or greater length, only lenght characters of string are copied.
 *               In this case no 0 are produced. the string is not zero-terminated than. It's the same like C-strncpy!
 * @return the destination itself.
*/
METHOD_C int copyToBuffer_StringJc(const StringJc ythis, char* buffer, int maxSizeBuffer);

/** Set this as reference to a String as a light copy. It means, no back reference is stored
  * if a garbage collector is used. This method should be only used, if the dst is temporary.
  * The string src may be a constant string, a substring or a StringBuffer.
  * @param src The source StringJc. Note: It is given per value, not per reference,
           so direct output from StringJc-returning methods are useable as argument.
           The value consist of 2 processor registers.
  */
#define lightCopy_StringJc(ythis, src) { *ythis = src; } //{(ythis)->s.refbase = (src).s.refbase; (ythis)->s.ref = (src).s.ref; }


/**gets the char at position idx.
* @javalike, but no test of idx is done.
*/
#define charAt_StringJc(YTHIS, idx)  _STRING_R2_StringJc(YTHIS)[idx] //( ((StringJcx*)&(YTHIS))->s.ref[idx])

/* Gets the char-pointer and the number of chars stored in a string.
 * This is a common way to get a the content of a StringJc-Instance if a char-pointer is used.
 * It is a fast C-routine, because the StringJc is stored in CPU-registers, no memory is need.
 * The char-pointer is the same reference as in the StringJc. It may be persistent or not.
 * In generally, the user shouldn't store the reference persistently outside of the StringJc-concept.
 * The routine is given to support evaluating the chars in a C-like-way.
 * @param length destination variable to store the realy length.
 * @return the pointer to the chars. It may be null if the StringJc contains no String.
 */
METHOD_C char const* getCharsAndLength_StringJc(StringJc const* ythis, int* length);


#endif //__fw_String_h__
