/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 *
 * @content Implementation of ObjectJc
 *
 * @author Hartmut Schorrig
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut adap:The elements idSemaphore and idWaitingThread are combined to idSyncHandles.
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/

#include <stdlib.h>     //malloc
#include <string.h>     //memset
#include "Jc/ObjectJc.h"  //:NOTE: after stdlib.h because malloc maybe defined.
//#include "StringJc.h"
#include "Jc/ReflectionJc.h"
#include "Fwc/fw_Exception.h"
#include "Fwc/fw_ThreadContext.h"
//#include "os_wrapper.h"

/********************************************************************************************/
//Object

bool castOk_ObjectJc;


ObjectJc* init_ObjectJc(ObjectJc* ythis, int sizeObj, int identObj)
{ memset(ythis, 0, sizeObj);
  ythis->ownAddress = ythis;
  ythis->idSyncHandles = kNoSyncHandles_ObjectJc;
  setSizeAndIdent_ObjectJc(ythis, sizeObj, identObj, 0);
  return ythis;
}


ObjectJc* ctorM_ObjectJc(MemC mem) //ObjectJc* ythis)
{ ObjectJc* ythis = PTR_MemC(mem, ObjectJc);
  init0p_MemC(ythis, sizeof(ObjectJc));  //note: ctorc don't initialze with 0.
  //obsolete: ythis->offsetToHeapBlock = offsetBlock_MemC(mem);  //NOTE 0 if no offset is given.
  ctorc_ObjectJc(ythis);
  return ythis;
}

//old version, clears ythis-area.
void ctor_Object_Jc(ObjectJc* ythis)
{
  init0p_MemC(ythis, sizeof(ObjectJc));  //note: ctorc don't initialze with 0.
  ctorc_ObjectJc(ythis);
}



void ctorc_ObjectJc(ObjectJc* ythis)
{ 
  if(ythis->ownAddress != ythis)  //if the ownAddress is already set, it is initalized!
  { ythis->ownAddress = ythis;
    ythis->idSyncHandles = kNoSyncHandles_ObjectJc;
    ythis->offsetToHeapBlock = 0;
    //
    //NOTE: the both next values cannot be setted from focus ObjectJc,
    //      they should be setted outside the ctor behind them from focus of the derivated class.
    //      use setReflection_ObjectJc(..) therefore.
    ythis->objectIdentSize = 0;
    ythis->reflectionClass = null;
  }
  else
  { //it is already initialized.
  }
}



ObjectJc* alloc_ObjectJc(const int size, const int32 typeInstanceIdent, ThCxt* _thCxt)
{ ObjectJc* othis;
  STACKTRC_TENTRY("alloc_ObjectJc");
  #ifdef USE_BlockHeapJc
  { const char* sCallingMethodName = getCallingMethodName_StacktraceThreadContext( &_thCxt->stacktraceThreadContext, 1);  
    othis = allocObject_s_sBlockHeapJc(size, typeInstanceIdent, sCallingMethodName, _thCxt);
  }
  #else
  { MemC mem;
    mem = alloc_MemC(size);
    othis = PTR_MemC(mem, ObjectJc);
    init_ObjectJc(othis, size, typeInstanceIdent);
  }
  #endif
  STACKTRC_LEAVE; return othis;
}

/*
ObjectJc* ctorO_ii_ObjectJc(ObjectJc* othis, const int size, const int32 typeInstanceIdent, ThCxt* _thCxt)
{ int32 objectIdentSize;
  checkConsistenceOrInit_ObjectJc(othis, size, null, _thCxt);
  //set the size of the Object, regarding the typsInstanceIdent limits for size:
  if( (typeInstanceIdent & mSizeBits_objectIdentSize_ObjectJc) == kIsSmallSize_objectIdentSize_ObjectJc)
  { if(size > mSizeSmall_objectIdentSize_ObjectJc)
    { THROW_s0(IndexOutOfBoundsException, "too large", size);
    }
    objectIdentSize = (typeInstanceIdent & ~mSizeSmall_objectIdentSize_ObjectJc) | size;
  }
  else if( (typeInstanceIdent & mSizeBits_objectIdentSize_ObjectJc) == kIsMediumSize_objectIdentSize_ObjectJc)
  { if(size > mSizeMedium_objectIdentSize_ObjectJc)
    { THROW_s0(IndexOutOfBoundsException, "too large", size);
    }
    objectIdentSize = (typeInstanceIdent & ~mSizeMedium_objectIdentSize_ObjectJc) | size;
  }
  else if(typeInstanceIdent & mIsLargeSize_objectIdentSize_ObjectJc)
  { if(size > mSizeLarge_objectIdentSize_ObjectJc)
    { THROW_s0(IndexOutOfBoundsException, "too large", size);
    }
    objectIdentSize = (typeInstanceIdent & ~mSizeLarge_objectIdentSize_ObjectJc) | size;
  }
  else
  { objectIdentSize = 0;
    THROW_s0(IndexOutOfBoundsException, "undefined size", typeInstanceIdent);
  }
  othis->objectIdentSize = objectIdentSize;
  STACKTRC_LEAVE; return othis;
}
*/


void checkConsistence_ObjectJc(ObjectJc* ythis, int size, struct ClassJc_t const* clazzReflection, ThCxt* _thCxt)
{ STACKTRC_TENTRY("checkConsistence_ObjectJc");
  { //it is initialized, check it
    int sizeObj = getSizeInfo_ObjectJc(ythis);
    if(ythis->ownAddress != ythis) THROW_s0(IllegalArgumentException, "faut ownAddress", (int)ythis->ownAddress);
    if(sizeObj == 0)
    { //not initialized
      sizeObj = size;
    }
    else if(sizeObj < size) THROW_s0(IllegalArgumentException, "faut size", sizeObj);
    
    if(ythis->reflectionClass == null)
    { ythis->reflectionClass = clazzReflection;
    }
    else
    { //TODO test Reflection, it can be a derived class.
    }
  }
  STACKTRC_LEAVE;
}



void checkConsistenceOrInit_ObjectJc(ObjectJc* ythis, int size, struct ClassJc_t const* clazzReflection, ThCxt* _thCxt)
{ STACKTRC_TENTRY("checkConsistence_ObjectJc");
  if(ythis->ownAddress == null)
  { //not initialized
    ctorc_ObjectJc(ythis);
    setReflection_ObjectJc(ythis, clazzReflection, 0);
    setSizeAndIdent_ObjectJc(ythis, size, 0, 0);
  }
  else
  { checkConsistence_ObjectJc(ythis, size, clazzReflection,_thCxt);
  }
  STACKTRC_LEAVE;
}



void setReflection_ObjectJc(ObjectJc* ythis, struct ClassJc_t const* reflection,  int32 typeInstanceSizeIdent)
{ ythis->reflectionClass = reflection;
  if(ythis->objectIdentSize == 0){ ythis->objectIdentSize = typeInstanceSizeIdent; }
  else
  { //TODO setIdent_ObjectJc(typeInstanceSizeIdent);
  }
}




void setSizeAndIdent_ObjectJc(ObjectJc* ythis, int sizeObj, int identObj, int nrofArrayDimensions)
{
  if(identObj < 0 || sizeObj < 0 || nrofArrayDimensions <0 || nrofArrayDimensions > 3)
  { STACKTRC_ENTRY("setSizeAndIdent_ObjectJc");
    THROW_s0(RuntimeException, "negativ values for all parameter are unexpected, or nrofArrayDimensions >0", sizeObj);
  }
  else if(sizeObj <= mSizeSmall_objectIdentSize_ObjectJc && identObj <= (mTypeSmall_objectIdentSize_ObjectJc >> kBitTypeSmall_objectIdentSize_ObjectJc))
  { ythis->objectIdentSize = sizeObj | kIsSmallSize_objectIdentSize_ObjectJc | (identObj<<kBitTypeSmall_objectIdentSize_ObjectJc) | (nrofArrayDimensions << kBitArray_objectIdentSize_ObjectJc);
  }
  else if(sizeObj <= mSizeMedium_objectIdentSize_ObjectJc && identObj <= (mTypeMedium_objectIdentSize_ObjectJc >> kBitTypeMedium_objectIdentSize_ObjectJc))
  { ythis->objectIdentSize = sizeObj | kIsMediumSize_objectIdentSize_ObjectJc | (identObj<<kBitTypeMedium_objectIdentSize_ObjectJc) | (nrofArrayDimensions << kBitArray_objectIdentSize_ObjectJc);
  }
  else if(sizeObj <= mSizeLarge_objectIdentSize_ObjectJc && identObj <= (mTypeLarge_objectIdentSize_ObjectJc >> kBitTypeLarge_objectIdentSize_ObjectJc))
  { ythis->objectIdentSize = sizeObj | mIsLargeSize_objectIdentSize_ObjectJc | (identObj<<kBitTypeLarge_objectIdentSize_ObjectJc) | (nrofArrayDimensions << kBitArray_objectIdentSize_ObjectJc);
  }
  else
  { STACKTRC_ENTRY("setSizeAndIdent_ObjectJc");
    THROW_s0(RuntimeException, "not matched ident and size", sizeObj);
  }
}




int getSizeInfo_ObjectJc(ObjectJc const* ythis)
{ if(ythis->objectIdentSize & mIsLargeSize_objectIdentSize_ObjectJc) 
    return ythis->objectIdentSize & mSizeLarge_objectIdentSize_ObjectJc;
  else if( (ythis->objectIdentSize & mSizeBits_objectIdentSize_ObjectJc) == kIsMediumSize_objectIdentSize_ObjectJc)
    return ythis->objectIdentSize & mSizeMedium_objectIdentSize_ObjectJc;
  else
    return ythis->objectIdentSize & mSizeSmall_objectIdentSize_ObjectJc;
}

int getIdentInfo_ObjectJc(ObjectJc const* ythis)
{ if(ythis->objectIdentSize & mIsLargeSize_objectIdentSize_ObjectJc) 
    return (ythis->objectIdentSize & mTypeLarge_objectIdentSize_ObjectJc) >>kBitTypeLarge_objectIdentSize_ObjectJc;
  else if( (ythis->objectIdentSize & mSizeBits_objectIdentSize_ObjectJc) == kIsMediumSize_objectIdentSize_ObjectJc)
    return (ythis->objectIdentSize & mTypeMedium_objectIdentSize_ObjectJc) >>kBitTypeMedium_objectIdentSize_ObjectJc;
  else
    return (ythis->objectIdentSize & mTypeSmall_objectIdentSize_ObjectJc) >>kBitTypeSmall_objectIdentSize_ObjectJc;
}






struct ClassJc_t const* getClass_ObjectJc(ObjectJc const* ythis)
{ return ythis->reflectionClass;//->theClass;
}


ObjectJc* clone_Object_F(ObjectJc const* ythis, MemC buffer)
{
  return null;
}

bool equals_Object_F(ObjectJc const* ythis, ObjectJc const* cmp)
{
  return false;
}

void finalize_Object_F(ObjectJc* ythis, ThCxt* _thCxt)
{
}

int32 hashCode_Object_F(ObjectJc const* ythis)
{
  return 0;
}


/************************************************************************/



/**calculates the index insided the jumptable of a dynamic call,
 * regarding the type of reference and the type of the instance.
 * @param reflectionObj The reflection of the referenced instance.
 * @param reflectionRef The reflection of the reference. It should be the same as reflectionObj
 *        or it should be found as a superclass or interface of the reflectionObj.
 *        This param is only used with its pointer value, no access to the referenced memory location will be done.
 * @return The index of the part of jumptable of the reference inside the jump table of the object.
 *         * It is 0, if reflectionObj == reflectionRef, it means the reference is from the same type as the Object.
 *         * It is mIdxMtbl_ObjectJc if reflectionObj is null. This case is possible if the Object has no reflection infos.
 *           If the index with this value is used as an index of jumptable, an exception occurs.
 *           But if it is not used, it is a valid case, especially if no dynamic linked call occurs.
 */
int getIdxMtbl_ClassJc(ClassJc const* reflectionObj, ClassJc const* reflectionRef)
{ int idxMtbl = -1;
  ClassOffset_idxMtblJcARRAY const* reflectionSuper;
  STACKTRC_ENTRY("getIdxMtbl_ClassJc");
  if(reflectionObj == null)
  { //if no reflection is used, it is able in C++ environment or if no dynamic linked methods are used.
    idxMtbl = -1; //mIdxMtbl_ObjectJc;  //causes an error if it will be used!
  }
  else
  { if(reflectionRef == null)  //if no reflection is prescribed:
    { idxMtbl = 0;  //returns the Mtbl_ObjectJc
    }
    else if(reflectionRef == reflectionObj)
    { idxMtbl = 0;  //returns the whole Mtbl for the type.
    }
    else
    { ClassOffset_idxMtblJcARRAY const* reflectionIfc = 
        (ClassOffset_idxMtblJcARRAY const*)reflectionObj->interfaces;
      if( reflectionIfc != null)
      { int idxIfc;
        for(idxIfc = 0; idxMtbl < 0 && idxIfc < reflectionIfc->head.length; idxIfc++)
        { ClassOffset_idxMtblJc const* reflectionChild;
          reflectionChild = &reflectionIfc->data[idxIfc];
          if(reflectionChild->clazz == reflectionRef)
          { idxMtbl = reflectionChild->idxMtbl;
          }
        }
      }
      else
      { idxMtbl = -1;
      }
    }
    if(idxMtbl < 0 && (reflectionSuper = reflectionObj->superClasses) != null)
    { int idxSuper = 0;
      for(idxSuper = 0; idxMtbl < 0 && idxSuper < reflectionSuper->head.length; idxSuper++)
      { ClassOffset_idxMtblJc const* reflectionChild;
        reflectionChild = &reflectionSuper->data[idxSuper];
        if(reflectionChild->clazz == reflectionRef)
        { idxMtbl = reflectionChild->idxMtbl;
        }
        else
        { //Recursive call because deeper inheritance:
          idxMtbl = getIdxMtbl_ClassJc(reflectionChild->clazz, reflectionRef);
        }
      }
    }
    { //search in superclasses and there interfaces
      //old if only 1 superclass:
      //idxMtbl = getIdxMtbl_ClassJc(reflectionObj->superClass, reflectionRef);
    }

  }
  STACKTRC_LEAVE; return(idxMtbl);
}






struct MT_TypeJc_t const* getMtbl_ObjectJc(ObjectJc const* ythis, ClassJc const* type)
{ ClassJc const* clazz;
  struct MT_TypeJc_t const* mtbl;
  int idxMtbl;
  STACKTRC_ENTRY("getMtbl_ObjectJc");
  if(ythis == null) THROW_s0(NullPointerException,"this is null", 0);
  if(ythis != ythis->ownAddress) THROW_s0(RuntimeException, "ownAddress fault", (int)ythis->ownAddress);
  clazz = ythis->reflectionClass;
  //if(clazz->object.reflectionClass != &reflection_ClassJc) THROW_s0(RuntimeException, "reflection fault", (int)ythis->ownAddress);
  //may be a derivated type!!! TODO.
  //if(clazz != type) THROW_s0(RuntimeException, "fault type", (int)ythis->ownAddress);
  if(clazz == null)
  { mtbl = null;  //no reflection is given, no mtbl, but no error
  }
  else
  {
    idxMtbl = getIdxMtbl_ClassJc(clazz, type);
    if(idxMtbl < 0) THROW_s0(RuntimeException, "fault type", (int)ythis->ownAddress);
    ASSERT(clazz->mtbl != null);
    //clone() is the first method of ObjectJc, indices the first method of the interface or superclass.
    mtbl = (struct MT_TypeJc_t const*)( &(&clazz->mtbl->clone)[idxMtbl]);  
    //mtbl = (struct MT_TypeJc_t const*)( ((MT_void_Method_void*)(clazz->mtbl)) + idxMtbl);
  }
  STACKTRC_LEAVE; return mtbl;
}



bool instanceof_ObjectJc(ObjectJc const* ythis, struct ClassJc_t const* reflection)
{ 
  int idxMtbl = getIdxMtbl_ClassJc(ythis->reflectionClass, reflection);
  return idxMtbl >=0;
}



StringJc toString_Object_F(ObjectJc* ythis)
{ //StringBuffer* ss = StringBuffer::new_();
  //ss->append("Object at @").append(Integer::toHexString((int)(this)));
  StringJc ss = CONST_StringJc("No information"); //only constant
  return ss;   //return by value, that is 2 words (8 Byte).
}


//typedef StringJc (*Method_toString_Object)(ObjectJc_rp);

#if defined(__CPLUSGEN) && defined(__cplusplus)
  ObjectifcBaseJcpp::ObjectifcBaseJcpp()
  : significance_ObjectifcBase(kSignificance_ObjectifcBase)
  //, significanceAddress_ObjectifcBase(this)  NOTE: note possible because this is not accessible here! 
  { //NOTE the significanceAddress_ObjectifcBase is const void* to prevent change outside,
    //     but it can't set in constructor list with this.
    *(void**)(&significanceAddress_ObjectifcBase) = this;
  }
#endif //__CPLUSGEN


/********************************************************************************************/
//Object__Array


void* get_i_ObjectArrayJc(ObjectArrayJc const* ythis, int idx)
{
  if(idx < 0 || idx > ythis->length){ return null; } //IndexOutOfBoundaryException
  else
  {
    int offset = idx * ythis->sizeElement;
    if(true) //ythis->mode == kDirect_ObjectArrayJc)
    {
      return ((MemUnit*)(ythis + 1)) + offset;
    }
    else
    { return 0;
    }
  }
}


int size_Object__ArrayJc(const ObjectArrayJc* ythis)
{
  return ythis->length;
}


/** Gets the number of bytes of the whole array calculated from the informations inside the head.
  */
METHOD_C int getNrOfBytes_ObjectArrayJc(ObjectArrayJc const* ythis)
{ return sizeof(ObjectArrayJc) + ythis->length * ythis->sizeElement;
}


void_YP ctorO_AYJc(ObjectJc* othis, int nBytesPerElement, int nSize)
{ void_YP ythis = (void_YP)(othis);
  if(ythis != null)
  { ctorc_ObjectJc(&ythis->object);
    ythis->object.objectIdentSize |= kArray_objectIdentSize_ObjectJc;
    ythis->length = nSize;
    ythis->sizeElement = nBytesPerElement;
    ythis->mode = 0; //kDirect_ObjectArrayJc;
  }
  return ythis;
}



ObjectArrayJc* ctorc_ObjectArrayJc(ObjectArrayJc* ythis, int nSize, int nBytesPerElement, struct ClassJc_t const* reflection, int32 typeInstanceIdent)
{ //ObjectArrayJc* arrayHead = (ObjectArrayJc*)(ythis);  //the universal type of ARRAYJc
  STACKTRC_ENTRY("ctorc_ObjectArrayJc");
  if(ythis != null)
  { int32 sizeArray;
    int32 sizeInfoObject;
    ctorc_ObjectJc(&ythis->object);
    sizeArray = nSize * nBytesPerElement + sizeof(ObjectArrayJc);
    sizeInfoObject = getSizeInfo_ObjectJc(&ythis->object);  //may be setted or it should be 0
      if(sizeInfoObject > 0 && sizeInfoObject < sizeArray)
      { THROW_s0(IllegalArgumentException, "size mismatch", sizeArray);
      }    
      if( (typeInstanceIdent & mSizeBits_objectIdentSize_ObjectJc) == kIsSmallSize_objectIdentSize_ObjectJc)
      { if(sizeArray > mSizeSmall_objectIdentSize_ObjectJc)
        { THROW_s0(IndexOutOfBoundsException, "too large", sizeArray);
        }
        typeInstanceIdent &= ~mSizeSmall_objectIdentSize_ObjectJc;
      }
      else if( (typeInstanceIdent & mSizeBits_objectIdentSize_ObjectJc) == kIsMediumSize_objectIdentSize_ObjectJc)
      { if(sizeArray > mSizeMedium_objectIdentSize_ObjectJc)
        { THROW_s0(IndexOutOfBoundsException, "too large", sizeArray);
        }
        typeInstanceIdent &= ~mSizeMedium_objectIdentSize_ObjectJc;
      }
      else if(typeInstanceIdent & mIsLargeSize_objectIdentSize_ObjectJc)
      { if(sizeArray > mSizeLarge_objectIdentSize_ObjectJc)
        { THROW_s0(IndexOutOfBoundsException, "too large", sizeArray);
        }
        typeInstanceIdent &= ~mSizeLarge_objectIdentSize_ObjectJc;
      }
      else
      { THROW_s0(IndexOutOfBoundsException, "undefined size", typeInstanceIdent);
      }
      typeInstanceIdent |= sizeArray;
      //it is always 1-dimension.
      typeInstanceIdent = (typeInstanceIdent & ~mArray_objectIdentSize_ObjectJc) | kArray_objectIdentSize_ObjectJc;
    }
    setReflection_ObjectJc(&ythis->object, reflection, typeInstanceIdent);

    ythis->length = nSize;
    ythis->sizeElement = nBytesPerElement;
    ythis->mode = 0; //kDirect_ObjectArrayJc;
    { MemUnit* data = (MemUnit*)(ythis) + sizeof(ObjectArrayJc);  //The data are assigned always immediately after the head. This is the law.
      memset(data, 0, nSize * nBytesPerElement);
    }
  STACKTRC_LEAVE;
  return ythis;
}


ObjectArrayJc* ctorO_ObjectArrayJc(ObjectJc* othis, int size, int nBytesPerElement, struct ClassJc_t const* reflection, int32 typeInstanceIdent)
{ ObjectArrayJc* ythis = (ObjectArrayJc*)othis;
  ctorc_ObjectArrayJc(ythis, size, nBytesPerElement, reflection, typeInstanceIdent);
  return ythis;
}



METHOD_C ObjectArrayJc* new_ObjectArrayJc(int SIZE, int BYTESELEMENT, struct ClassJc_t const* REFLECTION, int32 typeInstanceIdent)
{ int nrOfBytes = sizeof(ObjectArrayJc) + SIZE * BYTESELEMENT;
  ObjectArrayJc* address;
  //STACKTRC_ENTRY("new_ObjectArrayJc");
  address = (ObjectArrayJc*)(malloc(nrOfBytes));
  //if(address != null){ memset(address, 0, nrOfBytes); }
  ctor_ObjectArrayJc(address, SIZE, BYTESELEMENT, REFLECTION, typeInstanceIdent);
  //STACKTRC_LEAVE;
  return address;
  //return &address->object;

}









int32ARRAY* ctor_int32ARRAY(int32ARRAY* ythis, int nrOfElements)
{ if(ythis != null)
  { ythis->head.length = nrOfElements;
    ythis->head.sizeElement = sizeof(int32);
    ythis->head.mode = 0; //kDirect_ObjectArrayJc;
    memset(ythis->data, 0, nrOfElements * ythis->head.sizeElement);
  }
  return ythis;
}





extern ClassJc const XXXreflection_StringJc;
const struct Reflection_Fields_StringJc_t
{ ObjectArrayJc head;
  FieldJc data[2];
} XXXreflection_Fields_StringJc =
{ CONST_ObjectArrayJc(FieldJc, 2, OBJTYPE_FieldJc, null, &XXXreflection_Fields_StringJc)
, {
    { "refbase"
    , 0 //nrofArrayElements
    , REFLECTION_int32
    , (4<<kBitPrimitiv_Modifier_reflectJc) //bitModifiers
    , (int16)((int32)(&((StringJc*)(0x1000))->value__) - (int32)(StringJc*)0x1000)
    , 0  //offsetToObjectifcBase
    , &XXXreflection_StringJc
    }
  , { "ref"
    , 0 //nrofArrayElements
    , REFLECTION_char
    , (1<<kBitPrimitiv_Modifier_reflectJc)| mReference_Modifier_reflectJc //bitModifiers
    , (int16)((int32)(&((StringJc*)(0x1000))->ptr__) - (int32)(StringJc*)0x1000)
    , 0  //offsetToObjectifcBase
    , &XXXreflection_StringJc
    }
} };


ClassJc const XXXreflection_StringJc =
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &XXXreflection_StringJc, null)
, "StringJc"
, 0     //posObjectJc
, sizeof(StringJc)
, (FieldJcArray const*)&XXXreflection_Fields_StringJc  //attributes and associations
, null  //method
, null  //superclass
, null  //interfaces
, 0x0  //modifiers
, null  //Mtbl
};



const struct xxxReflection_Fields_ObjectJcREF_t
{ ObjectArrayJc head;
  FieldJc data[2];
} xxxreflection_Fields_ObjectJcREF =
{ CONST_ObjectArrayJc(FieldJc, 2, OBJTYPE_FieldJc, null, &xxxreflection_Fields_ObjectJcREF)
, {
    { "refbase"
    , 0 //nrofArrayElements
    , REFLECTION_int32
    , (4<<kBitPrimitiv_Modifier_reflectJc) //bitModifiers
    , (int16)((int32)(&((StringJc*)(0x1000))->value__) - (int32)(StringJc*)0x1000)
    , 0  //offsetToObjectifcBase
    , &XXXreflection_StringJc
    }
  , { "ref"
    , 0 //nrofArrayElements
    , REFLECTION_char
    , (1<<kBitPrimitiv_Modifier_reflectJc)| mReference_Modifier_reflectJc //bitModifiers
    , (int16)((int32)(&((StringJc*)(0x1000))->ptr__) - (int32)(StringJc*)0x1000)
    , 0  //offsetToObjectifcBase
    , &XXXreflection_StringJc
    }
} };


const ClassJc xxxreflection_ObjectJcREF =
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &xxxreflection_ObjectJcREF, null)
, "ObjectJcREF"
, 0
, sizeof(ObjectJcREF)
, (FieldJcArray const*)&xxxreflection_Fields_ObjectJcREF  //attributes and associations
, null  //method
, null  //superclass
, null  //interfaces
, 0x0  //modifiers
};




