/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 * @content This file defines basicly properties of each data like java.lang.Object.
 *
 * @author Jchartmut www.vishia.org
 * @version 0.82
 * list of changes:
 * 2008-04-22: JcHartmut docu
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/
#ifndef __ObjectJc_h__
#define __ObjectJc_h__

/** The system depending definitions*/
#include <os_types_def.h>
#include "FwConv/fw_SysConventions.h"
#include "Fwc/fw_SimpleC.h"
#include "Fwc/fw_MemC.h"
#include "Fwc/fw_ThreadContext.h"  //knowledge of Thcxt, Exception is supplied
#include "Fwc/objectBaseC.h"
#include "Fwc/fw_String.h"


/*@CLASS_C ObjectJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**
 */
//NOTE: the Definition of the struct is containing in objectBaseC.h

/** Macro for constant initialisation with a typesize and a given reflection class.
  * @param TYPESIZEOF see attribute ,,objectIdentSize,,.
  * @param REFLECTION the reflection class of the constant object. It may be ,,null,, also.
*/
#define CONST_ObjectJc(TYPESIZEOF, OWNADDRESS, REFLECTION) {TYPESIZEOF, OWNADDRESS, 0, kNoSyncHandles_ObjectJc, REFLECTION }

/**Initialization of the basicly data of Object.
 * This method should be used for static and embedded instances.
 * @param sizeObj The size of the whole Object (instance type) in memory.
 * @param identObj An identification info, see [[class_ObjectJc.objectIdentSize]] 
 * return ythis
*/
METHOD_C ObjectJc* init_ObjectJc(ObjectJc* ythis, int sizeObj, int identObj);



/** The default construtor. An offset to BlockHeap in mem is considered. */
METHOD_C ObjectJc* ctorM_ObjectJc(MemC mem); //ObjectJc* ythis);
#define ctor_ObjectJc(MEM) ctorM_ObjectJc(MEM)

/** The default construtor in C-manner. It assumes, that the area is cleared or is initialized already. */
METHOD_C void ctorc_ObjectJc(ObjectJc* ythis);

/**Allocates an memory area and initialized it with the basicly values of ObjectJc.
 * * The element objectIdentSize is set with the size information.
 * * The element ownAddress is set
 * * The element reflectionClass is set to null.  
 * * All other elements are set initially.
 * @param size The size of the Object to allocate.
 * @param typeInstanceIdent Information bits, see [[setIdentAndSize_ObjectJc(int, int)]],
 *        but without the size information. 
 *        The bits kIsMediumSize_objectIdentSize_ObjectJc and mIsLargeSize_objectIdentSize_ObjectJc
 *        are regarded especially.
 * @return the reference to the initialized Object.
 * @throws IndexOutOfBoundsException if the size doesn't match to typeInstanceIdent. 
*/
METHOD_C ObjectJc* alloc_ObjectJc(const int size, const int32 typeInstanceIdent, ThCxt* _thCxt);


/** sets the reflection class. This method is to be called by user in its constructor of a derivated class.
  * @param reflection The reflection class of the instance, appropriate to the derivated type.
  * @param objectIdentSize The type or instance and size ident.

           This value will be stored immediately in the attribute objectIdentSize.
           The user may be set a 0-value, if no size or object ident is used in the users scope.
           But if the data are saved and transfered as POD plain old data to evaluate it,
           this information may be necessary for evaluation. The user should calculate this value
           carefully in respect to the size, instance and array bits, see enum ,,k..._objectIdentSize_ObjectJc,,.
           If no ident ifno is necessary, the size may be stored here. If the object has a low size (up to 64 k in a 32-bit-System),
           a simple way is to write ,,sizeof(Type),, or ,,sizeof(ythis),,,
           because the kIsSmallSize_objectIdentSize_ObjectJc is defined with 0.
           But it is exacting to write ,,kIsSmallSize_objectIdentSize_ObjectJc + sizeof(ythis),,,
           because it is shown here that it is a small size object.
  * @return void.
*/
METHOD_C void setReflection_ObjectJc(ObjectJc* ythis, struct ClassJc_t const* reflection, int32 objectIdentSize);


/**Sets the ident and size of an ObjectJc.
 * The bit position of ident and size, and the bits to define which bits are using for ident and size [[mSizeBits_objectIdentSize_ObjectJc]]
 * are calculated from given size and type.
 * @throws RuntimeException if the identObj and sizeObj are not matching. See Bits of [[class_ObjectJc_objectIdentSize]].
 */
METHOD_C void setSizeAndIdent_ObjectJc(ObjectJc* ythis, int sizeObj, int identObj, int nrofArrayDimensions);


/**Tests wether it is an ObjectJc-based instance or not. The criterium is, the ownAddress inside ObjectJc must be identical with the instance address.
 * @throws an exception if it is not true.
 */
#define testConsistence_ObjectJc(SRC) { if( (SRC) != null && (SRC) != (SRC)->ownAddress ) THROW(RuntimeException, "no ObjectJc-based instance.", (int)(SRC)); }




METHOD_C void checkConsistence_ObjectJc(ObjectJc* ythis, int size, struct ClassJc_t const* clazzReflection, ThCxt* _thCxt);


/** gets the reflection class.*/
METHOD_C struct ClassJc_t const* getClass_ObjectJc(ObjectJc const* ythis);

/** gets the size of the Object stored in attribute objectIdentSize. This value helps to step through data
  * particularly by using direct composed plain old data,
  * but the user is responsible for the exactly realisation.
  */
METHOD_C int getSizeInfo_ObjectJc(ObjectJc const* ythis);

/**gets the ident and size of the Object stored in attribute objectIdentSize.
 * This value is useable as debug information.
 */
METHOD_C int getIdentInfo_ObjectJc(ObjectJc const* ythis);
//#define getIdentInfo_ObjectJc(YTHIS) (

/**tests wether the given object is an instance of the requested Type.
 * Javalike: instanceof-operator.
 * @param ythis any Object
 * @param reflection The reflection of the type to compare.
 */
METHOD_C bool instanceof_ObjectJc(ObjectJc const* ythis, struct ClassJc_t const* reflection);

/****************************************************************************************************
 * Virtual methods of ObjectJc:
 * The virtual methods are defined as Methodtypes.
 */

/** Produces a new instance equal to the old one. The method is virtual,
  * because it may be able that some register operations or others should be done
  * by the users class. This operation is the object-oriented replacement
  * for the old C-style memcpy of instances data.
*/
typedef METHOD_C ObjectJc* MT_clone_ObjectJc(ObjectJc const* ythis, MemC buffer);

/**Tests wether or not both objects are equal. The statement of homogeneousness
 * may be depended on some informations inside the instances, other informations
 * may be differenced, though both objects are equal. Thats why the method is virtual.
 * This operation is the object-oriented replacement
 * for the old C-style memcpy of instances data.
 */
typedef METHOD_C bool MT_equals_ObjectJc(ObjectJc const* ythis, ObjectJc const* second);

/**This method should be called before the instance is deleted finally. It is called
 * by garbage collection. Some action such as deregister or free memory may be taken
 * from the users overload class.
 */
typedef METHOD_C void MT_finalize_ObjectJc(ObjectJc* ythis, ThCxt* _thCxt);

/**This method supplies a hash code for thid onstance.
 */
typedef METHOD_C int32 MT_hashCode_ObjectJc(ObjectJc const* ythis);

/**This method supplies a string representation of the content of the instance,
 * useable especially for debug or analyze information.
 */
typedef METHOD_C StringJc MT_toString_ObjectJc(ObjectJc* ythis);

/** The definition of ordered method pointer, it is a jump table.
  * The virtual methods are callable in a C-environment adequate to virtual methods in C++.
  * This struct of method pointer determines the order of virtual methods of ObjectJc for all implementation classes.
  */
typedef struct Mtbl_ObjectJc_t
{ MT_clone_ObjectJc*    clone;
  MT_equals_ObjectJc*   equals;
  MT_finalize_ObjectJc* finalize;
  MT_hashCode_ObjectJc* hashCode;
  MT_toString_ObjectJc* toString;
} Mtbl_ObjectJc;


/**returns the method table of the given instance. 
 * This routine tests 
 * *the consistence of the instance: the own address have to be in ObjectJc.
 * *the existence of reflection information: no null-pointer
 * *the correctness of reflection information: Significance test.
 * *the existence of the Methode table: not null pointer
 * *the correctness of the Method table: Significance test.
 * If all this conditions are met, the reference to the Method table is returned.
 * If any condition are not correct, a RuntimeException is thrown. 
 *
 * This is useable in safety critical software: If any error in data occurs, 
 * the programm is aborted, the throw may be catched or not, but the software works deterministic.
 *
 * The Method table may be a derivated table, the Mtbl_ObjectJc-base is returned.
 * Wether the method table is derivated or not, it means wether a up-casting is allowed,
 * depends on a derivated class. It should be defined there. This method is used basicly.
 */
METHOD_C struct MT_TypeJc_t const* getMtbl_ObjectJc(ObjectJc const* ythis, struct ClassJc_t const* type);



/**To call from user the user should use the following macros:
 * NOTE: REF is a ObjectJc* pointer, not a ObjectJcREF-instance.
 * It is different from interface method call.
 * This variant enables using virtual methods of ObjectJc without enhanced references.
*/
#define clone_ObjectJc(REF)          ((MT_clone_ObjectJc*)   ( (REF)->reflectionClass->mtbl[0]))((REF))
#define equals_ObjectJc(REF, SECOND) ((MT_equals_ObjectJc*)  ( (REF)->reflectionClass->mtbl[1]))((REF))
#define finalize_ObjectJc(REF)       ((MT_finalize_ObjectJc*)( (REF)->reflectionClass->mtbl[2]))((REF))
#define hashCode_ObjectJc(REF)       ((MT_hashCode_ObjectJc*)( (REF)->reflectionClass->mtbl[3]))((REF))
#define toString_ObjectJc(REF)       ((MT_toString_ObjectJc*)( (REF)->reflectionClass->mtbl[4]))((REF))

/** The implemented method for ObjectJc forward declared here.
 *  They should be known if a user class will not overload some methods from ObjectJc with its own methods
 *  but use this methods.
 *  In generally, implementation methods needn't be forward declared, they need only be defined in the C file.
 */
extern MT_clone_ObjectJc clone_Object_F;
extern MT_equals_ObjectJc equals_Object_F;
extern MT_finalize_ObjectJc finalize_Object_F;
extern MT_hashCode_ObjectJc hashCode_Object_F;
extern MT_toString_ObjectJc toString_Object_F;


/** The virtual table of non overloaded methods from object,
    useable as part of a users virtual table.
  */
#define CONST_MtblStd_ObjectJc \
{ clone_Object_F    \
, equals_Object_F   \
, finalize_Object_F \
, hashCode_Object_F \
, toString_Object_F \
}


/**********************************************************************************************
 * Methods for thread synchronization
 */


/**The synchronized(obj) keyword from Java. */
METHOD_C void synchronized(ObjectJc* obj);

/**The synchronizedEnd-Methode should be called on end of a synchronized block.
 * write:
 * ,,  synchronized(obj);
 * ,,  { //critical section
 * ,,  }synchronizedEnd(obj);
 */
METHOD_C void synchronizedEnd(ObjectJc* obj);

/**The wait method, see java.lang.Object.wait(long),
 * In opposite to original Java, the millisecond arg is 32 bit, so the max wait time is limited to about 24 days.
 */
METHOD_C void wait_ObjectJc(ObjectJc* obj, int milliseconds);


/**The notify method, see java.lang.Object.wait(long),
 */
METHOD_C void notify_ObjectJc(ObjectJc* obj);

/**The notifyAll method, see java.lang.Object.wait(long),
 */
METHOD_C void notifyAll_ObjectJc(ObjectJc* obj);



/*@CLASS_C ObjectArrayJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**Base data of an array.
 * This type is used as base data of an user defined array. The array data followes directly
 * in the user struct. The array data need not have ObjectJc as base.
 * *ObjectJc object: The array base type based on Object. Reflection is useable for the whole array.
    *int32 length: acual no of elements, in java it is the final public attribute length
    *int16 sizeElement size of an element, this is the sizeof a pointer if it is a array of pointers.
    *int16 mode: mode of operation, see enum Emode. Here is coding the no of elements in 1 block
      and the using of pointers or not.

*/
typedef struct  ObjectArrayJc_t
{ //ARRAYJc
  /**Head struct, it contains especially the number of dimensions. */
  ObjectJc object;

  /**Nr of elements of the array. It may be greater than 65000. */
  int32 length;

  /**size of an element. If the element is a pointer, the sizeof is the sizeof the pointer. */
  int16 sizeElement;

  /**Rerserve Element. */
  int16 mode;
  //#define kPointered_ObjectArrayJc 1
  //#define kDirect_ObjectArrayJc 2
}ObjectArrayJc;


/** Constant definition of the head of any array
  * @param TYPE the type of the elements, used in sizeof(TYPE) and in reflection##TYPE
  * @param SIZE number of elements
  */
#define CONST_ObjectArrayJc(TYPE, SIZE, IDENT, REFLECTION, OWNADDR) \
  { CONST_ObjectJc(IDENT + sizeof(ObjectArrayJc) + (SIZE) * sizeof(TYPE), OWNADDR, REFLECTION), SIZE, sizeof(TYPE), 0 }


/** initializes the data of the array. This includes the head and the associated data. It is the rule or law,
 *  that all data are disposed immediately after the head of array. The data are set to 0.
 *
 *  @param ythis The head of the array.
 *  @param size  The number of elements of the 1-dimensional array
 *  @param nBytesPerElement sizeof(ElementType)
 *  @param reflection The reflection definition
 *  @param typeInstanceIdent The Type or Instance Id without size information but with the kind of size information, see kIsMediumSize_objectIdentSize_ObjectJc.
 *
 */
METHOD_C ObjectArrayJc* ctorc_ObjectArrayJc(ObjectArrayJc* ythis, int size, int nBytesPerElement, struct ClassJc_t const* REFLECTION, int32 typeInstanceIdent);
#define ctor_ObjectArrayJc(YTHIS, SIZE, BYTESPERELEMENT, REFLECTION, IDENT) ctorc_ObjectArrayJc(YTHIS, SIZE, BYTESPERELEMENT, REFLECTION, IDENT) 

/**initializes a ObjectArrayJc, see [[ctorc_ObjectArrayJc(...)]].
 * @param rawMem raw memory area, it may be uninitialized. 
 *        The size may be greater as necessary. The constructor initializes only its own area.
 * @throws IllegalArgumentException if the sizeof rawMem is to less.  
 */
METHOD_C ObjectArrayJc* ctorM_ObjectArrayJc(MemC rawMem, int size, int nBytesPerElement, struct ClassJc_t const* REFLECTION, int32 typeInstanceIdent);


/**initializes a ObjectArrayJc, see [[ctorc_ObjectArrayJc(...)]].
 * @param rawMem raw memory area, it may be uninitialized. 
 *        The size may be greater as necessary. The constructor initializes only its own area.
 * @throws IllegalArgumentException if the sizeof rawMem is to less.  
 */
METHOD_C ObjectArrayJc* ctorO_ObjectArrayJc(ObjectJc* objthis, int size, int nBytesPerElement, struct ClassJc_t const* REFLECTION, int32 typeInstanceIdent);




/**creates an new Array of any desired struct type. The memory for the array is reserved, all elements are setted to 0,
 * and the ObjectArray-Head-Structure is initialized.
 * Do not use this method to create an array of a C++ class, because there constructors are not called
 * and no virtual-table-pointer are initialized.
 * @param size Number of elements of the 1-dimensional array.
 * @param bytesPerElement This value should be written as ,,sizeof(ElementType),,.
 * @param reflection The reflection class of the elements. Note: The elements need not have ObjectJc as baseclass.
 *        The access to reflection info is possible via the array head.
 * @param typeInstanceIdent It may be 0. If the data are evaluate in a POD plain old data data image,
 *        the value may be important. See element ,,objectIdentSize,, in ,,ObjectJc,,.
 *        But here only the ident bits, not the size should be given.
 * @param STACKTRCP My be null, if no memory allocation error is expected.
 * @return The pointer of the allocated data as reference to ObjectArrayJc. A casting outside is necessary, see macro new_ARRAYJc.
 * @throws RuntimeException if no memory is allocatable.
 */
METHOD_C ObjectArrayJc* new_ObjectArrayJc(int size, int bytesPerElement, struct ClassJc_t const* reflection, int32 typeInstanceIdent);



/** Gets the pointer to an element int the data area of the array, indexed by idx.
This method returns a not constant pointer*/
METHOD_C void* get_i_ObjectArrayJc(ObjectArrayJc const* ythis, int idx);

/** Gets the size of the array, it means number of elements of the array, not number of bytes.
  It is the content of element length. Because it is a simple access a define may be a possible implementation.
  An compiler-errormessage may be clear cut, if the user works with wrong argument BASE.
*/
#define size_ObjectArrayJc(BASE) ((BASE)->length)
//METHOD_C int size_ObjectArrayJc(const ObjectArrayJc* base);


/** Gets the number of bytes of the whole array calculated from the informations inside the head.
  */
METHOD_C int getNrofBytes_ObjectArrayJc(ObjectArrayJc const* ythis);
#define getNrOfBytes_ObjectArrayJc(THIS) getNrofBytes_ObjectArrayJc(THIS)


/*@DEFINE ARRAYJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**This macro helps to define user specific javalike array pointer. 
 * The structure of the user defined type follows the structure of ObjectArrayJc. 
 * But because a direct access to the element length should be ... 
 */
#define TYPEDEF_AYJc(TYPE) \
typedef struct TYPE##_Y_t{ ObjectJc object; int32 length; int16 sizeElement; int16 mode; TYPE data[50]; } *TYPE##_Y; \
typedef struct TYPE##_YREF_t{ ObjectRefValuesJc refbase; TYPE##_Y ref; } TYPE##_YREF

#define TYPEDEF_AYPJc(TYPE) \
typedef struct TYPE##_YP_t{ ObjectJc object; int32 length; int16 sizeElement; int16 mode; TYPE* data[50]; } *TYPE##_YP; \
typedef struct TYPE##_YPREF_t{ ObjectRefValuesJc refbase; TYPE##_YP ref; } TYPE##_YPREF

TYPEDEF_AYJc(int8);
TYPEDEF_AYJc(int16);
TYPEDEF_AYJc(int32);
TYPEDEF_AYJc(int64);
TYPEDEF_AYJc(bool);
TYPEDEF_AYJc(char);
TYPEDEF_AYJc(StringJc);

TYPEDEF_AYPJc(void);
TYPEDEF_AYPJc(ObjectJc);

//typedef struct StringJc_AY_t{ ObjectJc object; int32 length; int16 sizeElement; int16 mode; StringJc data[50]; } *StringJc_AY;
//typedef struct StringJc_AYREF_t{ ObjectJc object; int32 length; int16 sizeElement; int16 mode; StringJc data[50]; } *StringJc_AYREF;


/*@DEFINE ARRAYJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/**Macro to define a Object based array.
   syntactical using as type in form
   , typedef ARRAYJc(MyElementType, 23) MyArrayType;
   or
   , ARRAYJc(MyElementType, 23) myArrayData;
   It results as a definition in form:
   , struct { Object_ArrayJc head; TYPE data[SIZE]; }
   @param TYPE the type of the elements
   @param SIZE nr of elements to shown in the array type. Often the defined arraytype
          is used as pointer, in that cases the number of elements may not be identical
          with the number of elements defined here. Use new_ARRAYJc to get a instance.
   @return inappropriate, it is used like a type.
 */
#define ARRAYJc(TYPE, SIZE) struct { ObjectJc object; int32 length; int16 sizeElement; int16 mode; TYPE data[SIZE]; }
//#define ARRAYJc(TYPE, SIZE) struct { ObjectArrayJc head; TYPE data[SIZE]; }


#define ctor_AYJc(OBJ, SIZE) { ctorc_ObjectJc(&(OBJ)->object); setIdentAndSize_ObjectJc(&(OBJ)->object, 0, sizeof_AYJc((OBJ)->data[0], SIZE)); (OBJ)->length = SIZE; (OBJ)->sizeElement = sizeof((OBJ)->data[0]); } 

METHOD_C void_YP ctorO_AYJc(ObjectJc* othis, int sizeElement, int nrofElements);

/** Gets the reference of the addressed element of an array with any desired type.
  * This is a macro because the type is unknown here and kept from the users ARRAY
  * by macro expansion.
  * @param ARRAY instance of an array, must construct with "ObjectArrayJc head; Usertype data;"
  * @param IDX the index of the instance
  * @return the "left value" as C-syntax of the element.
  * @throws IndexOutOfBoundsException if the IDX is failed.
  *Note: 3thrd Parameter TYPE is necessary for universal use, especially THROWJc
  */
#define element_AYJc(ARRAY, IDX) \
  (  (IDX) < 0 || (IDX) >= (ARRAY)->length ? /*(TYPE)THROWJc(IndexOutOfBoundsExceptionJc, "", 0)*/ null :  &((ARRAY)->data[IDX]) )
#define element_ARRAYJc(ARRAY, IDX) \
  (  (IDX) < 0 || (IDX) >= (ARRAY)->head.length ? /*(TYPE)THROWJc(IndexOutOfBoundsExceptionJc, "", 0)*/ null :  &((ARRAY)->data[IDX]) )


  /** calculates the size of an array with ObjectArrayJc-Head in compiletime.
    * This is a macro because the type is given.
    * @param TYPE The type of the elements.
    * @param SIZE The number of the elements.
    * @return The number of bytes
    */
#define sizeof_AYJc(TYPE, SIZE) (sizeof(ObjectJc) + 2*sizeof(int32) + (SIZE) * sizeof(TYPE))
#define sizeof_ARRAYJc(TYPE, SIZE) (sizeof(ObjectArrayJc) + (SIZE) * sizeof(TYPE))


/**creates an new Array of the given struct type. The memory for the array is reserved, all elements are setted to 0,
 * and the ObjectArray-Head-Structure is initialized.
 * Do not use this method to create an array of a C++ class, because there constructors are not called
 * and no virtual-table-pointer are initialized.
 *
 * @param TYPE The type of the elements. It should be a struct type.
          It is used to calculate the sizeof(TYPE) for memory allocation and to cast the result pointer to the expected type.
 * @param SIZE Number of elements of the 1-dimensional array.
 * @param REFLECTION The reflection class of the elements. Note: The elements need not have ObjectJc as baseclass.
 *        The access to reflection info is possible via the array head.
 * @param IDENT It may be 0. If the data are evaluate in a POD plain old data data image,
 *        the value may be important. See element ,,objectIdentSize,, in ,,ObjectJc,,.
 *        But here only the ident bits, not the size should be given.
 * @return The pointer of the allocated data in the correct type reference.
 * @throws RuntimeException if no memory is allocatable. In the environment the STACKTRC_ENTRY("name") should be present.
 */
#define new_ARRAYJc(TYPE, SIZE, REFLECTION, IDENT) ((TYPE##Array*)(new_ObjectArrayJc(SIZE, sizeof(TYPE), REFLECTION, IDENT)))



/*@CLASS_C int8ARRAY @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/



/** Array of direct adressed integer values with 32 bit.
  * This may be a template to build users arrays:
  * Copy it and replace 'int32' with the users type.
  * If the users array is a array of pointers, the users type is such as 'Userclass*'
*/
typedef struct int8ARRAY_t
{ /** consist of the base data of the array*/
  ObjectArrayJc head;
  /** direct data followed immediately after the ObjectArrayJc.
      The size of the array given here is only a helper for debugging, the debugger
      shows the given number of value. The really value is held inside array.
  */
  int8 data[0x100];
}int8ARRAY;

/** The constructor accepts also a null pointer (on error) and returns the pointer itself,
  * necessary to use inside definition new_type
  */
METHOD_C int8ARRAY* ctor_int8ARRAY(int8ARRAY* ythis, int nrOfBytes);


/** Gets the sizeof of the instance with given nr of elements. It is a static-like method.
 * @param SIZE number of elements
*/
#define sizeof_int8ARRAY(SIZE) (sizeof(ObjectArrayJc) + SIZE * sizeof(int32))

#define staticSize_int8ARRAY (sizeof(((int8ARRAY*)(0x1000))->data) / sizeof(int8))

#define CONST_int8ARRAY(OBJP) { CONST_ObjectArrayJc(int8, 100, 0, REFLECTION_int8, OBJP->head), {0} }
//#define CONSTaddSize_int32ARRAY(OBJP, ADDSIZE) { CONST_ObjectArrayJc(int32, 100 + ADDSIZE, 0, REFLECTION_int32, (OBJP)->head), {0} }
#define CONSTaddSize_int8ARRAY(OBJP, ADDSIZE) { CONST_ObjectArrayJc(int8, 100 + ADDSIZE, 0, null, null), {0} }

/*@CLASS_C Int32ARRAY @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/**Number of array elements of a instance definition of a int32ARRAY.
 * An int32ARRAY with any desired number of elements may be created dynamically, the int32ARRAY- Type
 * is than used as a pointer type.
 * But if an instance of int32ARRAY is defined, the number of elements are fixed as defined in the struct.
 * This macro returns the defined number of.
 */
#define instanceSize_int32ARRAY 100

/** Array of direct adressed integer values with 32 bit.
  * This may be a template to build users arrays:
  * Copy it and replace 'int32' with the users type.
  * If the users array is a array of pointers, the users type is such as 'Userclass*'
*/
typedef struct int32ARRAY_t
{ /** consist of the base data of the array*/
  ObjectArrayJc head;
  /** direct data followed immediately after the ObjectArrayJc.
      The size of the array given here is only a helper for debugging, the debugger
      shows the given number of value. The really value is held inside array.
  */
  int32 data[instanceSize_int32ARRAY];
}int32ARRAY;





/** The constructor accepts also a null pointer (on error) and returns the pointer itself,
  * necessary to use inside definition new_type
  */
METHOD_C int32ARRAY* ctor_int32ARRAY(int32ARRAY* ythis, int nrOfBytes);

/** Gets the element with given Index. The element is the value itself, of type 'int32'*/
#define get_int32ARRAY(ARRAY,IX) (*(int32*)(get_i_ObjectArrayJc(&ARRAY->array, IX)))

/** Sets the element with given Index*/
#define set_int32ARRAY(ARRAY,IX, VAL) (*(int32*)(get_i_ObjectArrayJc(&ARRAY->head, IX)) = VAL)

/** Gets the sizeof of the instance with given nr of elements. It is a static-like method.
 * @param SIZE number of elements
*/
#define sizeof_int32ARRAY(SIZE) (sizeof(ObjectArrayJc) + SIZE * sizeof(int32))

/** Create a new Instance with the given number of elements
 * @param size number of elements
*/
//METHOD_C Int32ARRAY* new_Int32ARRAY(int size);

#define new_int32ARRAY(SIZE) (ctor_int32ARRAY( (int32ARRAY*)malloc(sizeof_int32ARRAY(SIZE)), SIZE ))



#define CONST_int32ARRAY(OBJP, SIZE) { CONST_ObjectArrayJc(int32, SIZE, 0, REFLECTION_int32, &(OBJP)->head), {0} }

/*@ARRAY ObjectJcARRAY @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

//TYPEDEF_ARRAYJc(ObjectJc, 50)
typedef struct ObjectJcARRAY{ ObjectArrayJc head; ObjectJc* data[50]; }ObjectJcARRAY;





/*@CLASS_CPP @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/



#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)


  /*=====================================================================================*/
  /* C++-version of Object, named ObjectJcpp */

  class ObjectJcpp;
  struct ClassJc_t;



  class ObjectBaseJcpp: public ObjectifcBaseJcpp //, public virtual ObjectBaseJc
  {

    /** The destructor. It should be virtual because the correct destructor
        for an instance by referencing the instance via a base class must be used.
    */
    public: virtual ~ObjectBaseJcpp(){}

    virtual StringJc toString()=0;

    virtual void finalize()=0;

    public: virtual ObjectJcpp* toObjectJcpp()=0;
    public: virtual ObjectJc* toObjectJc()=0;


  };

//  #endif
//  #endif
//  #endSBNF

  class  ObjectJcpp: public ObjectifcBaseJcpp, public ObjectJc
  {
    /** The data from the C-like class: */
    //private: ObjectJc objectdata;


    /** RuntimeHeap is the class to get Block in Heap on Runtime.
        The protected method finalize() should be accesable from Runtimeheap.
    */
    friend class RuntimeHeap;

    /** Access also to private attributes in reflection.
    */
    friend class Reflection__Object;


    /** default constructor with no reflection information. The reflection  pointer will be
        setted to Reflection__Object.
    */





   /**copy-constructor*/
    //Object(const Object&);


    /** Implements virtual method from Object__BaseJc. Because the method here
        is not virtual, no further overload can be done.<br/>
        The type conversion to Object* is required in baseclass (interface) Object__Base
        and is implemented here. It returns this. Note that the value of the this-pointer
        dependence on the inheritance structure. A this-pointer of type of an interface
        pointered exactly the virtual table of the interface, a this-pointer of a multiple
        inheranced base class pointered the part of the class represented this base class.
        It is not equal to the this-pointer to the instance itself, referenced with the
        instance-type pointer. The compiler adds or subtracts some adress differences.
    */
    //public: operator ObjectJcpp*(){ return this; }  //type conversion

    /** see java-documentation*/
    public: int hashCode() const;

    /** wake up one waiting thread if any threads called wait().
        It wakes up exactly one thread, but it is not defined which thread if several
        threads are waiting.
    */
    public: void notify();

    /** wake up all waiting thread if any threads called wait().
    */
    public: void notifyAll();

    /** to fall asleep the current thread. The Thread will be waked up
        either by calling notify on this object from another thread or if the time is expired.
        If the value of time is less 0, no timeout is used.
    */
    public: void wait(long nTime = -1);


    public: ObjectJcpp* clone() const;

    public: class ClassJcpp const* getClass() const;


    /** casting from Object* to String
    */
    //operator class String();
    //operator class String&();
    //operator class String*();




    /** substitue the java operator instanceof
    */
    public: bool instanceof( struct ClassJc_t const* reflection);

    /** compare to content of the instance, take the length in memory over reflection
    */
    public: virtual bool equals(ObjectJcpp const* obj) const
    {
      //return this == obj;
      return false; //toObjectJc() == obj;
    }

    //void synchronized(class ObjectJcpp* obj);

    //void synchronizedEnd(class ObjectJcpp* obj);


    /** In the runmode parentRef() is to be execute for all member references of a class.
     ** The runmode is to coordinate with the using of RuntimeHeap.
     ** parentRef() are necesserry for the garbage collector.
    */
    static bool isRunMode();





    /** setuseless is used instead of garbage collection in C++. If any application don't need this instance
        furtherly, it called setuseless(). The instance itself must clarify the conditions to delete or destroy it.
        Thats why the user should overwrite this method.<br/>
        The default manner is to delete the instance immediately.
    */
    public: virtual void setuseless(){ delete this; }

    /** delete__ called by garbage collector at last, to free system ressources*/
    public: void delete__();

    public: ObjectJcpp()
    { ObjectJc* ythis = static_cast<ObjectJc*>(this);
      init0p_MemC(ythis, sizeof(ObjectJc));  //note: ctorc don't initialze with 0.
      ctorc_ObjectJc(ythis); 
    }


    void setReflection(struct ClassJc_t const* reflection, int32 typeInstanceSizeIdent = 0)
    //{ setReflection_Object(&(this->objectdata), reflection);
    {
      setReflection_ObjectJc(static_cast<ObjectJc*>(this), reflection, typeInstanceSizeIdent);
    }


    virtual StringJc toString(){ return toString_Object_F(static_cast<ObjectJc*>(this)); }

    virtual void finalize(){}



    public: virtual ObjectJc* toObjectJc(){ return this;}
    public: virtual ObjectJcpp* toObjectJcpp(){ return this;}
  };

#endif  /*__CPLUSPLUSJcpp*/




/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*@DEFINE_C References @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/




#if defined(mBackRef_ObjectJc) || defined(mIdxMtbl_ObjectJc)
  /**If enhanced references are used, there are defined here.
   */
  /** This file should not be present in a system without enhanced references.*/
  #include "ObjectRefJc.h"

#else
  /**No enhanced references are used. Therefore some defines are simple or empty. */
  #define NOENHANCEDREFJc
  #define setBackRefJc(REF, OBJ)
  #define clearBackRefJc(REF)
  #define REF_ObjectJc ObjectJc*
  #define ObjectJcREF ObjectJc*
  #define REFP_ObjectJc ObjectJc*
  #define ObjectJcREFP ObjectJc*
  #define REFTYPEDEFJc(TYPE) typedef struct TYPE##_t* TYPE##REF;  typedef struct TYPE##_t* TYPE##REFP;

  /** The method to test wether a reference is null or not*/
  #define ISNULLREFJc(REF) ((REF) == null)
  /**Sets a reference to null.
   * Use this macro if the source may be used in a system conventions environment with enhanced references.
   * If the reference is simple, it is a simple setting.
   * If the reference is enhanced, it deletes a backward reference if the reference is used before.
   */
  #define CLEARREFJc(REF) ((REF) = null)
  #define SETREFJc(REF, OBJP, REFTYPE) (REF = OBJP)
  /**Access to a reference of the instance, variant if no enhanced reference is used, it is the reference itself.
   */
  #define REFJc(REF) (REF)
  #define activateGarbageCollectorAccess_BlockHeapJc(OBJ)

#endif



/*@CLASS_C Thread_Jc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


METHOD_C void sleep_Thread_Jc(int32 milliseconds);



/*@CLASS_C MathJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

#define PI_MathJc 3.141592653589793

METHOD_C double pow_MathJc(double a, double b, ThCxt* _thCxt);


/*@CLASS_C IntegerJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

#define MAX_VALUE_IntegerJc 0x7FFFFFFF
#define MIN_VALUE_IntegerJc 0x80000000


/*@DEFINE_C all standard reference types @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
REFTYPEDEFJc(StringBufferJc);
REFTYPEDEFJc(ConcurrentLinkedQueueJc);
REFTYPEDEFJc(LogMessageFW);
REFTYPEDEFJc(FileWriterJc);

/*@DEFINE_C standard_includes @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**It should able to work with Stacktrace. */

//problems if include windows.h after this file: 
#define boolean bool


#include "Fwc/fw_LogMessage.h"
#include "Fwc/fw_Formatter.h"


#endif //__ObjectJc_h__
