/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 * @content Implementation of String processing
 *
 * @author JcHartmut www.vishia.org
 * @version 0.81
 * list of changes:
 * 2006-05-00: JcHartmut creation
 *
 ****************************************************************************/

//#include "CRuntimeJavalike.h"
#include "Jc/ObjectJc.h"
#include "Fwc/fw_Exception.h"
#include "Fwc/fw_Formatter.h" 
//#include "StringBufferJc.h"
#include "Jc/StringJc.h"
#include "Jc/ReflectionJc.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>  //using sprintf
#include <stdarg.h>
//#include "RuntimeHeapJc.h"



/**
  Strings are necessary in control applications for
  <ul><li>Assignment of warning and error messages with some data,<li>
      <li>some other small things,</li>
  </ul>
  but not to process large string problems. Thats while the maximum number of chars
  may be limited to less than 32000, typical there are 250 or 1000 chars.<br/>
  Strings may be:
  <ul><li>contant string literals, written in form "fix string",</li>
      <li>parts of a given string (from method String.substring)</li>
      <li>Strings in a buffer, managed by StringbufferJc.</li>
  </ul>
  In all this cases the string is identified with the pointer to the chars and the
  actual length. The C-convention: identification of a string by a zero-terminated
  array of chars, is not the matching solution, because only constant strings
  or the whole string in a buffer may be identified in this manner.<br>
  Because a string may be also a part of a content of StringBufferJc, the followed rule
  is found:
  <ul><li>If the string is a constant string, the pointer to the array of chars is stored
          in <code>value</code> and the number of chars is stored in <code>count</code>.
          The <code>offset</code> is set to <code>-1</code>.</li>
      <li>If the string is a part or the whole content of the buffer in StringBufferJc,
          The <code>value</code> pointered the start of buffer in the StringBufferJc.
          The adress of the StringBufferJc-Structur is ascertainable by subtraction of the
          difference between the Buffer inside a StringBufferJc and the StringBufferJc.
          The <code>offset</code>-Attribute is used to pointer the position inside buffer.
          </li>
  </ul>
  A StringBufferJc must known if a StringJc is referenced to it, by two causes:
  <ul><li>For garbage collection: A string should be handled like a reference.</li>
      <li>If a StringJc references the StringBufferJc and a change of the StringBufferJc
          will proceed, the StringBufferJc should copied in a new memory area and than modified,
          to held the content for pointering strings constant.</li>
  </ul>
  This requested knowledge is realized in StringBufferJc by using the array of back-references,
  see RuntimeHeapJc, and by query a flag in the StringBufferJc-attributes.
*/






StringBufferJc* ctorM_StringBufferJc(MemC rawMem)
{ StringBufferJc* ythis = PTR_MemC(rawMem, StringBufferJc);
  int size;
  char* buffer;
  STACKTRC_ENTRY("ctorM_StringBufferJc");
  size = size_MemC(rawMem) - sizeof(StringBufferJc);
  if(size < 20) THROW_s0(IllegalArgumentException, "ctor mem-size insufficient", size_MemC(rawMem));
  buffer = (char*)address_MemC(rawMem, sizeof(StringBufferJc), size);
  init0_MemC(rawMem);
  ctorc_s0i_StringBufferJc(ythis, buffer, size);
  return ythis;
}



StringBufferJc* ctorM_i_StringBufferJc(MemC mthis, int size, ThCxt* _thCxt)
{
  if(size_MemC(mthis) >= (sizeof(StringBufferJc) + size))
  { //enaugh size for StringBufferJc-control structure and the buffer itself:
    return ctorM_s0i_StringBufferJc(mthis, (char*)address_MemC(mthis, sizeof(StringBufferJc),0), size);
  }
  else
  { //not enaugh size:
    MemC mBuffer = alloc_MemC(size);
    init0_MemC(mBuffer);
    return ctorM_s0i_StringBufferJc(mthis, (char*)address_MemC(mBuffer, 0,0), size);
  }
}

/** 2008-12-24: written new without call of sub-ctor, with regarding value direct.
 */
StringBufferJc* ctorO_i_StringBufferJc(ObjectJc* othis, int size, ThCxt* _thCxt)
{ StringBufferJc* ythis = (StringBufferJc*)othis;
  int sizeObj;
  STACKTRC_TENTRY("ctorO_i_StringBufferJc");
  checkConsistence_ObjectJc(othis, sizeof(StringBufferJc), null/*&reflection_StringBufferJc*/, _thCxt); 
  sizeObj = getSizeInfo_ObjectJc(othis);
  ythis->count = 0;
  if(sizeObj >= (int)(sizeof(StringBufferJc) - sizeof(ythis->value) + size ))
  { //enaugh size for StringBufferJc-control structure and the buffer itself:
    ythis->size = size;  //positiv: use value as buffer direct
    memset(&ythis->value, 0, size);  
    //MemC mem = build_MemC(newObj, sizeObj);
    //return ctorc_s0i_StringBufferJc((StringBufferJc*)newObj, (char*)address_MemC(mem, sizeof(StringBufferJc),0), size);
  }
  else
  { //not enaugh size:
    MemC mBuffer = alloc_MemC(size);
    init0_MemC(mBuffer);
    ythis->size =-size;
    ythis->value.buffer = PTR_MemC(mBuffer, char);
  }
  STACKTRC_LEAVE; return ythis;
}



StringBufferJc* ctorO_StringBufferJc(ObjectJc* newObj, ThCxt* _thCxt)
{ int sizeObj = getSizeInfo_ObjectJc(newObj);
  MemC mem = build_MemC(newObj, sizeObj);
  int size = sizeObj - sizeof(StringBufferJc);  //the size is >=0 if checkConsistence don*t fail, but it may be 0.
  checkConsistence_ObjectJc(newObj, sizeof(StringBufferJc), null/*&reflection_StringBufferJc*/, _thCxt); 
  return ctorc_s0i_StringBufferJc((StringBufferJc*)newObj, (char*)address_MemC(mem, sizeof(StringBufferJc),0), size);
}



StringBufferJc* ctorO_S_StringBufferJc(ObjectJc* othis, StringJc src, ThCxt* _thCxt)
{ StringBufferJc* ythis = ctorO_StringBufferJc(othis, _thCxt);
  return append_s_StringBufferJc(ythis, src, _thCxt);
}



StringBufferJc* ctorM_s0i_StringBufferJc(MemC rawMem, char* buffer, int size)
{
  StringBufferJc* ythis = PTR_MemC(rawMem, StringBufferJc);
  STACKTRC_ENTRY("ctorM_ci_StringBufferJc");
  if(size_MemC(rawMem) < sizeof(StringBufferJc)) THROW_s0(IllegalArgumentException, "ctor mem-size insufficient", size_MemC(rawMem));
  init0p_MemC(ythis, sizeof(*ythis));
  return ctorc_s0i_StringBufferJc(ythis, buffer, size);

}

StringBufferJc* ctorO_ci_StringBufferJc(ObjectJc* yObj, char* buffer, int size, ThCxt* _thCxt)
{ StringBufferJc* ythis = (StringBufferJc*)yObj;
  STACKTRC_TENTRY("ctorO_ci_StringBufferJc");
  checkConsistence_ObjectJc(yObj, sizeof(StringBufferJc), null/*&reflection_StringBufferJc*/, _thCxt); 
  STACKTRC_LEAVE; return ctorc_s0i_StringBufferJc(ythis, buffer, size);

}

StringBufferJc* ctorc_s0i_StringBufferJc(StringBufferJc* ythis, char* buffer, int size)
{ if(ythis != null)
  { int count;
    ctorc_ObjectJc(&ythis->object);
    if(size < 0)
    { size = -size;
    }
    ythis->size = -size;  //negative to sign it is a refernce.
    count = strlen(buffer);  //from the given content
    if(count > size)
    { count = size; }        //but never greater than given size. The content may be a part of 0-terminate C-String.
    ythis->count= count;
    ythis->value.buffer = buffer;
  }
  return ythis;
}



/**reads the nrofChars and the reference into local variable. Static subroutine for inner organisation:*/
char* _getCharsAndSize_StringBufferJc(StringBufferJc* ythis, int* size)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  *size = (ythis->size < 0 ? -ythis->size : ythis->size);
  return( buffer);
}


char* _getCharsAndCount_StringBufferJc(StringBufferJc* ythis, int* count)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  *count = (ythis->count < 0 ? -ythis->count : ythis->count);
  return( buffer);
}

char* _getCharsSizeCount_StringBufferJc(StringBufferJc* ythis, int* size, int* count)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  *size = (ythis->size < 0 ? -ythis->size : ythis->size);
  *count = (ythis->count < 0 ? -ythis->count : ythis->count);
  return( buffer);
}


void _setCount_StringBufferJc(StringBufferJc* ythis, int count)
{ ythis->count = count;
}




void clone_String(ObjectJc* ythis)
{
}

void finalize_String(ObjectJc* ythis)
{
}

int32 hashCode_String(ObjectJc* ythis)
{
  return 0;
}



StringJc toString_String(StringJc* ythis)
{ return *ythis;
}




const char* getConstChar_StringJc(const StringJc* ythis)
{ //int count;
  //const char* sRet = getCharsAndCount_StringJc(ythis, &count);
  const char* sRet = _STRING_R2_StringJc(*ythis);
  return sRet;
}


/* Implementation notes: The functionallity of conversion from a given charset is ignored yet.
   Only the copiing of  bytes is done.
 */
StringJc new_bYiiCharset_StringJc(int8_Y volatile bytes, int offset, int length, StringJc charsetName, ThCxt* _thCxt)
{ StringBufferJc* sbuffer;
  STACKTRC_TENTRY("new_bYiiCharset_StringJc");
  sbuffer = new_StringBufferJc(length, _thCxt);
  { int zBuffer; char* buffer = _getCharsAndSize_StringBufferJc(sbuffer, &zBuffer);
    memcpy(buffer, bytes->data + offset, length);
  }
  STACKTRC_LEAVE; return toString_StringBufferJc(sbuffer);
}



StringJc new_cY_StringJc(char_Y chars, ThCxt* _thCxt)
{
  StringJc ret; init_s0i_StringJc(ret, chars->data, chars->length);
  return ret;

}




StringJc new_cYi_StringJc(char_Y chars, int offset, int count, ThCxt* _thCxt)
{
  StringJc ret; 
  STACKTRC_TENTRY("new_bYiiCharset_StringJc");
  if(offset < 0) THROW_s0(IndexOutOfBoundsException, "offset <0", offset);
  if(count < 0) THROW_s0(IndexOutOfBoundsException, "count <0", count);
  if(offset + count > chars->length) THROW_s0(IndexOutOfBoundsException, "offset + count > chars.length", offset + count);
  
  init_s0i_StringJc(ret, chars->data + offset, count);
  STACKTRC_LEAVE; return ret;

}





const char* toChar_StringJc(const StringJc* ref)
{ //int count;
  //const char* sRet = getCharsAndCount_StringJc(ref, &count);
  const char* sRet = _STRING_R2_StringJc(*ref);
  return sRet;
}


bool equals_StringJc(StringJc const ythis, StringJc const cmp)
{ int countThis = _LENGTH_R2_StringJc(ythis);
  int countCmp = _LENGTH_R2_StringJc(cmp);
  const char* strythis = _STRING_R2_StringJc(ythis);
  const char* strcmp = _STRING_R2_StringJc(cmp);

  if(countThis == mLength__StringJc) countThis = strlen(strythis);
  if(countCmp == mLength__StringJc) countCmp = strlen(strcmp);
  if(countCmp != countThis) return false;
  else return (memcmp(strythis, strcmp, countThis) == 0);
}

#if 0
bool equals_StringJc(StringJc const* ythis, StringJc const* cmp)
{ STACKTRC_ENTRY("equals_StringJc")
  int countThis;
  const char* sThis;
  int countCmp;
  const char* sCmp;
  sThis = getCharsAndCount_StringJc(ythis, &countThis);
  sCmp  = getCharsAndCount_StringJc(cmp  , &countCmp );
  if(countCmp != countThis) STACKTRC_LEAVE; return( false;
  else STACKTRC_LEAVE; return( (memcmp(sThis, sCmp, countThis) == 0);
}
#endif





//StringJc format_BV_StringJc(StringJc text, struct StringBufferJc_t* sbuffer, struct VaArgBuffer_t const* args)
StringJc format_BV_StringJc(StringJc text, struct StringBufferJc_t* sbuffer, va_list vargList)
{
  int zText; 
  char const* sText = getCharsAndLength_StringJc(&text, &zText);
  int nrofChars;
  char* buffer = (sbuffer->size < 0 ? sbuffer->value.buffer : sbuffer->value.direct);
  int zBuffer = (sbuffer->size < 0 ? -sbuffer->size : sbuffer->size);

  STACKTRC_ENTRY("format_BV_StringJc");
  nrofChars = format_va_arg_Formatter_FW(_thCxt, sText, zText, buffer, zBuffer, vargList);
  sbuffer->count = nrofChars;  
  STACKTRC_LEAVE; return s0i_StringJc(buffer, nrofChars);
}




/**This method tests whether a StringJc refers a StringBuffer already. If it is so, the StringBufferJc will use.
 * If it is not so, or the String is referenced from other sources too,
 * A new StringBuffer is created and the content of src is filled there.
 */
static StringBufferJc* transsubstantiateToStringBuffer_String(StringJc* ythis, ThCxt* _thCxt)
{ StringBufferJc* buffer;
  ObjectRefValuesJc refbase = value_OS_ValuePtr(*ythis);
  STACKTRC_TENTRY("transsubstantiateToStringBuffer_StringJc");
  if(refbase & mBuffered__StringJc)
  { //it is directly the begin of the StringBufferJc.
    buffer = (StringBufferJc*)((MemUnit*)(_STRING_R2_StringJc(*ythis)) - offsetChars_StringBufferJc);
  }
  else
  { int length = refbase & mLength__StringJc;
    const char* str = _STRING_R2_StringJc(*ythis);
    if(length == mLength__StringJc)
    { length = strlen(str);
    }
    { buffer = new_StringBufferJc(-1, _thCxt);  //allocate with standard size
      if(length > buffer->size) THROW_s0(IndexOutOfBoundsException,"string length", length);
      memcpy(buffer->value.direct, str, length);
      buffer->count = length;
      setFromBuffer_StringJc(ythis, buffer);
    }
  }
  STACKTRC_LEAVE; return( buffer);
}



StringJc replace_StringJc(StringJc ythis, char oldChar, char newChar, ThCxt* _thCxt)
{
  StringBufferJc* sbuffer = transsubstantiateToStringBuffer_String(&ythis, _thCxt);
  int size;
  char* buffer = _getCharsAndSize_StringBufferJc(sbuffer, &size);
  int ii;
  for(ii = 0; ii < sbuffer->count; ii++)
  { if(buffer[ii] == oldChar)
    { buffer[ii] = newChar;
    }
  }
  return toString_StringBufferJc(sbuffer);
}

METHOD_C void setLength_StringBufferJc(StringBufferJc* ythis, int newLength, ThCxt* _thCxt)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  STACKTRC_TENTRY("setLength_StringBufferJc");
  if(newLength < 0)
  { THROW_s0(IndexOutOfBoundsException, "", newLength);
  }
  else if(newLength < ythis->count)
  { memset(buffer + newLength, 0, ythis->count - newLength);
    ythis->count = newLength;
  }
  else if(newLength <= size)
  { memset(buffer + ythis->count, 0, newLength - ythis->count);
    ythis->count = newLength;
  }
  else
  { //the newLength increases the String with \0-chars
    THROW_s0(IndexOutOfBoundsException, "unrecognize", newLength);
  }
  STACKTRC_LEAVE;
}





void add_s0_StringJc(StringJc* ythis, const char* addP)
{ const StringJc add = s0_StringJc(addP);
  add_s_StringJc(ythis, add);
}


void add_s_StringJc(StringJc* ythis, StringJc add)
{ StringBufferJc* buffer;
  STACKTRC_ENTRY("add_s_StringJc");

  buffer = transsubstantiateToStringBuffer_String(ythis, _thCxt);
  append_s_StringBufferJc(buffer, add, _thCxt);
  _SETLENGTH_R2_StringJc(*ythis,  buffer->count);
  STACKTRC_LEAVE;
}


void add_i_StringJc(StringJc* ythis, int add)
{ StringBufferJc* buffer;
  STACKTRC_ENTRY("add_i_StringJc");

  buffer = transsubstantiateToStringBuffer_String(ythis, _thCxt);
  append_i_StringBufferJc(buffer, add, _thCxt);
  _SETLENGTH_R2_StringJc(*ythis,  buffer->count);
  STACKTRC_LEAVE;
}






char charAt_StringBufferJc(StringBufferJc* ythis, int index, ThCxt* _thCxt)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int count = ythis->count;
  STACKTRC_TENTRY("charAt_StringBufferJc");
  if(index < 0 || index >=count) THROW_s0(IndexOutOfBoundsException, "", index);
  STACKTRC_LEAVE; return buffer[index];
}




void setCharAt_StringBufferJc(StringBufferJc* ythis, int index, char ch, ThCxt* _thCxt)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  STACKTRC_TENTRY("setCharAt_StringBufferJc");

  if(index >= size || index <0)
  { //StringJc msg = s0_StringJc("StringBuffer to many chars");
    THROW_s0(IndexOutOfBoundsException, "index fault", index);
  }
  buffer[index] = ch;

  STACKTRC_LEAVE;
}



StringBufferJc* append_si_StringBufferJc(StringBufferJc* ythis, const char* add, int lengthAdd, ThCxt* _thCxt)
{ int nChars;
  char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  STACKTRC_TENTRY("append_s_StringBufferJc");

  if(lengthAdd < 0){ lengthAdd = strlen(add); }
  nChars = ythis->count + lengthAdd;
  if(nChars > size)
  { //StringJc msg = s0_StringJc("StringBuffer to many chars");
    THROW_s0(RuntimeException, "StringBuffer to many chars", nChars);
  }
  memcpy(buffer + ythis->count, add, lengthAdd);
  ythis->count = nChars;

  STACKTRC_LEAVE;
  return( ythis);
}

StringBufferJc* append_s0_StringBufferJc(StringBufferJc* ythis, const char* add, ThCxt* _thCxt)
{ STACKTRC_TENTRY("append_s0_StringBufferJc");
  append_si_StringBufferJc(ythis, add, -1, _thCxt);
  STACKTRC_LEAVE;
  return( ythis);
}

StringBufferJc* append_s_StringBufferJc(StringBufferJc* ythis, StringJc add, ThCxt* _thCxt)
{ int lengthAdd = _LENGTH_R2_StringJc(add);
  STACKTRC_TENTRY("append_s_StringBufferJc");
  if(lengthAdd == mLength__StringJc){ lengthAdd = -1; }
  append_si_StringBufferJc(ythis, _STRING_R2_StringJc(add), lengthAdd, _thCxt);
  STACKTRC_LEAVE;
  return( ythis);
}


StringBufferJc* append_Sb_StringBufferJc(StringBufferJc* ythis, StringBufferJc* add, ThCxt* _thCxt)
{ int lengthAdd;
  char* buffer; 
  STACKTRC_TENTRY("append_Sb_StringBufferJc");
  buffer = _getCharsAndCount_StringBufferJc(add, &lengthAdd);
  append_si_StringBufferJc(ythis, buffer, lengthAdd, _thCxt);
  STACKTRC_LEAVE;
  return( ythis);
}






StringBufferJc* append_c_StringBufferJc(StringBufferJc* ythis, char add, ThCxt* _thCxt)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  int nChars = ythis->count + 1;
  STACKTRC_TENTRY("append_c_StringBufferJc");
  if(nChars > size)
  { //StringJc msg = const_String("StringBuffer to many chars");
    THROW_s0(RuntimeException, "StringBuffer to many chars", nChars);
  }
  buffer[ythis->count] = add;
  ythis->count = nChars;

  STACKTRC_LEAVE;
  return( ythis);
}



StringBufferJc* replace_s0_StringBufferJc(StringBufferJc* ythis, int start, int end, const char* add, int nAdd, ThCxt* _thCxt)
{ int countNew;
  char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size); //size of the buffer
  int nInsert;  //nr of chars to insert netto
  STACKTRC_TENTRY("insert_s0i_StringBufferJc");
  if(nAdd < 0) { nAdd = strlen(add); }  //nr of chars of add
  nInsert = nAdd - (end - start); //nr of chars to insert netto, may be positive or negative.
  countNew = ythis->count + nInsert;
  if(countNew > size)
  { //StringJc msg = const_String("StringBuffer to many chars");
    //in opposite to Java the StringBuffer isnot increased, no dynamically memory management.
    //TODO perhaps test whether it is possible
    THROW_s0(RuntimeException, "StringBuffer to many chars", countNew);
  }
  { int nRest = ythis->count - end;  //nr of chars from end of replace area to actual end
    if(nRest > 0) memmove(buffer + end + nInsert, buffer + end, nRest);  //moves in both directions
  }
  //The area to replace is free.
  memcpy(buffer + start, add, nAdd);
  ythis->count = countNew;

  STACKTRC_LEAVE; return( ythis);
}



StringBufferJc* replace_StringBufferJc(StringBufferJc* ythis, int start, int end, StringJc add, ThCxt* _thCxt)
{
  int nAdd; 
  const char* sAdd = getCharsAndCount_StringJc(&add, &nAdd);
  return replace_s0_StringBufferJc(ythis,start, end, sAdd, nAdd, _thCxt); 
}

StringBufferJc* insert_c_StringBufferJc(StringBufferJc* ythis, int offset, char add, ThCxt* _thCxt)
{ return replace_s0_StringBufferJc(ythis, offset, offset, &add, 1, _thCxt);
}


/** found an algorithm unusing division, because some divisions may be expensive in time
  * at some processors.
*/
StringBufferJc* insert_i_StringBufferJc(StringBufferJc* ythis, int offset, int32 value, int radix, ThCxt* _thCxt)
{ char buffer[33];    //max 32 bits and negativ sign for "-10000000000000000000000000000000"
  StringBufferJc* ret;
  int nrofChars;
  STACKTRC_TENTRY("insert_i_StringBufferJc");
  nrofChars = toString_Integer_FW(buffer, sizeof(buffer), value, radix, 0, _thCxt);

  ret = insert_s0_StringBufferJc(ythis, offset, buffer, nrofChars, _thCxt);
  STACKTRC_LEAVE; return( ret);
}


/** found an algorithm unusing division, because some divisions may be expensive in time
  * at some processors.
*/
StringBufferJc* insert_i8_StringBufferJc(StringBufferJc* ythis, int offset, int64 value, int radix, ThCxt* _thCxt)
{ char buffer[65];    //max 32 bits and negativ sign for "-10000000000000000000000000000000"
  char cc;
  /**Array of values to test the position in digit. Fill it with 10, 100 etc if radix = 10;*/
  int64 testValues[64];

  int idxTestValues = 0;
  int nChars = 0;
  int64 test = radix;
  StringBufferJc* ret;
  STACKTRC_TENTRY("insert_i_StringBufferJc");
  if(value < 0)
  { value = -value; //may be -0x80000000
    buffer[nChars++] = '-';
  }
  //
  //fill in digigts to detect positions in digit
  while(value >= test)
  { testValues[idxTestValues++] = test;
    test *= radix;
  }
  //
  //test input digit
  while(--idxTestValues >=0)
  { cc = '0';
    test =testValues[idxTestValues];
    while(value >= test)
    { cc += 1;
      value -= test;
    }
    if(cc > '9')
    { cc += ('a' - '9' -1);
    }
    buffer[nChars++] = cc;
  }
  cc = (char)('0' + value); //last digit
  if(cc > '9')
  { cc += ('a' - '9' -1);
  }
  buffer[nChars++] = cc;
  ret = insert_s0_StringBufferJc(ythis, offset, buffer, nChars, _thCxt);
  STACKTRC_LEAVE; return( ret);
}


StringJc toString_DoubleJc(double value, ThCxt* _thCxt)
{ StringBufferJc* sbuffer;
  STACKTRC_TENTRY("toString_DoubleJc");
  { int size;
    char* buffer;
    sbuffer = new_StringBufferJc(-1, _thCxt);
    buffer = _getCharsAndSize_StringBufferJc(sbuffer, &size); 
    sbuffer->count = sprintf(buffer, "%d", value);
  }
  STACKTRC_LEAVE;
  return toString_StringBufferJc(sbuffer);
}




#if 0
StringBufferJc* append_i_StringBufferJc(StringBufferJc* ythis, int value)
{ STACKTRC_ENTRY("append_i_StringBufferJc");
  insert_i_StringBufferJc(ythis, ythis->count, value, 10, _thCxt);
  STACKTRC_LEAVE; return ythis;
}
#endif


StringBufferJc* insert_sii_StringBufferJc(StringBufferJc* ythis, int offset, StringJc add, int start, int end, ThCxt* _thCxt)
{ STACKTRC_TENTRY("insert_sii_StringBufferJc");
  { int nAdd; 
    const char* sAdd = getCharsAndCount_StringJc(&add, &nAdd);
    if(end < start || end > nAdd || start <0) THROW_s0(IndexOutOfBoundsException, "", end);
    sAdd += start;
    nAdd = end - start;
    replace_s0_StringBufferJc(ythis, offset, offset, sAdd, nAdd, _thCxt);
  }
  STACKTRC_LEAVE; return ythis;
}


/**The next routines are used to concatenate String */

StringBufferJc* s0BXJc(const char* value, ThCxt* _thCxt)
{ int len = strlen(value);
  StringBufferJc* ythis = new_StringBufferJc(len, _thCxt);
  insert_s0_StringBufferJc(ythis, 0 , value, len, _thCxt); 
  return ythis;
}

StringBufferJc* sBXJc(StringJc value, ThCxt* _thCxt)
{ return transsubstantiateToStringBuffer_String(&value, _thCxt);
}

StringBufferJc* bBXJc(StringBufferJc* value, ThCxt* _thCxt)
{
  return value;
}


StringBufferJc* s0XJc(StringBufferJc* ythis, const char* value, ThCxt* _thCxt)
{
  return insert_s0_StringBufferJc(ythis, ythis->count, value, -1, _thCxt);
}

StringBufferJc* sXJc(StringBufferJc* ythis, StringJc value, ThCxt* _thCxt)
{ int count;
  const char* chars = getCharsAndCount_StringJc(&value, &count); 
  return insert_s0_StringBufferJc(ythis, ythis->count, chars, count, _thCxt);
}

StringBufferJc* bXJc(StringBufferJc* ythis, StringBufferJc* value, ThCxt* _thCxt)
{ StringJc add = toString_StringBufferJc(value);
  int count;
  const char* chars = getCharsAndCount_StringJc(&add, &count); 
  return insert_s0_StringBufferJc(ythis, ythis->count, chars, count, _thCxt);
}

StringBufferJc* dXJc(StringBufferJc* ythis, double value, ThCxt* _thCxt)
{
  return insert_d_StringBufferJc(ythis, ythis->count, value, _thCxt);
}

StringBufferJc* fXJc(StringBufferJc* ythis, float value, ThCxt* _thCxt)
{
  return insert_f_StringBufferJc(ythis, ythis->count, value, _thCxt);
}

StringBufferJc* iXJc(StringBufferJc* ythis, int32 value, ThCxt* _thCxt)
{
  return insert_i_StringBufferJc(ythis, ythis->count, value, 10, _thCxt);
}


StringBufferJc* i8XJc(StringBufferJc* ythis, int64 value, ThCxt* _thCxt)
{
  return insert_i8_StringBufferJc(ythis, ythis->count, value, 10, _thCxt);
}







StringBufferJc* append_i_StringBufferJc(StringBufferJc* ythis, int value, ThCxt* _thCxt)
{ insert_i_StringBufferJc(ythis, ythis->count, value, 10, _thCxt);
  return ythis;
}

StringBufferJc* insert_f_StringBufferJc(StringBufferJc* ythis, int index, float value, ThCxt* _thCxt)
{ char buffer[20];
  int nChars;
  nChars = sprintf(buffer, "%f", value);
  return insert_s0_StringBufferJc(ythis, index, buffer, nChars, _thCxt);
}




StringBufferJc* insert_d_StringBufferJc(StringBufferJc* ythis, int index, double value, ThCxt* _thCxt)
{ char buffer[40];
  int nChars;
  nChars = sprintf(buffer, "%d", value);
  return insert_s0_StringBufferJc(ythis, index, buffer, nChars, _thCxt);
}





METHOD_C StringBufferJc* delete_StringBufferJc(StringBufferJc* ythis, int start, int end, ThCxt* _thCxt)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  STACKTRC_TENTRY("delete_StringBufferJc");
  if(end >= start && end <= size && start >=0)
  { int nrofBytesFromEndOfDeleteAreaToRealyEnd = ythis->count - end;
    int offset = end - start;
    if(offset > 0 && nrofBytesFromEndOfDeleteAreaToRealyEnd >0)
    { memmove(buffer + start, buffer + start + offset, nrofBytesFromEndOfDeleteAreaToRealyEnd);
      memset(buffer + start + nrofBytesFromEndOfDeleteAreaToRealyEnd, 0, offset);  //store 0 at rest to clean it.
    }
  }
  else
  { THROW_s0(StringIndexOutOfBoundsException, "", 0);
  }
  STACKTRC_LEAVE;
  return ythis;
}


METHOD_C void cleanToSize_StringBufferJc(StringBufferJc* ythis)
{ char* buffer = (ythis->size < 0 ? ythis->value.buffer : ythis->value.direct);
  int size = (ythis->size < 0 ? -ythis->size : ythis->size);
  if(ythis->count >= 0 && ythis->count <= size)
  { memset(buffer + ythis->count, 0, size - ythis->count);  //store 0 at rest to clean it.
  }
  else
  { STACKTRC_ENTRY("cleanToSize_StringBufferJc");
    THROW_s0(RuntimeException, "corrupt content, count=", ythis->count);
    STACKTRC_LEAVE;
  }
}
