#ifndef __MsgDispatcherCpp_MSG_h__
#define __MsgDispatcherCpp_MSG_h__


#include "Fwc/fw_LogMessage.h"

#include "MsgDispatcher_MSG.h"

/*@CLASS_C MsgDispatcher_MSG @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/**There is only one instance, singelton. The reference to it is stored globally. */ 
extern struct MsgDispatcher_MSG_t* instance_MsgDispatcher_MSG;

/**create the singelton. This method should called only one time at startup. */
struct MsgDispatcher_MSG_t* createSingleton_MsgDispatcher_MSG(int maxDispatchEntries, int maxQueue);





/**creates the Thread for output to File and Print.
 * @return >=0 if success.
 * @throws RuntimeException if any problem with createThread.
 */
int startSystem_MsgDispatcher_MSG(struct MsgDispatcher_MSG_t* ythis, int prioThread);


/**sets or removes the dispatching of output for a message with given ident, regarding levels. 
 * @param fromIdent first Identification of the Log or State message.
 * @param toIdent last Identfification of a range starting with fromIdent. If 0 than no range is used.
 * @param out The output instance. If null, than the message will not outputted. 
 *            If level=0, the output to the given destination will be resetted.
 * @param level Level of priority from 0 to 6. Depending of user settings a message will be outputted or not, 
 *              see Definitions xxx_Level_iLogMessageFW
 */
//int setOutput_MsgDispatcher_MSG(struct MsgDispatcher_MSG_t* ythis, int fromIdent, int toIdent, int32 maskBit, int level); 

/**sets a destination for output to dispatch to it, corresponding to maskBit in setOutput_LogStateCtrlFw. 
 * @param maskBit The mask bit with should select this output. 
 * @param out The output instance. If null, than the message will not outputted. 
 *            If level=0, the output to the given destination will be resetted.
 */
//int setOutputRoutine_MsgDispatcher_MSG(struct MsgDispatcher_MSG_t* ythis, int idx, LogMessageFW_i* out); 


//int writeProt_tv_MsgDispatcher_MSG(struct Message_ifcFW_t* ithis, int ident, const char* text, OS_TimeStruct time, struct Values_Entry_MsgDispatcher_MSG_t* args);

//MT_writeProt_tv_iLogMessageFW writeProt_tv_MsgDispatcher_MSG;


//MT_hexDump_Message_ifcFW hexDump_MsgDispatcher_MSG;


#if defined(__CPLUSGEN) && defined(__cplusplus)


class MsgDispatcher_MSG
: public iLogMessageFW          //the C++-Interface for output
{
  /**The data. There are non static, but stored in the static known singleton. 
   * The data are the same as instance_MsgDispatcher_MSG.
   */
  private: struct MsgDispatcher_MSG_t* const instanceData;

  
  //private: LogMessageFW_i* const instanceifcData;


  /**The singleton instance reference, private, use getInstance() to get it. */
  private: static MsgDispatcher_MSG* instance; 
  
  /**create the singelton. This method should called only one time at startup. */
  public: static MsgDispatcher_MSG* createSingleton(int maxDispatchEntries, int maxQueue);


  /**create a file output for logs or messages. 
   * @param sFilename should contain $MMdd_HHmmss$ or other date format, else the date will be written at first part of filename.
   */
  public: static LogMessageFW_i* createLogMessageFile(char const* sFilename, int32 nrofSecondsToFlush, int32 nrofHoursPerFile);


  /**gets the singelton. This method may be called every time, it is static, 
   * only a dependency to this class is necessary. 
   * But it is better to use the reference getted from createSingelton().
   */
  public: static MsgDispatcher_MSG* getInstance()
  { return instance;
  }

  /**private constructor to prevent multiple instances. */
  private: MsgDispatcher_MSG(struct MsgDispatcher_MSG_t* data);


  public: int startSystem(int prioThread){ return startSystem_MsgDispatcher_MSG(instanceData, prioThread); }

  public: virtual int setOutputRoutine(int32 idx, bool bQueued, iLogMessageFW* out)
  { setOutputRoutine_MsgDispatcher_MSG( instanceData, idx, bQueued, out->provide_LogMessageFW_i(), getCurrent_ThreadContextFW()); 
    return 0;
  }
  
  /**Sets the output from given text.
   * @param nrofChars if -1, than nrofChars = strlen(src) will be invoked.
   */
  public: void setOutputFromString(char const* src, int nrofChars)
  { setOutputFromString_MsgDispatcher_MSG(instanceData, s0i_StringJc(src, nrofChars), getCurrent_ThreadContextFW());
  }
   
  public: int setOutputRange(int fromIdent, int toIdent, int32 mask, int mode, int level)
  { return setOutputRange_MsgDispatcher_MSG( instanceData, fromIdent, toIdent, mask, mode, level, getCurrent_ThreadContextFW()); 
  }

  public: virtual iLogMessageFW* provide_iLogMessageFW()
  { return this; }

  public: virtual int hexDump(int ident, void* pBuffer, int length, int bytesPerWord)
  { return 0; //hexDump_MsgDispatcher_MSG( instanceifcData, ident, pBuffer, length, bytesPerWord); 
  }
    
  public: virtual int writeProt(int ident, const char* text, ...);

  public: virtual int writeState(int ident, bool state, const char* text, ...)
  { return -1; 
  }

  public: virtual LogMessageFW_i* provide_LogMessageFW_i(){ return &instanceData->super; } 

};

#endif //__cplusplus

#endif // __MsgDispatcherCpp_MSG_h__
