/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content Log Message at console. 
 *
 * @author Hartmut Schorrig
 * @version sf-0.83
 * list of changes:
 * 2008-12-01: Hartmut implementation of the LogMessage-Concept
 *
 ************************************************************************************************/


//include the own header at first to test its correct dependencies.
#include "Fwc/fw_LogMessage.h"

#include "Fwc/fw_Exception.h"
#include "Jc/ReflectionJc.h"
#include "Jc/StringJc.h"
//#include "fw_basic.h"
#include <string.h>
#include "Fwc/fw_Formatter.h"



typedef struct MsgOutputFile_MSG_t
{
  /**The C-like-Interfaces. 
   * NOTE: More as one are a unit, because there content are congruent
   *       and they have to be start at begin of struct because casting.
   */  
  union
  { /**The data based on object, necessary for check. */
    ObjectJc object;
    /**C-Interface-struct for Log-Output. */
    LogMessageFW_i logStateFw;
  } ifc;

  //Mtbl_LogMessageFW const* mtbl;

  /**The name of the file. The maximal nrof chars are limited. The String is zero-terminated. */
  StringBufferJc sFilename; char sFilename_addChars[120];

  /**The format of timestamp in the name. It is zero-terminated. */
  StringJc sFormatTimestampFilename; char charsFormatTimestampFilename[32];

  /**The position in file name where the timestamp are placed. */
  int posTimestampInFilename;

  /**The file to output the log directly at file system. */
  FILE* file;

  /**Time of the last file open. A file should be closed after 1 hour. */
  OS_TimeStruct timeFileOpen;
  
  /**Temporary buffer for 1 line, used inside run. It contains the last outputted line. */
  char buffer[1000];


}MsgOutputFile_MSG_s;

extern int const sizeof_MsgOutputFile_MSG;
int const sizeof_MsgOutputFile_MSG = sizeof(MsgOutputFile_MSG_s);


/*@INTERNAL_PROTOTYPES MsgOutputFile_MSG ************************************************************/
/**Check wether the instanceof is correct, return the type-correct reference. */

METHOD_C MsgOutputFile_MSG_s* castO_MsgOutputFile_MSG(ObjectJc* zthis);


extern ClassJc const reflection_MsgOutputFile_MSG;



//MsgOutputFile_MSG_s* 
LogMessageFW_i* ctorO_LogMessageFile(ObjectJc* othis
, StringJc sFileName
, int nrofSecondsToFlush
, int nrofHoursPerFile
)
{
  extern ClassJc const reflection_MsgOutputFile_MSG;
  MsgOutputFile_MSG_s* ythis = (MsgOutputFile_MSG_s*)othis;
  STACKTRC_ENTRY("ctorO_LogMessageFile");
  if(othis == null)
  { //create it if null as instance
    othis = alloc_ObjectJc(sizeof(MsgOutputFile_MSG_s), 0, _thCxt);
    ythis = (MsgOutputFile_MSG_s*)othis;
  }
  ctorc_ObjectJc(othis);
  setReflection_ObjectJc(othis, &reflection_MsgOutputFile_MSG, 0);
  ythis->file = null;
  //The size of this Object blankets both instances, they are one after another in struct:
  init_ObjectJc(&ythis->sFilename.object, sizeof(ythis->sFilename) + sizeof(ythis->sFilename_addChars), 0);
  ctorO_i_StringBufferJc(&ythis->sFilename.object, sizeof(ythis->sFilename_addChars), _thCxt);
  { int pos2TimestampInFilename;
    StringJc sTimestampInFilename = {0};
    ythis->posTimestampInFilename = indexOf_StringJc(sFileName, '$');
    if(ythis->posTimestampInFilename >=0)
    { pos2TimestampInFilename = indexOf_i_StringJc(sFileName, '$', ythis->posTimestampInFilename+1);
      if(pos2TimestampInFilename >0)
      { sTimestampInFilename = substring_StringJc(sFileName, ythis->posTimestampInFilename+1, pos2TimestampInFilename, _thCxt);
        copyToBuffer_StringJc(sTimestampInFilename, ythis->charsFormatTimestampFilename, sizeof(ythis->charsFormatTimestampFilename)); 
        append_s_StringBufferJc(&ythis->sFilename, substring_StringJc(sFileName, 0, ythis->posTimestampInFilename, _thCxt), _thCxt);
      }
      else THROW_s0(IllegalArgumentException, "second $ to delimit timestamp in filename missing.", ythis->posTimestampInFilename);
    }
    else
    { //no timestamp in filename given, use default: Timestamp on start of filename.
      ythis->posTimestampInFilename = 0;
      pos2TimestampInFilename = -1;  //postfix of filename starts at 0
      strncpy(ythis->charsFormatTimestampFilename, "MMdd_HHmm", sizeof(ythis->charsFormatTimestampFilename));
    }
    append_s0_StringBufferJc(&ythis->sFilename, ythis->charsFormatTimestampFilename, _thCxt);
    append_s_StringBufferJc(&ythis->sFilename, substring_i_StringJc(sFileName, pos2TimestampInFilename+1, _thCxt), _thCxt);
  }

  STACKTRC_LEAVE; return &ythis->ifc.logStateFw;
}



LogMessageFW_i* create_MsgOutputFile_MSG(const char* sFileName)
{ LogMessageFW_i* outFile;
  STACKTRC_ENTRY("createLogOutputFile");
  { ObjectJc* ofile = alloc_ObjectJc(sizeof(MsgOutputFile_MSG_s), 0, _thCxt);
    outFile = ctorO_LogMessageFile(ofile, s0_StringJc(sFileName), 0, 1); 
  }
  STACKTRC_LEAVE; return outFile;

}





MsgOutputFile_MSG_s* castO_MsgOutputFile_MSG(ObjectJc* zthis)
{ //TODO check type via reflection, fast algorithm because used in any interface method.
  MsgOutputFile_MSG_s* ythis = (MsgOutputFile_MSG_s*)zthis;
  ASSERT(instanceof_ObjectJc(zthis, &reflection_MsgOutputFile_MSG));
  //ASSERT(ythis->mtbl == &mtbl_MsgDispatcher_MSG);  //an derivated class isnot accepted! 
  return ythis;
}







/**Write to file. This routine should only be called in the MsgDispatcher_MSG.run()-Method.
 */ 
void sendMsgArgBuffer_MsgOutputFileMSG
( struct LogMessageFW_t* ithis
, int32 identNumber
, OS_TimeStamp const timestamp
, StringJc const text
, va_list args
//, struct VaArgBuffer_t const* const args
, ThCxt* _thCxt
)
//int writeFile_MsgDispatcher_MSG(struct Message_ifcFW_t* zthis, int ident, const char* text, OS_TimeStruct time, Values_Entry_MsgDispatcher_MSG_s* values)
{ 
  MsgOutputFile_MSG_s* ythis = castO_MsgOutputFile_MSG(&ithis->object); //(MsgDispatcher_MSG_s*)zthis;
  int posBuffer;
  //os_setCurrentThreadContext(_threadContext);
  //int lengthBuffer;
  char* buffer2;
  STACKTRC_TENTRY("writeFile_MsgDispatcher_MSG");
  
  if(ythis->file != null)
  { //file is opened
    OS_TimeStruct timeTestClose;
    int secDiff;
    timeTestClose = os_getDateTime();
    secDiff = timeTestClose.time_sec - ythis->timeFileOpen.time_sec;
    if(secDiff >= 3600)
    { if(ythis->file != null)
      { printf("\nclose log-file\n");
        fclose(ythis->file);
        ythis->file = null;   //it would be opened if necessary as a new file. 
      }
    }
  }
  if(ythis->file == null)
  { //file open because data are to write:
    ythis->timeFileOpen = os_getDateTime();
    strcpy(ythis->buffer, "..\\Data\\LogState");
    posBuffer = strlen(ythis->buffer); 
    posBuffer += toStringFormat_Fw(ythis->buffer + posBuffer, sizeof(ythis->buffer) - posBuffer, &ythis->timeFileOpen, "yyMMdd_hhmmss", 0);
    strcpy(ythis->buffer + posBuffer, ".log");
    ythis->file = fopen(ythis->buffer, "wb");
    if(ythis->file == null)
    { printf("\nerror open Log-file %s\n", ythis->buffer);
    }
    else
    { printf("\nopen Log-file %s\n", ythis->buffer);
    }
  }
  { int maxBuffer = sizeof(ythis->buffer);
    posBuffer = toStringFormat_Fw(ythis->buffer, maxBuffer, &timestamp, "\nYY-MM-dd-hh:mm:ss.SSS: ", 2);
    buffer2 = ythis->buffer + posBuffer;
    maxBuffer -= posBuffer;

    posBuffer = sprintf(buffer2, "%5.5i :", identNumber);
    buffer2 += posBuffer; 
    maxBuffer -= posBuffer;

    posBuffer = _prepareOutputText_MsgDispatcher_MSG(buffer2, maxBuffer, identNumber, text, timestamp, args); 
  }
  //printf(ythis->buffer);

  if(ythis->file != null)
  { fprintf(ythis->file, ythis->buffer);
    fflush(ythis->file);
  }

  STACKTRC_LEAVE;
}

/*@REFLECTION MsgOutputFile_MSG ************************************************************/


/**To organize dynamic link method call the jump table of virtual methods is neccessary. */
struct Mtbl_MsgOutputFile_MSG_t
{ Mtbl_ObjectJc ObjectJc;  //same method types as ObjectJc
  Mtbl_LogMessageFW LogMessage_ifcFW;
} const mtbl_MsgOutputFile
=
{ { clone_Object_F
  , equals_Object_F
  , finalize_Object_F
  , hashCode_Object_F
  , toString_Object_F
  }
, { &sign_Mtbl_LogMessageFW
  , sendMsgArgBuffer_MsgOutputFileMSG
  }
};


extern ClassJc const reflection_LogMessageFW_i;

static struct interfaces_MsgOutputFile_MSG_t
{ /** Base data of every array */
  ObjectArrayJc head;
  ClassOffset_idxMtblJc data[1];
}interfaces_MsgOutputFile_MSG =
{ CONST_ObjectArrayJc(ClassOffset_idxMtblJc, 1, OBJTYPE_ClassOffset_idxMtblJc, null, null)
, { {&reflection_LogMessageFW_i, OFFSET_Mtbl(struct Mtbl_MsgOutputFile_MSG_t, LogMessage_ifcFW) }
  }
};


   
ClassJc const reflection_MsgOutputFile_MSG =
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &reflection_MsgOutputFile_MSG, null)
, "MsgOutputFile_MSG"
, 0     //posObjectJc
, sizeof(MsgOutputFile_MSG_s)
, null  //attributes and associations
, null  //method
, null  //superclass
, (ClassOffset_idxMtblJcARRAY*)&interfaces_MsgOutputFile_MSG  //interfaces
, 0x4000000  //modifiers
, &mtbl_MsgOutputFile.ObjectJc
};


