/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content Declaration of mutex routines.
 * The implementation is os-specific. A mutex should regard a second call from the same thread.
 * It is more complex as a simple semaphore request. 
 * Some operation systems supports this request immediately. 
 * If a opeation system does know only the semaphore mechanism, the thread identification
 *   should be stored in a mutex structure.
 *
 * @author Hartmut Schorrig
 * @version sf-0.83
 * list of changes:
 * 2008-02-01: Hartmut creation 
 *
 ************************************************************************************************/
#ifndef __os_mutex_h__
#define __os_mutex_h__
#include <os_types_def.h>

/**Creates a mutex object.
 * @param pName A textual identification for the operation system.
 * @param pMutexId return identification for the mutex object.
 *        The data of the mutex are saved in the os internally, not in user space.
 * @return 0 if ok, negativ value on error.
 */
int os_createMutex(char* pName, uint* pMutexID);
 
 
/**Deletes a mutex object.
 * @param mutexID The identification of the mutex object, which is returned from os_createMutex(...).
 * @return 0 if ok, negativ value on error.
 */
int os_deleteMutex(uint mutexID);
 
 
/**Lockes a mutex object.
 * If the same thread has locked this mutex object already, the routine call should be non-blocked,
 * The return value should be 0 (it is successfull).
 *
 * If another thread has locked this mutex object, the calling thread blockes. 
 * The calling thread should be resume, if the other thread releases this mutex object.
 *
 * If more as one thread waits on the same mutex, only one thread (may me with the highest priority)
 * is to resume. All other threads should wait furthermore.
 * 
 *
 * @param mutexID The identification of the mutex object, which is returned from os_createMutex(...).
 * @return 0 if ok, negativ value on error.
 */
int os_lockMutex(uint mutexID);


/**Unlockes a mutex object.
 * 
 * If another thread waits for this mutex object, that thread should be resumed, but with locking this mutex.
 * If more as one thread waits for this mutex object, only the thread with the highest priority should be resumed. 
 *
 * @param mutexID The identification of the mutex object, which is returned from os_createMutex(...).
 * @return 0 if ok, negativ value on error.
 */
int os_unlockMutex(uint mutexID);


#endif  // __os_mutex_h__
