/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content Definition of memory access routines. 
 * This routines are implemented in a operation system specific way. 
 *
 * @author Hartmut Schorrig
 * @version sf-0.83
 * list of changes:
 * 2008-02-01: Hartmut creation 
 *
 ************************************************************************************************/
#ifndef __os_thread_h__
#define __os_thread_h__

#include <os_types_def.h>



// Default stack size for tasks.
#define OS_DEFAULT_STACK      8000

// Small stack size for tasks.
#define OS_SMALL_STACK        2000

// High priority for tasks.
#define OS_PRIORITY_CTRL      200

// High priority for tasks.
#define OS_PRIORITY_HIGH      100

// Normal priority for tasks.
#define OS_PRIORITY_NORMAL    80

// Low priority for tasks.
#define OS_PRIORITY_LOW       40

// Priority for timer tasks.
#define OS_PRIORITY_TIMER     20





//NOTE: Diese Dinge mssten besser in einem os-spezifischen Header definiert werden. Was ist, wenn die Zuordnung nicht passt??
typedef struct OS_HandleThread_t const* OS_HandleThread;

/**The thread routine should have the followed prototype.
 * @param data Pointer to data. Mostly it is a class structure. 
 *        It is the same value given in createThread, param pUserData.
 * @param _thC The ThreadContext structure. 
 * @return The exit value of the thread. The operation system can protocol or show the exit value in a special kind.
 */
typedef int OS_ThreadRoutine(void* data);





// init adapter
int os_initLib();

/**Returns the os-specific value for a thread priority.
 *
 * @param abstractPrio A value between 1 and 255. 1 is the lowest priority, 255 the highest one.
 * @return associated priority of the os.
 */
long os_getRealThreadPriority(unsigned int abstractPrio);


/**Creates and starts a thread.
 * The thread will be end if the thread routine returns.
 * @param pHandle reference to a numerical identificator of the thread.
 *        Note: all internal management data of the thread are stored os-internally.
 * @param routine Pointer to the thread routine, which is started.
 * @param pUserData This value will be transported to the first parameter of the user routine.
 *        Typically it are the user instance data of the thread method.
 * @param sThreadName A textual identification for the operation system.
 * @param abstractPrio Priority between 1 (lowest) and 255 (highest).
 *        Note: The opeation system may assign different abstract priorities 
 *        to the same real priority. It is not guaranteed, that such threads have different priorities. 
 * @param stackSize The necessary size of stack. If 0,than the default stack size is used.
 *        Note: Possible stack sizes depends on possibilities of the operation system.
 *        It is a good style that the stack size should be in range of 2..16 kBytes, 
 *        the user shouldn't placed large data in stack.
 *        Note, that interrupt handlers uses the stack also. 
 * @return 0 if ok, negativ value on error.
 */ 
int os_createThread
( OS_HandleThread* pHandle
, OS_ThreadRoutine routine
, void* pUserData
, char const* sThreadName
, uint abstactPrio
, uint stackSize
);

/**Changes the thread priority.
 */
int os_setThreadPriority(OS_HandleThread handle, uint abstractPrio);


//int os_suspendThread(OS_HandleThread handle);


//int os_resumeThread(OS_HandleThread handle);


/**Gets the current thread handle.
 */
OS_HandleThread os_getCurrentThreadHandle(void);


METHOD_C OS_ValuePtr os_getCurrentUserThreadContext();

METHOD_C int os_setCurrentUserThreadContext(OS_ValuePtr mem);



/**Stops the execution of the executable because no error handling is possible.
 * This routine should only called in normally safe, but unexpected situations
 * at system level. At user level the routine shouldn't called.
 */
void os_fatalError(const char* text, int value);

#endif //__os_thread_h__
