/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content routines to handle wait and notify (thread interactions). 
 *
 * @author Hartmut Schorrig
 * @version sf-0.83
 * list of changes:
 * 2008-02-01: Hartmut creation 
 *
 ************************************************************************************************/
#ifndef __os_waitnotify_h__
#define __os_waitnotify_h__

#include <os_types_def.h>


/**The type of a OS_HandleWaitNotify is represented by a pointer to a not far defined struct. */ 
typedef struct OS_HandleWaitNotify_t const* OS_HandleWaitNotify;

/**creates a object for wait-notify Object. The inner type of such an Object is os-dependent.
 * Here only a forward declared struct pointer is knwon.
 * @return 0 if no error, or an error code.
 */
METHOD_C int os_createWaitNotifyObject(struct OS_HandleWaitNotify_t const** waitObject);


/**removes a object for wait-notify.
 */
METHOD_C int os_removeWaitNotifyObject(struct OS_HandleWaitNotify_t const* idxWaitObject);





/** Waits for a notification.
 */

METHOD_C int os_wait(struct OS_HandleWaitNotify_t const* waitObject, uint32 milliseconds);


/** Notifies all waiting thread to continue.
 * @return 0 if successfull notified,
             >0 if notified with warning (possible notified but nobody waits).
             <0 if an system error occurs. This should not occur in a tested system.
 */
METHOD_C int os_notifyAll(struct OS_HandleWaitNotify_t const* waitObject);


/** Notifies only one waiting thread to continue.
 */
METHOD_C int os_notify(struct OS_HandleWaitNotify_t const* waitObject);




#endif //__os_waitnotify_h__
