/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content Bodies of mutex routines for Windows-XP and Windows-2000.
 * The implementation is os-specific. A mutex should regard a second call from the same thread.
 * It is more complex as a simple semaphore request. 
 * Some operation systems supports this request immediately. 
 * If a opeation system does know only the semaphore mechanism, the thread identification
 *   should be stored in a mutex structure.
 *
 * @author Hartmut Schorrig
 * @version sf-0.83
 * list of changes:
 * 2008-02-01: Hartmut creation 
 *
 ************************************************************************************************/
#include "OSAL/inc/os_mutex.h"
#include "OSAL/inc/os_error.h"
#include <windows.h>
#include <stdio.h>

/* CONSTANTS *************************************************************************************/


/* GLOBAL VARIABLES ******************************************************************************/


/* EXTERNALS *************************************************************************************/


/* PROTOTYPES ************************************************************************************/


int os_createMutex(char* pName, uint* pMutexID)
{

    HANDLE    hIOMutex;
    DWORD err;
    
    hIOMutex = CreateMutex (NULL, FALSE, NULL);  // initially not owned
    if ( hIOMutex == NULL ){
		err = GetLastError();
        printf("os_createMutex: ERROR: create mutex failed with Win err= %d\n", err );
        *pMutexID = 0;
		return OS_SYSTEM_ERROR;
    }
	*pMutexID  = (uint)hIOMutex;
    return OS_OK;
}


int os_deleteMutex(uint mutexID)
{
    if ( CloseHandle( (HANDLE)mutexID ) == 0 ) {
    	DWORD err = GetLastError();
        printf("os_deleteMutex: ERROR: CloseHandle failed with Win err=%d\n", err );
		if (err==ERROR_INVALID_HANDLE){
			return OS_INVALID_HANDLE;
		}
		return OS_SYSTEM_ERROR;
    }
    return OS_OK; 
}


int os_lockMutex(uint mutexID)
{
	DWORD WinRet;
    	
    WinRet = WaitForSingleObject( (HANDLE)mutexID, INFINITE );
    if (WinRet == WAIT_FAILED){
    		DWORD err = GetLastError();
        printf( "os_waitMutex: ERROR: WaitForSingleObject failed with Win err=%d\n", err );
 			if (err==ERROR_INVALID_HANDLE){
			    return OS_INVALID_HANDLE;
			}
			return OS_SYSTEM_ERROR;
		}
		
//		if (WinRet==WAIT_ABANDONED){ // in case another thread terminated and did not release the mutex
//			return OS_SYSTEM_ERROR;
//		}
    return OS_OK; 
}


int os_unlockMutex(uint mutexID)
{
    
    if ( ReleaseMutex( (HANDLE)mutexID ) == 0 ){
       	DWORD err = GetLastError();
		if (err!=ERROR_NOT_OWNER){ // if mutex was already unlocked, it will not be an error.
			printf("os_signalMutex: ERROR: ReleaseMutex failed with Win err=%d\n", err );		
		    if (err==ERROR_INVALID_HANDLE){
			    return OS_INVALID_HANDLE;
		    }
			return OS_SYSTEM_ERROR;
		}	  
    }
    return OS_OK;
}





#if 0
int os_createMutex(struct OS_HandleMutex_t const** handle)
{ CRITICAL_SECTION* semaphore = (CRITICAL_SECTION*)malloc(sizeof(CRITICAL_SECTION));
  InitializeCriticalSection(semaphore);
  *handle = (struct OS_HandleMutex_t const*)semaphore;
  return 0;
}


int os_entryMutex(struct OS_HandleMutex_t const* handle)
{
  CRITICAL_SECTION* semphore = (CRITICAL_SECTION*)handle;
  EnterCriticalSection(semphore);
  return 0;
}


int os_releaseMutex(struct OS_HandleMutex_t const* handle)
{
  CRITICAL_SECTION* semphore = (CRITICAL_SECTION*)handle;
  LeaveCriticalSection(semphore); 
  return 0;
}

#endif

