/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content This file contains the implementation of os_waitnotify.h for MS-Windows.
 * @author Jchartmut www.vishia.org
 * @version 0.81d
 * list of changes:
 * 2007-10-01: JcHartmut creation
 *
 ****************************************************************************/


#include "OSAL/inc/os_thread.h"

#undef BOOL
#undef PBOOL
#undef boolean

#include <wtypes.h>
#include <Winbase.h>



int os_createWaitNotifyObject(struct OS_HandleWaitNotify_t const** waitObject)
{ //folg. Mechanismus ist nicht verfgbar unter Win2000, wre aber richtiger:
  //PCONDITION_VARIABLE var;
  //InitializeConditionVariable(&var);
  HANDLE semaphor;
  int error = 0;

  semaphor = CreateSemaphore
  ( null /*LPSECURITY_ATTRIBUTES lpSemaphoreAttributes,*/
  , 0    /*__in      LONG lInitialCount, */
  , 1    /*__in      LONG lMaximumCount, */
  , null /*__in_opt  LPCTSTR lpName */
  );
  if(semaphor==null)
  { error = GetLastError();
  }
  else
  { *waitObject = (struct OS_HandleWaitNotify_t*)semaphor;
  }
  return error;
}


/**removes a object for wait-notify.
 */
int os_removeWaitNotifyObject(struct OS_HandleWaitNotify_t const* handle)
{

  return null;
}


int os_wait(struct OS_HandleWaitNotify_t const* handle, uint32 milliseconds)
{ HANDLE semaphor = (HANDLE)handle;
  int error = WaitForSingleObject(semaphor, milliseconds);
  if(error == WAIT_FAILED)
  { error = GetLastError();
  }
  else
  { error = 0;
  }
  return error;
}


/** Notifies all waiting thread to continue.
 */
int os_notifyAll(struct OS_HandleWaitNotify_t const* handle)
{
  return -1;

}


/** Notifies only one waiting thread to continue.
 */
int os_notify(struct OS_HandleWaitNotify_t const* handle)
{ HANDLE semaphor = (HANDLE)handle;
  LONG prevCount;
  int error;
  //see http://msdn.microsoft.com/en-us/library/ms685071(VS.85).aspx
  int ok = ReleaseSemaphore(semaphor, 1, &prevCount);
  if(ok == 0) 
  { //If the function fails, the return value is zero. To get extended error information, call GetLastError.
    error = -((int)GetLastError()); 
    if(error == -ERROR_TOO_MANY_POSTS)
    { //warning
      error = 1;
    }
  }
  else 
  { //If the function succeeds, the return value is nonzero
    error = 0; 
  }
  return error;
}
