/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content Definition of basicly types adequate to the operation system, cpu and compiler.
 *
 * @author Hartmut Schorrig
 *************************************************************************************************/

#ifndef   __os_types_def_h__
#define   __os_types_def_h__


#ifndef __cplusplus
  //If C-compiling is used, define the C++-keywords for C
  #define bool int
  #define false 0
  #define true ~0
#endif


#ifndef FALSE
	#define FALSE	((BOOL)0)           ///< Define FALSE: Note: type of BOOL!
#endif

#ifndef TRUE
	#define TRUE	((BOOL)(!FALSE))   ///< Define TRUE
#endif

#ifndef NULL
	#define NULL	0                   ///< Define NULL
#endif

//This functions may be platform dependend but for all our platforms identical, also in C-Standard.
//do nut use platform specific headers. 

#define FW_NTOH16(x) ((u16) ((((u16)(x) & 0x00ff) << 8) | \
                             (((u16)(x) & 0xff00) >> 8)))

#define FW_HTON16(x) FW_NTOH16(x)

#define FW_NTOH32(x) ((u32) ((((u32)(x) & 0x000000ffU) << 24) | \
			                       (((u32)(x) & 0x0000ff00U) <<  8) | \
			                       (((u32)(x) & 0x00ff0000U) >>  8) | \
			                       (((u32)(x) & 0xff000000U) >> 24)))

#define FW_HTON32(x) FW_NTOH32(x)


// Folgender Schalter ist gesetzt zur Auswahl der Betriebssystemplattform Windows ist. Damit knnen Betriebssystemzugriffe bedingt compiliert werden.
#define __OS_IS_WINDOWS__

// Folgender Schalter ist gesetzt zur Auswahl des Compilers MSC6. Damit knnen spezifische Compilereigenschaften mittels bedingter Compilierung bercksichtigt werden.
#define __COMPILER_IS_MSC6__

// Folgender Schalter ist gesetzt zur Auswahl der C++-Compilierung. In einigen Quellen kann es einen Unterschied geben, wenn eine C-Abbildung der Quellen
// bercksichtigt werden muss. Das standardgem definierte Makro __cplusplus ist nicht immer fr dieses Zweck verwendbar, da auch der Einsatz der C++-Compilierung
// fr eine C-Abbildung erfolgen kann. Wenn die Gesamt-Source (beispielsweise ein Headerfile) gegebenenfalls auch als C-Abbildung
// compiliert werden muss, dann mssen alle Teile, die nur unter C++ lauffhig sind, mit diesem Schalter bedingt compiliert werden.
#define __CPLUSGEN

#define MemUnit char            //sizeof(MemUnit) muss 1 sein!
#define BYTE_IN_memUnit 1       //im PC gilt: 1 MemUnit = 1 Byte

typedef unsigned char        uint8;
typedef unsigned short       uint16;
typedef unsigned long        uint32;

typedef signed char          int8;
typedef short                int16;
typedef long                 int32;
typedef __int64              int64;

/**Die Definition spezieller Typen fr variable Argumentlisten ist daher notwendig, 
 * weil der GNU-Compiler bei variablen Argumenten int8 und int16 auf int promoted.
 * Bei Visual Studio ist das aber nicht so.
 * Verwendung nur in va_arg(..,TYP)-Makro.
 */
typedef signed char          int8va;
typedef short                int16va;


typedef float                float32;

//plattformunabhaengige Ergaenzungen
//folgende Typen sind besser schreib- und lesbar
#define ushort unsigned short int
#define uint unsigned int
#define ulong unsigned long int

//NULL soll nach wie vor fuer einen 0-Zeiger verwendet werden duerfen.
//Hinweis: (void*)(0) kann nicht einem typisiertem Zeiger zugewiesen werden, wohl aber 0
#undef  NULL
#define NULL 0
#undef null
#define null 0

// Folgendes Define wird nach einer Struktur insbesondere fr GNU-Compiler verwendet. Es ist fr MSC6 leer,
// weil stattdessen ein pragma pack(1) verwendet werden muss.
#define GNU_PACKED

// Das Makro METHOD_C soll fr C++ regeln, ob eine Methode/Routine/Funktion ggf. auch in C++-Umgebungen nach den C-Konventionen definiert ist.
// Fr eine reine C-Compilierung ist das Makro leer. Dann erscheint beim Linken nicht das Label nach C- sondern nach C++-Konventionen.
// Beide Varianten knnen zweckmig sein. Hier ist die C++-Labeling-Variante gewhlt.   
#ifdef __cplusplus
  //#define METHOD_C extern "C"
  #define METHOD_C
#else
  #define METHOD_C
#endif

/**Ein EventHandle ist unter Windows ein HANDLE und dort als void* definiert. 
 * Der Anwender soll aber keine windows.h und hnliches einziehen sollen (zuviel Mll), 
 * daher hier die Zurckfhrung auf die Schnittmenge zwischen windows.h und dem Rest der Welt.
 */
#define OS_HandleEvent void*   

/**This class provides basic working with memory allocation and its associated amount of bytes.
 * The Problem in C is: a Pointer to memory is not interpretable. It is a void* mostly.
 * How many bytes are there, it is unknown.
 *
 * The struct MemJc contains both:
 * * The pointer to the data as pointer to memory layout.
 * * The size of data in memory.
 * But there is no type info.
 */
typedef struct OS_ValuePtr_t
{ int32 value__;
  char* ptr__;
  //struct MemAreaC_t* ptr__;
  //union{ struct MemAreaC_t* memArea; char* str; } ptr__;
}OS_ValuePtr;




#define value_OS_ValuePtr(THIS) ((THIS).value__)

#define PTR_OS_ValuePtr(THIS, TYPE) ((TYPE*)(THIS).ptr__)

#define set_OS_ValuePtr(THIS, INT, PTR) { (THIS).value__ = (INT); (THIS).ptr__ = (char* )(PTR); }

#define setValue_OS_ValuePtr(THIS, INT) { (THIS).value__ = (INT); }



#endif  //__os_types_def_h__
