package org.vishia.bridgeC;


/**This class represents a buffer, which stores variable arguments in the kind of C language:
 * 
 * @author JcHartmut
 *
 */
public class VaArgBuffer
{
  private final Object[] args;

  public final int length;
  
  private final Va_list va_list;
  
  public VaArgBuffer(int size)
  { this.length = size;
    args = new Object[size];
    va_list = new Va_list(this);
  }
  
  private VaArgBuffer(Object[] src)
  { args = src;
    length = src.length;
    va_list = new Va_list(this);
  }
  
  /**Copies values from a source variable argument list. 
   * In C a variable argument list is only a byte sequence in stack, without any description.
   * The meaning of the bytes is only known from the evaluating routine, at example printf(...).
   * In Java a variable argument list is an array of instances.
   * 
   * @param inputVaArgs
   */
  public void copyFrom(Va_list input)  //Object... inputVaArgs)
  { int len = input.buffer.length;
    if(len > length)
    { throw new ArrayIndexOutOfBoundsException("too many arguments");
      //len = length;
    }
    for(int idx = 0; idx < len; idx++)
    { args[idx] = input.buffer.args[idx];
    }
  }
  
  
  
  public static VaArgBuffer represent(Object... inputArgs)
  {
    VaArgBuffer argBuffer = new VaArgBuffer(inputArgs);
    return argBuffer;
  }
  
  
  public Va_list get_va_list()
  { return va_list;
  }
  
  
  
  /**Cleans the values in Buffer. 
   * For java-programming this action cleans up unnecessary references, the associated objects may be freed.
   * For C-programming this action cleans up the storage to prevent errors. This maybe also an effect in Java.
   */
  public void clean()
  {
    for(int idx = 0; idx < length; idx++)
    { args[idx] = null;
    }
  }
  
  public Object[] get(){ return args; }
}
