package org.vishia.util;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

/**This class is a alternative to java.io.FileWriter. 
 * It based on java.util.OutputStreamWriter, adequate to java.io.FileWriter.
 * But in opposite to that, 
 * <ul>
 * <li>The instance of this class will be preserved, if the file is close, 
 *   respectively the instance will be created staticly, without open file.
 *   It is because using in fast realtime systems in C via Java2C-translator.
 *   The implementation in C is written without any dynamically memory.
 * <li>On open() no exceptions are thrown, a boolean return value is used.  
 * </ul>  
 * @author Hartmut Schorrig
 *
 */
public class FileWriter
{
  
  OutputStreamWriter writer;
  
  boolean open(String fileName)
  { boolean bOk = true;
    try
    { writer = new OutputStreamWriter(new FileOutputStream(fileName)); 
    }
    catch(IOException exc)
    { bOk = false;
    }
    return bOk;
  }

  
  boolean isOpen()
  { return writer != null;
  }
  
  void close() throws IOException{ writer.close(); writer = null; }
  
  void write(String text) throws IOException{ writer.write(text);}
  
  void flush() throws IOException{ writer.flush(); }
  
}
