package org.vishia.util;

import java.text.ParseException;
import java.util.Arrays;
//import java.util.Date;

import org.vishia.bridgeC.ConcurrentLinkedQueue;
import org.vishia.bridgeC.MemC;
import org.vishia.bridgeC.OS_TimeStamp;
import org.vishia.bridgeC.VaArgBuffer;
import org.vishia.bridgeC.Va_list;
import org.vishia.mainCmd.LogMessageConsole;



public class MsgDispatcher extends LogMessage
{

  /**If this bit is set in the bitmask for dispatching, the dispatching should be done 
   * in the dispatcher Thread. In the calling thread the message is stored in a queue. */
  public final static int mDispatchInDispatcherThread = 0x80000000;

  /**If this bit is set in the bitmask for dispatching, the dispatching should only be done 
   * in the calling thread. It is possible that also the bit {@link mDispatchInDispatcherThread}
   * is set, if there is more as one destination. */
  public final static int mDispatchInCallingThread =    0x40000000;

  /**If this bit is set, not a bit mask is used for dispatching, but a index.
   * TODO to dispatch a message to only one destination, but a greater number of several destinations.
   */
  public final static int mDispatchWithIndex = 0x20000000;
  
  /**Only this bits are used to indicate the destination via some Bits*/
  public final static int mDispatchBits =  0x07FFFFFF;
  
  /**Number of Bits in {@link mDispatchWithBits}, it is the number of destinations dispached via bit mask. */
  private final static int kDispatchWithBits = 28;
  
  public final static int mConsole = 0x01;
  public final static int mConsoleQueued = 0x04;
  public final static int mFile = 0x02;
  public final static int mFileQueued = 0x08;
  
  /**Used for argument mode from {@link #setOutputRange(int, int, int, int, int)} to add an output.
   * The other set outputs aren't change. 
   */
  public final static int mAdd = 0xcadd;
  
  /**Used for argument mode from {@link #setOutputRange(int, int, int, int, int)} to set an output.
   * Outputs before are removed. 
   */
  public final static int mSet = 0xc5ed;
  
  /**Used for argument mode from {@link #setOutputRange(int, int, int, int, int)} to remove an output.
   * All other outputs aren't change.
   */
  public final static int mRemove = 0xcde1;
  
  
  
  
  private static final class Entry
  {

    
    private static final class Values
    { /**Store values. @xxxjava2c=simpleArray. */
      final int[] v = new int[11];
    }
    
    /**Bit31 is set if the state is coming, 0 if it is going. */
    int ident;
     
    int dst;
  
    String text;
  
    OS_TimeStamp timestamp;

    /**Values from variable argument list of writeProt(ident, text, ...). */
    VaArgBuffer values = new VaArgBuffer(11);
    
    //final Values values = new Values();  //NOTE: 16*4 = 64 byte per entry. NOTE: only a struct is taken with *values as whole.

    public static int _sizeof(){ return 1; } //it is a dummy in Java.
    
  }
  
  /**This class contains some test-counts for debugging. It is a own class because structuring of attributes. 
   * @xxxjava2c=noObject.  //NOTE: ctor without ObjectJc not implemented yet.
   */
  private static final class TestCnt
  {
    private int noOutput;
  }
  
  /**This class contains all infomations for a output. There is an array of this type in MsgDispatcher. 
   * @java2c=noObject.
   */
  private static final class Output
  {
    /**Short name of the destination, used for {@link #setOutputRange } or {@link #setOutputFromString }
     * @xxxjava2c=simpleArray.
     */
    private String name;
    
    /**The output interface. */
    private LogMessage outputIfc;

    /**true if this output is processed in the dispatcher thread, 
     * false if the output is called immediately in the calling thread.
     */ 
    private boolean dstInDispatcherThread;
  
    
  }
  
  final TestCnt testCnt = new TestCnt();
  
  /**List of messages to process in the dispatcher thread.
   * @java2c=noGC.
   */
  final ConcurrentLinkedQueue<Entry> listOrders;
  
  /**List of entries for messages to use.
   * @java2c=noGC.
   */
  final ConcurrentLinkedQueue<Entry> freeOrders;
  
  
  /**List of idents, its current length. */ 
  private int actNrofListIdents;

  /**List of idents, a array with lengthListIdents elements.
   * @java2c=noGC.
   */
  private int[] listIdents;  //NOTE: It is a pointer to an array struct.

  /**List of destination bits for the idents.
   * @java2c=noGC.
   */
  private int[] listBitDst;  //NOTE: It is a pointer to an array struct.

  /**up to 30 destinations for output.
   * @java2c=noGC,embeddedArrayElements.
   */
  private Output[] outputs;
  //private LogMessage[] outputs = new LogMessage[28];
  
  public final LogMessage outputConsole = LogMessageConsole.create();
  
  //private boolean[] dstInDispatcherThread = new boolean[28];
  
  private int maxDst = 0;

      
  
  /**Initializes the instance.
   * @param maxDispatchEntries
   * @param maxQueue
   * @param maxOutputs
   * @param nrofMixedOutputs
   */
  public MsgDispatcher(int maxDispatchEntries, int maxQueue, int maxOutputs, int nrofMixedOutputs)
  { //Construct Linkedqueue:
    
    /**@java2c = embeddedArrayElements. */
    Entry[] entries = new Entry[maxQueue];
    for(int idxEntry = 0; idxEntry < entries.length; idxEntry++)
    { entries[idxEntry] = new Entry();
    }
    int idxEntry;
    MemC mNodes = MemC.alloc((maxQueue +2) * Entry._sizeof());
    this.freeOrders = new ConcurrentLinkedQueue<Entry>(mNodes);
    this.listOrders = new ConcurrentLinkedQueue<Entry>(this.freeOrders);
    for(idxEntry = 0; idxEntry < maxQueue; idxEntry++)
    { this.freeOrders.add(entries[idxEntry]);
    }
    { this.listIdents = new int[maxDispatchEntries];
      this.listBitDst = new int[maxDispatchEntries];
      this.listIdents[0] = 0;
      this.listIdents[1] = Integer.MAX_VALUE;
      this.listBitDst[0] = 0;
      this.listBitDst[1] = 0xFFFFFFFF;  //this value will be unused because MAX_VALUE don't be used.
      this.actNrofListIdents = 2;
    }
    /*allocate the output array: */
    outputs = new Output[maxOutputs];
    for(int idxDst = 0; idxDst < maxOutputs; idxDst++){ outputs[idxDst] = new Output();}
    
    /**Set default output to console. */
    setOutputRoutine(0, "CON", false, outputConsole);  //mConsole
    setOutputRoutine(1, "CON-queued", true, outputConsole);   //mConsoleQueued
    setOutputRange(0, Integer.MAX_VALUE, mConsole, MsgDispatcher.mSet, 3);
  }
  
  
  public void setDefaults(String fileOut)
  { setOutputRoutine(0, "CON", false, outputConsole);  //mConsole
    setOutputRoutine(2, "CON-queued", true, outputConsole);   //mConsoleQueued
    //LogMessage outputFile = new LogMessageFile(fileOut);
    //setOutputRoutine(1, false, outputFile);     //mFile
    setOutputRange(0, Integer.MAX_VALUE, mConsole, MsgDispatcher.mSet, 3);
  }
  
  
  private int searchDispatchBits(int ident)
  { int bitDst;
    int idx = Arrays.binarySearch(listIdents, 0, actNrofListIdents, ident);
    if(idx < 0) idx = -idx -2;  //example: nr between idx=2 and 3 returns -4, converted to 2
    if(idx < 0) idx = 0;        //if nr before idx = 0, use properties of msg nr=0
    bitDst = listBitDst[idx];     
    return bitDst;
  }

  
  
  
  

  /**inserts an ident range after given position:
   * <pre> 1 means any stored ident number, 7 means a number behind,
   * i,j means the inserted idents. The idx selects the 1.-ident.
   * before: ....1.7......
   * after:  ....1.i.j.7..</pre>
   * If fromIdent == toIdent, only one position is inserted:
   * <pre>
   * before: ....1.7....
   * after:  ....1.j.7..
   * @param idx Index, after them the ident is inserted.
   * @param fromIdent first value of the ident.
   * @param toIdent second value of the ident.
   * @param mask bit to or
   */
  private int insertIdent(int idx, int fromIdent, int toIdent)
  { final int ident0 = listIdents[idx];
    final int ident3 = listIdents[idx+1];
    final int idxFrom;  //idx where fromIdent is set
    if(ident0 < fromIdent && toIdent+1 < ident3)
    { idxFrom = idx+1;
      actNrofListIdents +=2;
      //copy after idx, src from idx+1, because the content at idx is okay. 
      System.arraycopy(listIdents, idx+1, listIdents, idx+3, actNrofListIdents - (idx+3));
      System.arraycopy(listBitDst, idx+1, listBitDst, idx+3, actNrofListIdents - (idx+3));
      listIdents[idxFrom] = fromIdent;
      listIdents[idxFrom+1] = toIdent+1;
      listBitDst[idxFrom] = listBitDst[idxFrom+1] = listBitDst[idx];
      //listBitDst[idxFrom] |= mask;
    }
    else if(ident0 < fromIdent)
    { assert(toIdent+1 == ident3);
      idxFrom = idx+1;
      actNrofListIdents +=1;
      System.arraycopy(listIdents, idx+1, listIdents, idx+2, actNrofListIdents - (idx+2));
      System.arraycopy(listBitDst, idx+1, listBitDst, idx+2, actNrofListIdents - (idx+2));
      listIdents[idxFrom] = fromIdent;
      listBitDst[idxFrom] =  listBitDst[idx];
      //listBitDst[idxFrom] |= mask; 
    }
    else if(ident0 == fromIdent && toIdent+1 < ident3)
    { actNrofListIdents +=1;
      idxFrom = idx;
      System.arraycopy(listIdents, idx+1, listIdents, idx+2, actNrofListIdents - (idx+2));
      System.arraycopy(listBitDst, idx+1, listBitDst, idx+2, actNrofListIdents - (idx+2));
      listIdents[idx+1] = toIdent+1;
      //listBitDst[idx] |= mask; 
    }
    else
    { assert(ident0 == fromIdent && toIdent+1 == ident3);
      idxFrom = idx;
      //replace only
      //listBitDst[idx] |= mask; 
    }
    return idxFrom;
  }
  

  
  
  
  /**Sets a destination interface to a index for dispatching.
   * @param dstIdx The index 0..28 or greater, which is associated to the destination.
   * @param bQueued true than the message is queued if this destination is used.
   * @param outputs The destination for message or log output.
   */
  public void setOutputRoutine(int dstIdx, String name, boolean bQueued, LogMessage dst)
  { int mask;  
    if(dstIdx > kDispatchWithBits)
    { mask = mDispatchWithIndex | dstIdx;
      if(maxDst <= dstIdx){ maxDst= dstIdx+1; }
    }
    else
    { mask = 1 << dstIdx;
    }
    if(bQueued)
    { mask |= mDispatchInDispatcherThread;
    }
    else
    { mask |= mDispatchInCallingThread;
    }
    this.outputs[dstIdx].outputIfc = dst;
    this.outputs[dstIdx].dstInDispatcherThread = bQueued;
    int zName = name.length();
    this.outputs[dstIdx].name = name;
    //if(zName > this.outputs[0].name.length)
    //{ zName = this.outputs[0].name.length; //shorten it.
    //}
    //name.getChars(0, zName, this.outputs[dstIdx].name, 0);
  }
  
  
  /**Sets the output dispatch bits for the given message number range.
   * @param fromIdent The first ident number for which the dispatch bit is valid.
   * @param toIdent The last ident number for which the dispatch bit is valid.
   * @param dst Bit or index for the destination for the given output range.
   *            An or-value {@link #mDispatchWithIndex} can be added. If this Bit is set,
   *            the rest of bits are the index of the output, greater as {@link #kDispatchWithBits}.
   *            In this case mode should be {@link #mSet}.
   *            <br>
   *            If the bit {@link #mDispatchWithIndex} isn't set, the bits 0..23 describe one or more outputs.
   *            <br>
   *            If the bit combination of dst isn't valid, an invalidArgumentException is thrown.
   * @param mode Ones of {@link #mAdd}, {@link #mSet}, {@link #mRemove} to presribe,
   *             what to do with dst.
   * @param level
   * @return
   */
  public int setOutputRange(final int fromIdent, int toIdent, final int dst, final int mode, final int level)
  { /**Found index where the idents are arranged in . */
    int idx1, idx2;
    final int maskSet = completeDispatchInThreadBits(dst); 
    if(toIdent == Integer.MAX_VALUE)
    { //the range is from ...inclusive to, let the MAX_VALUE outside, because there should be and end entry,
      toIdent = Integer.MAX_VALUE -1; 
    }
    idx1 = Arrays.binarySearch(listIdents, 0, actNrofListIdents, fromIdent);
    if(idx1 < 0) idx1 = -idx1 -2;  //example: nr between idx=2 and 3 returns -4, converted to 2
    if(idx1 < 0) idx1 = 0;        //if nr before idx = 0, use properties of msg nr=0
    //at idx1 : ident0  <= fromIdent
    
    int ident3 = listIdents[idx1+1];  //the ident after the fromIdent-area
    if(toIdent < ident3)
    { //the whole range fromIdent...toIdent is at or between idx1...idx1+1 
      int idxMask = insertIdent(idx1, fromIdent, toIdent);
      listBitDst[idxMask] |= maskSet;
    }
    else
    { int ident1 = listIdents[idx1+1];  
      int idx1Mask = insertIdent(idx1, fromIdent, ident1-1);  //inserts after idx1, if necessary
      
      idx2 = Arrays.binarySearch(listIdents, 0, actNrofListIdents, toIdent+1);
      if(idx2 < 0) idx2 = -idx2 -2;  //example: nr between idx=2 and 3 returns -4, converted to 2
      
      //at idx2 : ident3 <= toIdent //<= ident3
      ident3 = listIdents[idx2+1];
      int mask3 = listBitDst[idx2]; 
      int idx2Mask = insertIdent(idx2, toIdent+1, ident3-1);    //inserts before idx2+1, if necessary
      for(int imask = idx1Mask; imask < idx2Mask; imask++)
      { listBitDst[imask] |= maskSet;
      }
    }
    return 0;
  }

  
  
  /**Sets the output from a String content.
   * Syntax-Example:
   * <pre>
   * 123..512:0x27  //It is a bit mask, all messages from 123 to inclusive 521 are set to channels 0,1,2 and 5  
   * 1001:+CON.      //Message 1001 is sent additional to CON
   * </pre>
   * @param ctrl The control String.
   * @return null if all ok, otherwise an error. 
   */
  public String setOutputFromString(String ctrl, StringBuffer errorBuffer)
  {
    String sError = null;
    /** @java2c=stackInstance.*/
    StringPart spCtrl = new StringPart(ctrl);
    spCtrl.setIgnoreWhitespaces(true);
    boolean cont = true;
    try
    { do
      { int fromIdent, toIdent;
        if(spCtrl.seekNoWhitespaceOrComments().length() == 0)
        {
          cont = false;  //finish.
        }
        else if(spCtrl.scanInteger().scanOk())
        { fromIdent = (int)spCtrl.getLastScannedIntegerNumber();
          if(spCtrl.scan("..").scanInteger().scanOk())
          { toIdent = (int)spCtrl.getLastScannedIntegerNumber();
          }
          else 
          { toIdent = fromIdent;  
          }
          if(!spCtrl.scan(":").scanOk())
          { sError = "\":\" missing";
          }
          else
          {
            if(spCtrl.scanHexOrDecimal(8).scanOk())
            { int dst = (int)spCtrl.getLastScannedIntegerNumber();
              setOutputRange(fromIdent, toIdent, dst, mSet, 3);
              if(!spCtrl.scan(";").scanOk())
              { sError = "\";\" missing after number-output.";
              }
            }
            else
            { boolean bCont = true;
              do
              { int mode = mSet;
                String sOutput = null;
                if(spCtrl.scan("+").scanOk()){ mode = mAdd; }
                else if(spCtrl.scan("-").scanOk()){ mode = mRemove; }
                else if(spCtrl.scan(";").scanOk())
                { bCont = false; 
                }
                else
                { if(mode != mSet)
                  { sError = "\"+\" or \"-\" or \";\" expected";
                  }
                }
                if(bCont && sError == null)
                { if(spCtrl.scanIdentifier().scanOk())
                  { sOutput = spCtrl.getLastScannedString();
                  }
                  else
                  { sError = "\"DST\" expected, DST should be an identifier.";
                  }
                  if(sError == null)
                  { assert(sOutput != null);
                    int bitOutput = -1;
                    //search the ident
                    int idxDst = 0; 
                    while(idxDst < outputs.length && bitOutput == -1)
                    { if(outputs[idxDst].name != null && outputs[idxDst].name.equals(sOutput))
                      { bitOutput = 1<<idxDst;
                      }
                      else
                      { idxDst++; 
                      }
                    }
                    if(bitOutput == -1)
                    { sError = "Output not found";
                    }
                    else
                    {
                      setOutputRange(fromIdent, toIdent, bitOutput, mode, 3);
                      mode = -1;  //other mode as mSet expected.
                    }
                  }
                }  
              }while(sError == null && bCont);
            }//if hexInt else ident;
          }//: scanned
        }
        else
        { sError = "Number for first message to dispatch expected";
          cont = false;  
        }
             
      } while(cont && sError == null);
    }
    catch(ParseException exc)
    { sError = exc.getMessage(); 
    }
    errorBuffer.setLength(0);
    if(sError != null)
    { /*Write the error in the buffer, with fault input string position. */
      errorBuffer.append(sError).append(" at:").append(spCtrl.getCurrent(30));
    }
    return sError; 
  }
  
  
  
  /**Completes a destination bit mask with the information, whether any destinations are used
   * in the calling thread or in the dispatcher thread. 
   * @param dstBits The input mask
   * @return returns the same bits as in input mask, 
   *         but with Bits {@link mDispatchInDispatcherThread} and/or {@link mDispatchInCallingThread} 
   */
  int completeDispatchInThreadBits(int dstBits)
  { int dstBitRet = dstBits;
    assert(dstBits > 0);
    int idst = 0;
    while(dstBits != 0) //abort if no bits are set anymore.
    { if(  (dstBits & 1)!=0 )
      if( outputs[idst].dstInDispatcherThread )
      { dstBitRet |= mDispatchInDispatcherThread; 
      }
      else
      { dstBitRet |= mDispatchInCallingThread; 
      }
      dstBits >>=1;
      idst += 1;
    }
    return dstBitRet;
  }
  








  
  /**Dispatches a message. This routine is called either in the calling thread of the message
   * or in the dispatcher thread. 
   * @param dstBits Destination identificator. If the bit {@link mDispatchInDispatcherThread} is set,
   *        the dispatching should be done only for a destination if the destination is valid
   *        for the dispatcher thread. 
   *        Elsewhere if the bit is 0, the dispatching should be done only for a destination 
   *        if the destination is valid for the calling thread.
   * @param identNumber identification of the message.
   * @param creationTime
   * @param text
   * @param args @java2c=nonPersistent.
   */
  private void dispatchMsg(int dstBits, int identNumber, OS_TimeStamp creationTime, String text, Va_list args)
  { final boolean bDispatchInDispatcherThread = (dstBits & mDispatchInDispatcherThread)!=0;
    //assert, that dstBits is positive, because >>=1 and 0-test fails elsewhere.
    //The highest Bit has an extra meaning, also extract above.
    dstBits &= mDispatchBits;  
    int idst = 0;
    while(dstBits != 0) //abort if no bits are set anymore.
    { if(  (dstBits & 1)!=0 
        && outputs[idst].dstInDispatcherThread == bDispatchInDispatcherThread  //dispatch in the requested thread
        )
      { LogMessage out = outputs[idst].outputIfc;
        if(out != null)
        { out.sendMsgVaList(identNumber, creationTime, text, args);
        }
        else
        { testCnt.noOutput +=1;
        }
      }
      dstBits >>=1;
      idst += 1;
    }  
  }


  @Override
  public void sendMsgVaList(int identNumber, OS_TimeStamp creationTime, String text, Va_list args)
  {
    // TODO Auto-generated method stub
    int dstBits = searchDispatchBits(identNumber);
    if(dstBits != 0)
    { if((dstBits & mDispatchInDispatcherThread) == 0)
      { //dispatch in this thread:
        //VaArgBuffer argBuffer = VaArgBuffer.represent(va_args); //new VaArgBuffer(11); //@java2c=instack
        //argBuffer.copyFrom(args);
        dispatchMsg(dstBits, identNumber, creationTime, text, args);
        //dispatchMsg(dstBits, identNumber, creationTime, text, va_args);
      }
      else
      { //store in queue, dispatch in common thread.
        Entry entry = freeOrders.poll();  //get a new Entry from static data store
        if(entry == null)
        { //overflow, the message can't be displayed
          //test if it is a important message
        }
        else
        { entry.dst = dstBits;
          entry.ident = identNumber;
          entry.text = text;
          entry.timestamp = creationTime;
          //int maxValues = entry.values.length;
          //int maxValues = entry.values.v.length;
          //int maxInputValues = args.length;
          //int idxValue;
          /*
          for(idxValue = 0; idxValue < maxValues && idxValue < maxInputValues; idxValue++)
          { entry.values.v[idxValue] = args[idxValue];
          }
          */
          entry.values.copyFrom(args);
          listOrders.offer(entry);
        }
      }
    }
  }
  
  /**Dispatches all messages, which are stored in the queue. 
   * This routine should be called in one user thread.
   * @return number of messages which are found to dispatch, for statistic use.
   */
  public int dispatchQueuedMsg()
  { int nrofFoundMsg = 0;
    boolean bCont;
    do
    { Entry entry = listOrders.poll();
      bCont = (entry != null);
      if(bCont)
      { nrofFoundMsg +=1;
        dispatchMsg(entry.dst, entry.ident, entry.timestamp, entry.text, entry.values.get_va_list());
        entry.values.clean();
        /*
        int maxValues = entry.values.v.length;
        for(int idxValue = 0; idxValue < maxValues; idxValue++)
        { entry.values.v[idxValue] = 0;
        }
        */
        entry.ident = 0;  
        freeOrders.offer(entry);
      }
      
    } while(bCont);
    return nrofFoundMsg;
  }
  
  
  
  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  void stop(){}






}
