package org.vishia.java2C;

import org.vishia.java2C.SecondPass;

/**This class contains no code, only documentation. Every spanned subject of the translator 
 * is described associated to a method. The description contains some links to the java code.
 * So javadoc generates a well readable documentation. Use the private-view documentation
 * to trace to all links, because most of methods and fields are package private or private.  
 * 
 * @author JcHartmut
 *
 */
public class Docu
{
  
  /**While the translation processes the type or affiliation of some identifiers should be evaluated. 
   * In java this is done by the javac-Compiler. In C some more informations should be appropriated
   * to the C-Compiler. Therefore all identifiers are stored in Lists. This lists are TreeMap mostly
   * to support a fast searching.
   * <br>
   * <img src="../../../../img/ClassDataLocalIdents_omd.jpg" />
   * <br><br>
   * <b>LocalIdents</b>
   * <br>
   * The identifiers are valid always in a local context. This context may be a class context, 
   * a method context or a context of a statement block (some statements in <code>{...}</code>). 
   * The class {@link LocalIdents} holds references to identifiers and types,
   * with some add- and access methods, for all the contexts:
   * <ul> 
   * <li>{@link SecondPass.StatementBlock#localIdents}: for any statement block 
   * <li>{@link ClassData#classLevelIdents} for the class level context
   * <li>{@link Java2C_Main#userTypes} for all classes in the Java2C-translated sources: 
   * <li>{@link Java2C_Main#stdTypes} for all existing simple types and types, which are defined
   *     in the CRuntimeJavalike-library: .
   * </ul>
   * <br>
   * The {@link LocalIdents} contains TreeMaps for 
   * <ul>
   * <li>{@link LocalIdents#fieldIdents} identifier of all variables in this context. 
   *     Especially in statement block some variable may be defined, they aren't known in the class context.
   * <li>{@link LocalIdents#typeIdents} identifier of all types in this context. 
   *     In a statement block local visible classes are possible too.   
   * </ul>
   * The searching of a identifier uses always the Java name of identifier. 
   * In the found {@link FieldData} than the C name is contained.
   * <br><br>
   * <b>Identifiers of super levels</b>
   * <br>
   * Super levels are the levels above a statement block; 
   * <ul>
   * <li>the parent statement block, 
   * <li>the whole method,
   * <li>the class
   * <li>the super class and the outer class
   * <li>the super.super class etc, the outer.outer class etc.
   * <li>All global visible identifiers: {@link Java2C_Main#userTypes} and {@link Java2C_Main#stdTypes} 
   * </ul>
   * In generally, there are two ways to search identifiers of super levels:
   * <ol>
   * <li>An instance of {@link LocalIdents} contains only the identifiers of the current level. 
   *     It contains a reference to the super and outer LocalIdents. Than the searching process is lengthy,
   *     if a identifier isn't found locally, it should be searched in the next super and outer level and so on.
   * <li>An instance of {@link LocalIdents} contains all identifiers visible at this level. 
   *     If a new instance of LocalIdents is created, the identifiers of the parent level should be copied
   *     and designated with its context oriented from the current level. This copy process necessitates 
   *     some calculation time, but only one time per new level. Some more memory spaces is needed, but only temporary
   *     (the garbage collector have to be tidy up). But the searching is simple and fast.       
   * </ol>
   * The solution is: 
   * <ul>
   * <li>fields, variables are contained complete in the current {@link LocalIdents#fieldIdents}.
   *   If a new level of statement block is created, first the reference to the {@link LocalIdents} 
   *   of the super level is set. But if any block-local variable is found 
   *   (calling {@link GenerateClass#gen_variableDefinition(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents, java.util.List, char)}),
   *   a new instance of {@link LocalIdents} is created and all fieldIdent of the parent are copied into it.
   *   So any instance of {@link LocalIdents} contains all visible identifier of variables and class fields.
   *   The association is stored in the {@link FieldData}.
   * <li>Methods and Types are stored only in the accordingly instance of {@link LocalIdents}. 
   *   If types are searched, all {@link LocalIdent#parent} should be tested 
   *   until the type identifier is found.
   * </ul>     
   * <br>
   * <br>
   * <b>{@link Java2C_Main#userTypes}</b>
   * <br>
   * That are global visible identifiers. A Instance of named {@link LocalIdents} for that global visible identifier
   * is contained in the singleton {@link Java2C_Main}-class static. The lists of this LocalIdents,
   * espespecially {@link LocalIdents#typeIdents} are filled while running 
   * {@link FirstPass#runFirstPass(String, org.vishia.zbnf.ZbnfParseResultItem, String, ClassData, java.util.List, LocalIdents)}
   * while translating a user class. (In the older versions of Java2C this list were contained in an extra TreeMap allClassData).
   * <br>
   * <br> 
   * <b>{@link Java2C_Main#stdTypes}</b>
   * <br>
   * That are all types given from Language and given from the CRuntimeJavalike basic system. A Instance of named {@link LocalIdents} for that global visible identifier
   * is contained in the singleton {@link Java2C_Main}-class static. The lists of this LocalIdents,
   * espespecially {@link LocalIdents#typeIdents} are filled in the constructor of {@link Java2C_Main} 
   * respectively in the ctor of {@link Java2C_Main.CRuntimeJavalikeClassData}.
   * <br>
   * <br>
   * <b>Searching a type identifier</b>
   * <br>
   * If an identifier is given, which may be a type or not, it can search using 
   * {@link LocalIdents#getType(String)}. 
   * The search process starts at the current level (the local types) and continues at super levels, if the type isn't found.
   * Global types such int, String are found in the global levels.
   * <br>
   * Global types may be referenced directly using {@link Java2C_Main#stdTypes}. 
   * This is the same LocalIdents-instance like used as parent of the local levels.
   * <br>
   * If a identifier is given, which is a type identifier, and it should be exists or it should be
   * generated yet, the routine {@link GenerateClass#getType(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents)}
   * is called. This routine calls inside {@link LocalIdents#getType(String)},
   * but starts the translation process for the type, if it is not found yet. 
   * See {@link #makeAndDependentTypeProcessing()} 
   * <br>
   * <br>
   * <b>searching a method identifier with parameters</b>
   * <br>
   * Methods can't defined in a local context, only at class level. To know and search methods
   * in the actual context, the {@link ClassData} of the appropriate class of the local context 
   * can be used. They are accessable in the methods of {@link SecondPass} via {@link GenerateClass#classData}
   * See {@link #methodCall_WithParameterSensitivity()}
   * <br>
   * <br>
   * <b>{@link ClassData}</b>
   * <br>
   * That are the type informations. The {@link ClassData} are referenced in 
   * {@link LocalIdents.typeIdents}. Classes have their own {@link ClassData#classLevelIdents}.
   * <br>
   * <br>
   * <b>{@link FieldData}</b>
   * <br>
   * That are the variable and field informations. The {@link FieldData} are referenced in 
   * {@link LocalIdents.fieldIdents}. Fields based on {@link FieldData#typeClazz}, but have
   * some additional properties, also especially for the C-Code-representation. 
   * Also arrays with theire special properties are considered.
   * <br>
   * <br>
   * <b>{@link CCodeData}</b>
   * <br>
   * That are peaces of C-Code with its type-information {@link CCodeData#identInfo},
   * but also special properties. If an array element is addressed from a array-field,
   * it may be possible to store the access information in an extra {@link CCodeData#identInfo}.
   * But the solution is: The {@link FieldData} of the whole array is referenced, 
   * and the access properties to the array element are stored in extra data fields:
   * {@link CCodeData#dimensionArrayOrFixSize} and {@link CCodeData#modeAccess} of the element.
   */
  public void ListsOfTypesAndIdentifiers(){}
  

  /**Methods in Java are parameter sensitive. This feature is known also as <i>overloading of methods</i>. 
   * It means, that an method of class StringBuilder
   * <code> append(int value)</code>
   * is another method as
   * <code> append(String value)</code>.
   * It is clear for Java users. Okay. In C++ it is the same concept.
   * <br>
   * But in C, the methods should have an unambiguously name. 
   * Therefore, the same steps have to be done which were done by a javac-compiler:
   * <ul>
   * <li>Ascertaining the type of all parameters of method call.
   * <li>Searching the appropriate method variant in the environment class, where the method is member of.
   * <li>If no appropriate method is found, variation of the type of parameters to their 
   *     generalized types, it means int from short, long from int or float from int etc., 
   *     but also detection of super classes and interface types of references. Search again.
   * <li>Building the C-like method name in a special style, considering the parameter types and the environment class name.
   * </ul>
   * This steps are done in Java2C-translation time. The result is visible in the generated C-code. 
   * This steps are done only if ambiguously method with the same names are exist. 
   * If only unambiguously method names in one class are given, a simple rule is used 
   * to build an unambiguously method name common spanned: <code>methodname_classname</code>.
   * <br><br>
   * For some lang and util-classes the methods have their fix names in the 
   * [[CRuntimeJavalike]]-C-Library, that is also useable for C-level-programming. Thats why, and also 
   * to support a testing at C-level the names of methods considering parameter types should be 
   * shortly, readable and understandable 
   * <br><br>
   * The translation of  Java method name plus parameter types to that C- method names are given 
   * in a translation table. There is no common fix rule to built it. 
   * But for method names of the users Java code to translate to C a rule  is given. 
   * <br><br>
   * The following things are implemented in the Java2C-Translator:
   * <ul>
   * <li>The <b>type of arguments</b> in the method call is detect calling 
   *     {@link SecondPass.StatementBlock#gen_value(ZbnfParseResultItem, char)}. 
   *     The type is returned in its {@link CCodeData}-return data. 
   *     The type of a value is build from 
   *     <ul>
   *     <li>The type of a variable, returned from 
   *         {@link SecondPass.StatementBlock#gen_variable(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents, char, CCodeData)}
   *         {@link CCodeData}-return data.
   *     <li>The type of a constant value, returned from retType in
   *         {@link GenerateClass#genConstantValue(org.vishia.zbnf.ZbnfParseResultItem)}.
   *         The type of the constant value is detect while parsing the Java source code. The type of a numeric value
   *         is detect additional by testing the value range of the number. Especially the type of an string literal 
   *         in "string" is designated as {@link Java2C_Main.CRuntimeJavalikeClassData#clazz_s0}. 
   *         The <code>s0</code> means, C-like 0-terminated string. It is a special case of a String represented by
   *         { {@link Java2C_Main.CRuntimeJavalikeClassData#clazzStringJc}.
   *     <li>The type of a method is the return type. It is supplied in argument retType in
   *         {@link SecondPass.StatementBlock#gen_simpleMethodCall(org.vishia.zbnf.ZbnfParseResultItem, CCodeData, LocalIdents)}.
   *     <li>The type of a new Object is returned in retTypeValue from
   *         {@link SecondPass.StatementBlock#gen_newObject(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents)} 
   *         or {@link SecondPass.StatementBlock#gen_newArray(org.vishia.zbnf.ZbnfParseResultItem, ClassData[], LocalIdents)}.
   *     </ul>
   * <li>Both a <b>method call and a constructor is translated</b> using 
   *     {@link SecondPass.StatementBlock#gen_simpleMethodCall(org.vishia.zbnf.ZbnfParseResultItem, CCodeData, LocalIdents)}.
   *     While translating the actual parameters are read from the ZbnfParseResultItem. The values are generated
   *     calling {@link SecondPass.StatementBlock#gen_value(org.vishia.zbnf.ZbnfParseResultItem, char)}.
   *     This method returns the type of the parameters. The types are used to search the method.
   * <li>To <b>search the method</b> {@link ClassData#searchMethod(String sNameJava, java.util.List paramsType)} 
   *     is called with the {@link ClassData}-instance of the environment type. 
   *     That is the type of the instance in Java left from point of method call. This ClassData are referenced from
   *     the parameter <code>envInstanceInfo</code> of <code>gen_simpleMethodCall()</code>.
   *     If the environment instance is <code>this</code>, the <code>envInstanceInfo</code> came from 
   *     {@link ClassData#classTypeInfo} of the own ClassData-Instance. 
   *     <ul>
   *     <li>The method is searched first with their number of arguments. It is possible that they are 
   *       more as one ambiguous method with the same name and the same number of arguments. 
   *       If there is only one method, its okay and unambiguous. Otherwise the types of the arguments
   *       are tested now.
   *     <li>If the types are matched directly, its okay. Elsewhere the routine {@link ClassData#matchedTo(ClassData)}
   *         arbitrates whether the type is compatible. Than it is possible that a cast expression
   *         is necessary in C, without though an automatic casting in Java is done. 
   *         The cast expression is generated in 
   *         {@link SecondPass.StatementBlock#gen_simpleMethodCall(org.vishia.zbnf.ZbnfParseResultItem, CCodeData, LocalIdents)}.
   *     </ul> 
   * <li>The <b>method translating tables</b> from Java-name + parameter types to C-name are <b>created 
   *     for the CRuntimeJavalike-methods</b> in 
   *     {@link Java2C_Main.CRuntimeJavalikeClassData} - constructor. It is hand-made. 
   * <li>The method translating tables for translating classes will be created and filled 
   *     while executing 
   *     {@link FirstPass#runFirstPass(String, org.vishia.zbnf.ZbnfParseResultItem, String, ClassData, java.util.List, LocalIdents)}.
   *     <ul>
   *     <li>Before any other translation occurs, for all methods of the class,  {@link ClassData#testAndSetAmbiguousnessOfMethod(String)}
   *         is called with the java name of the method. If more as one method with the same name is found,
   *         the method name is ambiguous. Only than a complex method name for C should be built.
   *     <li>The methods are registered in first pass calling 
   *         {@link FirstPass#wrwrite_methodDeclaration(ZbnfParseResultItem parent, String sClassName, LocalIdents}.
   *         In this routine it is known whether or not the method is ambiguous, because all methods are tested before.
   *     <li>From there {@link FirstPass#gen_methodHeadAndRegisterMethod(org.vishia.zbnf.ZbnfParseResultItem, String, String, FieldData, boolean)},
   *         is called. In this routine the C-methodname is build either with argument-type-naming parts or not. 
   *         The argument-type-naming parts of the C-method-name are simple chars for standard types,
   *         see registration of standard types in {@link Java2C_Main.CRuntimeJavalikeClassData},
   *         calling the constructor of {@link ClassData#ClassData(String, String, String, String, String)}.
   *         A short name for user-Type-arguments is built with prominent chars of the type, 
   *         testing its unambiguously in global scope.
   *     <li>With that built C-Name and the argument types the method is registered in ClassData calling
   *         {@link ClassData#addMethod(String, String, FieldData, FieldData[])}.
   *     </ul>                 
   * </ul>        
   * 
   */
  public void methodCall_WithParameterSensitivity()
  { SecondPass a;
    
  }
  
  
  
  /**The concept of virtual methods is necessary for implementation of interfaces and 
   * for overridden methods in derived classes.
   * <br>
   * The current version 0.83 doesn't support the inheritance of classes and interfaces completely,
   * but virtual methods are implemented initially.
   * <br>
   * The class {@link ClassData} contains a field {@link ClassData#inheritanceInfo}. The Type
   * {@link ClassData.InheritanceInfo} contains the reference to the superclass's and interfaces InheritanceInfo. 
   * Any instance of InheritanceInfo for super-classes and interfaces
   * isn't associated to that classes, but it is an own instance in this tree. 
   * Rather the associated class is referenced in {@link ClassData.InheritanceInfo#classData},
   * it has its own InheritanceInfo-tree. The reason for that plurality of InheritanceInfo is: 
   * they contain the virtual method names. The virtual methods are differently 
   * for a class as superclass of another class (contains the derived virtual method) 
   * or for the class able to instantiate in another context. The field 
   * {@link ClassData.InheritanceInfo#methodTable} contains the names of all virtual methods. 
   * They are used to produce the <code>Mtbl...</code> definitions and constants in C (method table).   
   * <br>
   * The constructor {@link ClassData.InheritanceInfo#InheritanceInfo(ClassData, ClassData)}
   * uses the superclass (and all interfaces, TODO) to build this tree of Inheritance objects.
   * Initially all C-translated names of methods from the superclasses and interfaces are copied,
   * (String Member in {@linkClassData.InheritanceInfo#methodTable}. 
   * The index in this list is stored in {@link ClassData.Method#idxMtbl}. 
   * <br>
   * All methods of super-classes and interfaces are stored in the List {@link ClassData#methodsOverloadable},
   * if there are able to overload (not final in java). yet: only if {@link ClassData.Method#modeOverloadable}
   * is set while calling {@link ClassData#addMethod(String, String, int, FieldData, FieldData[])}. 
   * <br>
   * If a method of the current translated class is processed, it is searched in all super classes
   * and interfaces calling {@link ClassData#searchOverloadedMethod(String, FieldData[])}. 
   * If it is found there, it is an overridden method. Therefore a new Instance of {@link ClassData.Method}
   * is created, but the information about the {@link ClassData.Method#declaringClass} is taken 
   * from the found method. The name in the {@linkClassData.InheritanceInfo#methodTable} is replaced
   * with the actual method. 
   * <br>
   * In the generated C-code of overwritten methods the type of the instance pointer is named 
   * <code>ithis</code> instead <code>ythis.</code>, the type of this pointer is a reference to the
   * declaring class. The first instruction than is a cast to the instance type of the implementation.
   * But if <code>STRICT_CHECKJc</code> is defined, a type test is executed. All immediate simple not checked casts
   * may be source of mistakes in execution.
   * <br>
   * An override-able method tests at begin, if a overridden method is to execute. This is done 
   * using the reflection and method table infos. 
   * <br>
   */
  public void virtualMethods()
  { 
    
  }
  
  
  
  /**In Java a environment-type for a static method and a reference of a dynamic method are not differenced.
   * Both are written with a dot as separator:
   * <pre>
   * Arrays.binarySearch(..) //an example of static method of class Arrays
   * myInstance.method(...)  //a normal class method call.
   * </pre> 
   * The difference is detectable only searching the identifier in translator tables.
   * Therefore the identifier is tested in the method
   * in {@link SecondPass.StatementBlock#gen_reference(ZbnfParseResultItem, LocalIdents, char, IdentInfos[] retIdentInfo)}.
   * The getted association String from Zbnf parse result is tested calling {@link LocalIdents#getType(String)}
   * in the given local environment. It includes especially standard types, but also local types in the class definition.
   * If the association String is recognized as Type, the associated type is used in the next reference levels
   * respectively it is returned in parameter retIdent. Because that type is the base type info for searching
   * the referenced identifier, the method calls in C are translated with the name_Type-info.    
   */  
  public void referencesOrEnvironmentTypes()
  {
  }
  
  
  /**The translation start with any file of the input file set given with cmd line arguments
   * <code>-i:FILE</code> or <code>-if:INPFILES</code>. That is in routine 
   * <ul>
   * <li>{@link Java2C_Main#execute()}, from there 
   * <li>{@link Java2C_Main#runFirstPass(java.io.File, String, String, boolean)} is called with the input files.
   *     This routines tests whether the destination files are existing and whether 
   *     the source file is newer or not. Only if a new translation process is necessary,
   *     <ul>
   *     <li>{@link FirstPass#runFirstPass(String, String, String, org.vishia.zbnf.ZbnfParseResultItem, String, ClassData, java.util.List, LocalIdents)} 
   *         is called there inside for all classes contained in the file.
   *     </ul>
         If a <code>file.java</code> is translated in the past, the <code>file.c</code> and <code>file.h</code>
   *     are existing, a file <code>file.stc</code> was also built in the translation process. 
   *     Than the {@link ClassData} for this type are get parsing the <code>type.stc</code>-File.
   *     It is some times faster as parsing and evaluating the <code>file.java</code>-File.
   *     <ul>
   *     <li>{@link Java2C_Main#readStructToClassData(String)} processes it.
   *     </ul>        
   * </ul>
   * If any unknown yet type is used, it may be either a not yet translated depended class type, 
   * represented in a <code>type.java</code>-file with the given type-name, of the input file set,
   * or it may be an external type which is provided in the C-handwritten environment, with a only
   * emulating <code>type.java</code>-File in the Java sources.
   * <br>
   * The method {@link GenerateClass#getType(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents)} 
   * detects, whether the type is known. This routine calls the parsing and first pass of translation
   * if necessary. It is a deep recursively process, because the called first pass may need
   * another file. But last not least all files are translated and the {@link ClassData} are existing.
   * <br>
   * The access to another types respectively content of translated files is done with access
   * to the appropriate {@link ClassData}. All informations about using the type/file 
   * are contained there.
   * <br>
   * In generally, instead parsing a special <code>type.stc</code>-File, it may be possible
   * to get the {@link ClassData}-informations from the translated <code>type.class</code> Java file.
   * But some informations contained in comments (<code>@Java2C=</code>...annotations) aren't
   * contained in the class-file. Therefore this possibility is not trailed.
   */
  public void makeAndDependentTypeProcessing(){}
  
  
  
  /**The import statements are not useable for include generation 
   * because files from the same package are not imported in Java. But an include is necessary.
   * The import is forced if a type is used there is located in another class. 
   * The using of the type may be occured in the first or in the second pass of translation. 
   * The first pass produces the .h-file, the second pass the .c-File. So a using of a type leads to include
   * in the .h- or .c-File. The methods {@link GenerateFile#addIncludeC(String sFileName, String comment)} 
   * and {@link GenerateFile#addIncludeH(String sFileName, String comment)} represents both possibilities. This methods may be called 
   * more as one time for the same time. Therefore the TreeMap {@link GenerateFile#includes} saves the types for including
   * while running the first or second pass for this file.
   * <br><br>
   * The routine {@link GenerateFile#addIncludeH(String sFileName, String comment)} is only called inside the first pass in the routine
   * {@link GenerateClass#gen_variableDefinition(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents, java.util.List<ClassData.VariablesToInit>, char)}
   * corresponding to used variable types or method argument or method return types.
   * <br><br>
   * The routine {@link GenerateFile#addIncludeC(String,String)} is called in the second pass 
   * while detecting reference types in {@link SecondPass.StatementBlock#gen_reference(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents, CCodeData, char)},
   * while using a type in a new construct in {@link SecondPass.StatementBlock#gen_newObject(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents)}
   * or while definition of variables in bodies of methods inside 
   * {@link GenerateClass#gen_variableDefinition(org.vishia.zbnf.ZbnfParseResultItem, LocalIdents, java.util.List, char)}.
   * <br><br>
   * For C-compiling it is important that .h-Files don't contain includes which are not needed in the .h-File, only in the .c-File. 
   * Otherwise circular includes are generated, that isn't able to compile.    
   */
  public void includeGeneration(){}
  
  
}
