package org.vishia.java2C;

import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map.Entry;


/**This class contains all identifiers visible at class level or visible at a local position of code.
 * It means, visible at class level by generating the class members,
 * visible at level of a block statement including stack variable etc.
 * @author JcHartmut
 * {@link FieldData#typeClazz}
 *
 */
public class LocalIdents
{
  
  /**List of known fields of this context. */
  private final TreeMap<String, FieldData> fieldIdents;

  /**List of known types of this context. */
  //private final TreeMap<String, FieldData> typeIdents;
  private final TreeMap<String, ClassData> typeIdents;

  /**The parent idents. */
  private final LocalIdents parent;
  
  /**The ClassData for which is this member of, or null if it isn't classLevelIdents of {@link ClassData}. } */
  private final ClassData parentClassData;
  
  /**constructs a new empty instance. 
  public LocalIdents()
  { this(Java2C_Main.singleton.userTypes);
  }
 */ 
  
  
  /**constructs a new instance with copy all parent identifier as base. 
   * This method is called especially if the identifier infos
   * of a local scope is built. They contains all identifier infos of the parent scope,
   * but the parent scope should be not touched (changed).
   */
  public LocalIdents(LocalIdents parent, ClassData parentClassData)
  { fieldIdents = new TreeMap<String, FieldData>();
    if(parent != null) { fieldIdents.putAll(parent.fieldIdents); }
    typeIdents = new TreeMap<String, ClassData>();
    this.parent = parent;
    this.parentClassData = parentClassData;
    if(parentClassData != null && parentClassData.sClassNameJava.equals("Entry"))
      stop();
    //typeIdents.putAll(parent.typeIdents);
  }

  /**searches the infos to the given identifier.
   * 
   * @param sIdent The identifier.
   * @return null if not found. 
   */
  public FieldData get(String sIdent)
  { return fieldIdents.get(sIdent);
  }
  
  /**searches the infos to the given type identifier.
   * 
   * @param sIdent The identifier.
   * @return null if not found. 
   */
  //public FieldData getType(String sIdent)
  public ClassData getType(String sIdent)
  { if(sIdent.equals("iRequireMainController"))
      stop();
    final ClassData infos;
    if(this.parentClassData !=null && this.parentClassData.sClassNameJava.equals(sIdent))
    { infos = parentClassData;
    }
    else
    { infos = typeIdents.get(sIdent);
    }
    if(infos == null && parent != null)
    { return parent.getType(sIdent);  //try in parent, recursively. It tests also the outer (TODO)
    }
    else return infos;  //may be null if not found.
  }
  
  /**searches the infos to the given type identifier.
   * 
   * @param sIdent The identifier.
   * @return null if not found. 
   */
  //public FieldData getType(String sIdent)
  public ClassData getTypeContingentlyRunFirstPass(String sIdent, RunRequiredFirstPass_ifc runFirstPass)
  { ClassData infos;
    infos = getType(sIdent);
    if(infos ==null)
    { return runFirstPass.runRequestedFirstPass(sIdent);
    }
    return infos;  //may be null if not found.
  }
  
  /**puts a new class element in the container.
   * 
   * @param sIdent The textual representation of the identifier in java code context.
   * @param sType The associated type string if it is a field or method. null if it is a type.
   * @param typeClazz The associated type class
   * @param sModifier Kind of the identifier. See table in the description of the class {@link ClassData}.
   * @param clazz The associated class data
   */
  public void putClassElement
  ( String sIdent, String sType, ClassData typeClazz, char staticMode
  , char modeAccess
  , String sModifier
  , int dimensionArray
  , String[] fixArraySizes
  , char modeArrayElement
  , ClassData clazz
  )
  { FieldData value = new FieldData(sIdent, typeClazz, staticMode, modeAccess, dimensionArray, fixArraySizes, modeArrayElement, clazz);
    value.nClassLevel = 1;
    value.nOuterLevel = 1;
    fieldIdents.put(sIdent, value);
  }
  
  public void putClassElement(String sIdent, FieldData identInfo)
  { identInfo.nClassLevel = 1;
    identInfo.nOuterLevel = 1;
    fieldIdents.put(sIdent, identInfo);
  }
  
  
  
  /**puts a new inner class as a Type of the class in the container.
   * 
   * @param sTypeIdent The textual representation of the identifier in java code context.
   * @param sTypeNameC The type name in C, with outerclass__sTypeIdent.
   * @param typeClazz The associated type class
   * @param sModifier Kind of the identifier. See table in the description of the class {@link ClassData}.
   * @param clazz The associated class data
   * @deprecated
   */
  public void putClassType
  ( String sTypeIdent
  , String sTypeNameC
  , ClassData typeClazz
  , String sModifier
  , ClassData clazz
  )
  { putClassType(sTypeIdent, typeClazz);
  }
  
  
  public void putClassType
  ( String sTypeIdent
  , ClassData typeClazz
  )
  { //FieldData value = new FieldData(sTypeNameC, typeClazz, '.', sModifier, null, clazz);
    //value.nClassLevel = 1;
    //value.nOuterLevel = 1;
    //typeIdents.put(sTypeIdent, value);
    if(sTypeIdent.equals("SetValueGenerator"))
      stop();
    typeIdents.put(sTypeIdent, typeClazz);
  }
  
  public void putClassType
  ( ClassData typeClazz
  )
  { typeIdents.put(typeClazz.getClassNameJava(), typeClazz);
  }
  
  /**puts a new stack-local element in the container.
   * @param sIdent
   * @param identInfo
   */
  public void putLocalElement(String sIdent, FieldData identInfo)
  { identInfo.nClassLevel = 0;
    identInfo.nOuterLevel = 0;
    fieldIdents.put(sIdent, identInfo);
  }
  
  private final static String newLineIndent = "\n                                            ";
  
  String indent(int recursion){ return newLineIndent.substring(0, 2*recursion +1); } 
  
  
  public String writeStruct(int recursion)
  { String out = "";
    if(fieldIdents != null)
    { out = "fieldIdents";
      out += indent(recursion) + "{ ";
      Set<Entry<String,FieldData>> listEntries = fieldIdents.entrySet();
      for(Entry<String,FieldData> field: listEntries)
      { //String sName = field.getKey();
        FieldData identInfos = field.getValue();
        out +=  indent(recursion+1) + "field" + identInfos.writeStruct();
      }
      out += "}\n";
    }  
    if(typeIdents != null)
    { out += "typeIdents" + indent(recursion) + "{  ";
      Set<Entry<String,ClassData>> listEntries = typeIdents.entrySet();
      for(Entry<String,ClassData> field: listEntries)
      { //String sName = field.getKey();
        ClassData identInfos = field.getValue();
        out += indent(recursion+1) + "type;" + identInfos.getClassIdentName();
      }
      out += "}\n";
    }  
    return out;
  }
  
  
  
  /**Returns an iterable list with all typeIdents.
   */
  public Set<Map.Entry<String,ClassData>> getTypeSet()
  { return typeIdents.entrySet();
  }
  
  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  void stop(){}
}
